/* -------------------------------------------- */
/* @file: mtd.h		 			*/
/* $Id: mtd.h,v 1.1.1.1 2004/04/25 17:22:13 jompo Exp $ 					*/
/* -------------------------------------------- */

#ifndef _H_mtd

#define _H_mtd

#include "DTypes.h"

#define ANGLES		/* if angle to be processed	*/

#ifdef ANGLES
typedef short	INDX;		/* type of angles (short)	 */ 
#define EOINDX	((INDX)-1)	/* end of angle sequence	 */
#define SEQUENCEPTR	AngleSequencePtr
#include "libmtd_angle_io.h"
#else
typedef char	INDX;		/* type of symbol (char) */
#define EOINDX	'\000'	/* end of symbol sequence	 */
#define SEQUENCEPTR	FastaSequencePtr
#include "fasta_io.h"
#endif



/* -------------------------------------------- */
/* constants					*/
/* -------------------------------------------- */

#ifndef BUFSIZ
#define BUFSIZ 1024		/* usually in stdio.h	*/
#endif

#define EOD -1			/* end of data mark	*/

#define MAX_ORDER 256		/* max. Markov order	*/
#ifndef ANGLES
#define MAX_STATE 32		/* max. # states	*/
#else
#define MAX_STATE 256		/* max. # states	*/
#endif
/* #define MAX_MODEL 32		*//* max # models		*/ 
#define MAX_MODEL 2		/* max # models		*/

#define MIN_DELTA 1e-3		/* small delta value	*/
#define MIN_DLL   0.5		/* min change in LL	*/
#define MAX_ITER_FACTOR  10	/* max # iterations	*/

#define START_LAG 1		/* start of lag option	*/

#define EPSILON 1e-30		/* small REAL value	*/

/* -------------------------------------------- */
/* datatypes					*/
/* -------------------------------------------- */

#define MYDOUBLE
#ifdef MYDOUBLE
typedef double REAL;	 /* float representation */
#define ARG_REAL "%lf"
#else
typedef float REAL;	/* float representation */
#define ARG_REAL "%f"
#endif

typedef struct {		/* -- MTD Model ------- */
  int  order;			/* Markov order		*/	
  int  nstates;			/* # states		*/
  INDX *states;			/* labels of states (MAX_STATE + 1)	*/
  REAL *phi;			/* lag weights		*/
  REAL **q; 			/* transition matrix	*/
} MTDModel;

typedef struct {		/* -- MTD Model ------- */
  int  order;			/* Markov order		*/	
  int  nstates;			/* # states		*/
  int  muphi;			/* index of mu phi	*/
  int  muq;			/* index of mu q[0]	*/
  int  sizetot;			/* size tot of x	*/
  INDX *states;			/* labels of states (MAX_STATE + 1)	*/
  REAL *x;			/* lag weights+transition matrix	*/
  	/* in the lmodel                                                */
        /* x[1..lm->order-1] are the phi                                        */
        /* x[lm->order..muphi-1] are the q[i][j]                                */
        /* x[muphi] is the mu_phi lagrangian multiplier                 */
        /* x[muq..muq+nstates-1] are the nstates mu_q lagrangian multipliers    */
} MTDLModel;


typedef struct {		 /* -- Derivatives ---- */
  int  order;			/* Markov order		*/	
  int  nstates;			/* # states		*/
  REAL *dphi;			 /* lag weights		*/
  REAL **dq;			/* transition matrix	*/
} MTDJacobi;


typedef struct {		 /* -- Derivatives ---- */
  int  order;			/* Markov order		*/	
  int  nstates;			/* # states		*/
  int  muphi;			/* index of d mu phi	*/
  int  muq;			/* index of d mu q[i.]	*/
  int  sizetot;			/* size tot of d	*/
  				/* i.e. order + nstates*nstates +	*/
  				/* 1 + nstates		*/
  REAL *d;			 /* lag weights	+	*/
  				/* transition matrix +	*/
  				/* mu phi + mu q[i.]	*/
  				/* derivatives		*/
} MTDLJacobi;

typedef struct {		/* -- Data Set --------	*/
  int nseq;			/* # sequences		*/
  SEQUENCEPTR *seq;	/* vector of SeqPtr	*/
} DataSet;

typedef struct {		/* -- Model Set ------	*/
  int nmodel;			/* # models		*/
  REAL *lprior;			/* log(Prior)		*/
  MTDModel **model;		/* vector of pointer to models	*/
} ModelSet;

typedef struct {		/* -- Model Set ------	*/
  int nmodel;			/* # models		*/
  REAL *lprior;			/* log(Prior)		*/
  MTDLModel **model;		/* vector of pointer to models	*/
} LModelSet;

/* -------------------------------------------- */
/* macros					*/
/* -------------------------------------------- */

#define MemError	Error("not enough memory", 10)
#define CheckPtr(p) 	if (! (p)) MemError

#define LOG(x)		(REAL) log((double)(x))
#define EXP(x)		(REAL) exp((double)(x))


#endif	/* ifndef _H_mtd	*/

