/*
 * If you use this program standalone, the call should be on the form:
 * QBestFit <-bfxyz, -bfpdb> < crdfile.xyz > fitcrd.xyz

 * Input:
 *   first line: N, the number of coordinates
 *   following 2 x N lines: N coordinates (x y z, e.g. "0.134 2.345 12.3245", i.e. no commas)
 *   the first N xyz lines will be the template, the last N xyz lines will 
 *   undergo the best fit transformation.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "libqbestfit.h"


#define CALLOCY(var, typ, dim)  {var=(typ*)malloc((size_t)(dim)*sizeof(typ));\
                              if((var)==NULL){\
                                  fprintf(stderr,"pb malloc avec %s (%d %s)\n",\
                                  #var,__LINE__,__FILE__);exit(99);}}

#define SQUARED_DISTANCE(K,R) ((K).x - (R).x) * ((K).x - (R).x) + ((K).y - (R).y) * ((K).y - (R).y) + ((K).z - (R).z) * ((K).z - (R).z)


int main(int argc, char *argv[])
{
  COORDS *c1, *c2, p2;
  DtMatrix4x4 M = {{1.0,0.0,0.0,0.0},{0.0,1.0,0.0,0.0},{0.0,0.0,1.0,0.0},{0.0,0.0,0.0,1.0}};
  double squared_rmsd;
  int i,j;
  int n;
  int oCrd = 0;

  for(i=0; i < argc; i++) {
    if (!strcmp(argv[i],"-bfxyz"))
      oCrd = 1;
    if (!strcmp(argv[i],"-bfpdb"))
      oCrd = 2;
  }
  fscanf(stdin,"%d",&n);
  CALLOCY(c1, COORDS, n);
  CALLOCY(c2, COORDS, n);

  for (i=0; i<n; i++) {
    j = i*3 + 1;
    fscanf(stdin,"%f%f%f",&c1[i].x,&c1[i].y,&c1[i].z);
  }

  for (i=0; i<n; i++) {
    j = i*3 + 1 + n*3;
    fscanf(stdin,"%f%f%f",&c2[i].x,&c2[i].y,&c2[i].z);
  }

  squared_rmsd = 0.;
  for (i=0; i<n; i++) {
    squared_rmsd += SQUARED_DISTANCE(c1[i],c2[i]);
  }
  if (n) {
    squared_rmsd /= (double) n;
  }
  fprintf(stdout,"ori_rmsd %lf\n",sqrt(squared_rmsd));
  fprintf(stdout,"rms after rotation identity %lf\n", sqrt(compute_RMS(c1, c2, n, M)));

  squared_rmsd = zuker_superpose(c1, c2, n, M);

  fprintf(stdout,"bestfit_rmsd %lf (2 superimposed on 1)\n",sqrt(squared_rmsd));
  fprintf(stdout,"rms after rotation %lf\n", sqrt(compute_RMS(c1, c2, n, M)));

/* si la transformation s'ecrit y = R(x+t1)+t2					*/
/* avec R rotation, t1 et t2 translation					*/
/* la matrice 3*3 superieure gauche contient la rotation R			*/
/* attention la matrice est a l'envers	(fortran...???) */
/* le vecteur inferieur M[3] contient:						*/
/* - le produit scalaire de la premiere colonne de R avec t1 + t2.x		*/
/* - le produit scalaire de la deuxieme colonne de R avec t1 + t2.y		*/
/* - le produit scalaire de la troisieme colonne de R avec t1 + t2.z		*/
/* - 1.0 (facteur d'echelle)							*/
/* cad que M contient:								*/
/*     R[0][0]      R[0][1]      R[0][2]	0.0			*/
/*     R[1][0]      R[1][1]      R[1][2]	0.0			*/
/*     R[2][0]      R[2][1]      R[2][2]	0.0			*/
/*     R[0][0]*t1.x+R[1][0]*t1.y+R[2][0]*t1.z+t2.x  R[0][1]*t1.x+R[1][1]*t1.y+R[2][1]*t1.z+t2.y  R[0][2]*t1.x+R[1][2]*t1.y+R[2][2]*t1.z+t2.z  1.0 */
  fprintf(stdout,"%lf %lf %lf %lf\n",M[0][0], M[0][1], M[0][2], M[0][3]);
  fprintf(stdout,"%lf %lf %lf %lf\n",M[1][0], M[1][1], M[1][2], M[1][3]);
  fprintf(stdout,"%lf %lf %lf %lf\n",M[2][0], M[2][1], M[2][2], M[2][3]);
  fprintf(stdout,"%lf %lf %lf %lf\n",M[3][0], M[3][1], M[3][2], M[3][3]);

  if (oCrd == 1) {
    for (i=0; i<n; i++) {
      p2 = crdRotate(c2[i],M);
      fprintf(stdout,"%8.3lf%8.3lf%8.3lf\n",p2.x,p2.y,p2.z);
    }
  } else if (oCrd == 2) {
    for (i=0; i<n; i++) {
      p2 = crdRotate(c2[i],M);
      fprintf(stdout,"ATOM  %5d  %.3s%c%.3s %c%4d%c   %8.3lf%8.3lf%8.3lf\n",
	      i, "CA", ' ', "ALA", ' ',i,' ',p2.x,p2.y,p2.z);
    }
  }

  return 0;
}
