/* doz.c: 'Do Z' test/demo program for UNIX-like compression */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include <string.h>
#include <assert.h>
#include "zlib.h"

#include "libunz.h"
#include "lzpipe.h"

static FILE *inp;
struct stat s;

/* ------------------------------------------------------------	*/
/* uncompress Z part - see below for zlib (gz part)	        */
/* ------------------------------------------------------------	*/

static int getbyte(void)
{
   int b;
   b = getc(inp);
   if (ferror(inp)) {
       perror("Read error"); exit(-1);
   }
   return b;
}

/* uncompress FILE fin in FILE out, return the number	*/
/* of byte in out, or -1 if error			*/
int unZ(FILE *linp, FILE *out)
{
   int i = 0, nbytes = 0;
   char buf[1024];

   if (fstat(fileno(linp), &s) != 0) {
      perror("unZ function: no file");
      return -1;
   }
   inp = linp;

   if ((i = lzwopen(getbyte)) != 0) {
       if (i == -1) {
               (void)fprintf(stderr, "File is not in compressed format\n");
       } else {
               (void)fprintf(stderr, "Not enough memory");
               if (i >= 12) (void)fprintf(stderr," to process %d bits",i);
               (void)fprintf(stderr, "\n");
       }
       return -1;
   }
   do {
            i = lzwread(buf, sizeof(buf));
#ifdef DEBUG
	    {
	      char s[1025];
              strncpy(s,buf,1024);
	      s[1024] = '\0';
              printf(s);
	    }
#endif
            if (i > 0 && write(fileno(out), buf, i) != i) {
               (void)fprintf(stderr, "Write error\n");
               return -1;
            }
	    nbytes += i;
   } while (i == sizeof(buf));

	/* rewind the file for caller	*/
   rewind(out);

   return nbytes;
}

/* ------------------------------------------------------------------ */
/* zlib part                                                          */
/* ------------------------------------------------------------------ */

#define CHUNK 131072

#if 0
int gunz(FILE *source, FILE *dest)
{
/*
The local variables have the same functionality as they do for def(). The only difference is that there is no flush variable, since inflate() can tell from the zlib stream itself when the stream is complete.
*/

    int ret;
    unsigned have;
    z_stream strm;
    unsigned char in[CHUNK];
    unsigned char out[CHUNK];
/*
The initialization of the state is the same, except that there is no compression level, of course, and two more elements of the structure are initialized. avail_in and next_in must be initialized before calling inflateInit(). This is because the application has the option to provide the start of the zlib stream in order for inflateInit() to have access to information about the compression method to aid in memory allocation. In the current implementation of zlib (up through versions 1.2.x), the method-dependent memory allocations are deferred to the first call of inflate() anyway. However those fields must be initialized since later versions of zlib that provide more compression methods may take advantage of this interface. In any case, no decompression is performed by inflateInit(), so the avail_out and next_out fields do not need to be initialized before calling.

Here avail_in is set to zero and next_in is set to Z_NULL to indicate that no input data is being provided.
*/

    /* allocate inflate state */
    strm.zalloc = Z_NULL;
    strm.zfree = Z_NULL;
    strm.opaque = Z_NULL;
    strm.avail_in = 0;
    strm.next_in = Z_NULL;
    ret = inflateInit(&strm);
    if (ret != Z_OK)
        return ret;
/*
The outer do-loop decompresses input until inflate() indicates that it has reached the end of the compressed data and has produced all of the uncompressed output. This is in contrast to def() which processes all of the input file. If end-of-file is reached before the compressed data self-terminates, then the compressed data is incomplete and an error is returned.
*/

    /* decompress until deflate stream ends or end of file */
    do {
/*
We read input data and set the strm structure accordingly. If we've reached the end of the input file, then we leave the outer loop and report an error, since the compressed data is incomplete. Note that we may read more data than is eventually consumed by inflate(), if the input file continues past the zlib stream. For applications where zlib streams are embedded in other data, this routine would need to be modified to return the unused data, or at least indicate how much of the input data was not used, so the application would know where to pick up after the zlib stream.
*/

        strm.avail_in = fread(in, 1, CHUNK, source);
        if (ferror(source)) {
            (void)inflateEnd(&strm);
            return Z_ERRNO;
        }
        if (strm.avail_in == 0)
            break;
        strm.next_in = in;
/*
The inner do-loop has the same function it did in def(), which is to keep calling inflate() until has generated all of the output it can with the provided input.
*/

        /* run inflate() on input until output buffer not full */
        do {
/*
Just like in def(), the same output space is provided for each call of inflate().
*/

            strm.avail_out = CHUNK;
            strm.next_out = out;
/*
Now we run the decompression engine itself. There is no need to adjust the flush parameter, since the zlib format is self-terminating. The main difference here is that there are return values that we need to pay attention to. Z_DATA_ERROR indicates that inflate() detected an error in the zlib compressed data format, which means that either the data is not a zlib stream to begin with, or that the data was corrupted somewhere along the way since it was compressed. The other error to be processed is Z_MEM_ERROR, which can occur since memory allocation is deferred until inflate() needs it, unlike deflate(), whose memory is allocated at the start by deflateInit().

Advanced applications may use deflateSetDictionary() to prime deflate() with a set of likely data to improve the first 32K or so of compression. This is noted in the zlib header, so inflate() requests that that dictionary be provided before it can start to decompress. Without the dictionary, correct decompression is not possible. For this routine, we have no idea what the dictionary is, so the Z_NEED_DICT indication is converted to a Z_DATA_ERROR.

inflate() can also return Z_STREAM_ERROR, which should not be possible here, but could be checked for as noted above for def(). Z_BUF_ERROR does not need to be checked for here, for the same reasons noted for def(). Z_STREAM_END will be checked for later.

*/
            ret = inflate(&strm, Z_NO_FLUSH);
            assert(ret != Z_STREAM_ERROR);  /* state not clobbered */
            switch (ret) {
            case Z_NEED_DICT:
                ret = Z_DATA_ERROR;     /* and fall through */
            case Z_DATA_ERROR:
            case Z_MEM_ERROR:
                (void)inflateEnd(&strm);
                return ret;
            }
/*
The output of inflate() is handled identically to that of deflate().
*/

            have = CHUNK - strm.avail_out;
            if (fwrite(out, 1, have, dest) != have || ferror(dest)) {
                (void)inflateEnd(&strm);
                return Z_ERRNO;
            }
/*
The inner do-loop ends when inflate() has no more output as indicated by not filling the output buffer, just as for deflate(). In this case, we cannot assert that strm.avail_in will be zero, since the deflate stream may end before the file does.
*/

        } while (strm.avail_out == 0);
/*
The outer do-loop ends when inflate() reports that it has reached the end of the input zlib stream, has completed the decompression and integrity check, and has provided all of the output. This is indicated by the inflate() return value Z_STREAM_END. The inner loop is guaranteed to leave ret equal to Z_STREAM_END if the last chunk of the input file read contained the end of the zlib stream. So if the return value is not Z_STREAM_END, the loop continues to read more input.
*/

        /* done when inflate() says it's done */
    } while (ret != Z_STREAM_END);
/*
At this point, decompression successfully completed, or we broke out of the loop due to no more data being available from the input file. If the last inflate() return value is not Z_STREAM_END, then the zlib stream was incomplete and a data error is returned. Otherwise, we return with a happy return value. Of course, inflateEnd() is called first to avoid a memory leak.
*/

    /* clean up and return */
    (void)inflateEnd(&strm);
    return ret == Z_STREAM_END ? Z_OK : Z_DATA_ERROR;
}
/*
That ends the routines that directly use zlib. The following routines make this a command-line program by running data through the above routines from stdin to stdout, and handling any errors reported by def() or inf().

zerr() is used to interpret the possible error codes from def() and inf(), as detailed in their comments above, and print out an error message. Note that these are only a subset of the possible return values from deflate() and inflate().
*/

/* report a zlib or i/o error */
char *zerr(int ret)
{
    switch (ret) {
    case Z_ERRNO:
        return "error reading file";
        break;
    case Z_STREAM_ERROR:
        return "invalid compression level";
        break;
    case Z_DATA_ERROR:
        return "invalid or incomplete deflate data";
        break;
    case Z_MEM_ERROR:
        return "out of memory";
        break;
    case Z_VERSION_ERROR:
        return "zlib version mismatch!";
        break;
    case Z_OK:
        return "succesful completion";
        break;
    }
    return "unknown error";
}
#endif

/* ===========================================================================
 * Uncompress input to output then close both files.
 */

/* ===========================================================================
 * Display error message and exit
 */
void mygzerror(msg)
    const char *msg;
{
    fprintf(stderr, "%s\n", msg);
    exit(1);
}

int gz_uncompress(gzFile in, FILE *out)
{
    char buf[CHUNK];
    int len;
    int err;

    for (;;) {
        len = gzread(in, buf, sizeof(buf));
        if (len < 0) mygzerror (gzerror(in, &err));
        if (len == 0) break;

        if ((int)fwrite(buf, 1, (unsigned)len, out) != len) {
            mygzerror("failed fwrite");
        }
    }
    /* if (fclose(out)) mygzerror("failed fclose"); */
     rewind(out);

    if (gzclose(in) != Z_OK) mygzerror("failed gzclose");

    return 0;
}
