/* ---------------------------------------------------- */
/* @file: phipsi_main.c					*/
/*							*/
/* Calcul des liaisons a l'oeuvre dans les proteines	*/
/* lecture d'une entree de la pdb			*/
/* calcul des angles phi, psi, omega (option -P)	*/
/* calcul des angles alpha, tau et distance CA,CA+1	*/
/*				(option -A)		*/
/* calcul des angles chi1, chi2, chi3 (option -X)	*/
/* calcul de la distance entre CA-1 et CA+1 (option D)	*/
/*							*/
/* format de sortie :					*/
/*							*/
/* resname resnumber phi psi omega chi1 chi2 chi3 dist	*/
/* ....etc						*/
/*							*/
/* les angles sont donnes entre -180 et 180 degres	*/
/* si un angle ne peut etre determine pour une raison	*/
/* ou une autre dans un residu, il prend la valeur	*/
/* 500 (NO_ANGLE)					*/
/*							*/
/* codes d'erreur de sortie (shell) :			*/
/*							*/
/* exit 1: erreur d'ouverture de l'entree pdb		*/
/* exit 2: erreur de lecture ou de memoire		*/
/* exit 3: erreur de memoire				*/
/* ---------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>		/* for strcat	*/
#include <ctype.h>
#include <math.h>


/* ---------------------------------------------------- */
/* headers dans $HOME/include */
/* ---------------------------------------------------- */
#include "libphipsi.h"
#include "zlib.h"		/* for gz uncompress	*/
#include "libunz.h"		/* for uncompress	*/


/* #if (defined  Darwin) || (defined  Linux) || (defined  IRIX) */
#include <unistd.h>             /* for getopt   */
/* #else */  /* No getopt.h file, lost ?*/
/* #define NO_GETOPT */
/* #include <getopt.h>              if getopt non on the machine */ 
/* #endif */



/* -------------------- */
/* constants		*/
/* -------------------- */
#define ERR_OUTPUT_HEADER 1
#define ERR_NOT_ENOUGH_RES 2
#define GOOD		0
#define BAD		1
/* -------------------- */
/*internal prototypes	*/
/* -------------------- */
static void do_help(void);


/*--------------------------------------------------------------*/
/* main program							*/
/*--------------------------------------------------------------*/

int main(int argc, char **argv)
{
   FILE		*fil = NULL, *fol = NULL;
   char		header[256], base_path[PATH_MAX],
		path[PATH_MAX], specific_chain = NOCHAIN_CHAR,
		numstr[80];
   BinaryRes	*pbinres;
#ifdef NO_GETOPT
   char		*ptr;
#endif
   Residu	*cur_res, *first_res;
   float	x,y,z;
   int		nb_res_tot=0, comp_phipsi = 0, verbose = 0,
   		comp_dist13 = 0, comp_alphatau = 0,
   		comp_chi = 0, comp_xyz = 0, carg, errflag = 0,
   		nread, i, binary_output = 0, read_binary = 0,
                split_chain=0, nb_res_output = 0, numlen,
		minpdbnum = MINPDBNUM, maxpdbnum = MAXPDBNUM;
   int          nbChain=0, iChain=0;
   tyChain      *tChain=NULL;
   int          verifOut, nbResOut, lengthThreshold=-1;
   gzFile	mygzfile;

#ifndef Darwin
   extern	char *optarg;
   extern	int optind, opterr;
#endif

#ifdef THINK_C
   argc = ccommand(&argv);
#define NO_GETOPT
#endif


#ifdef NO_GETOPT

   for (i = 1; i < argc; i++) {		/* arguments			*/
      if (*argv[i] != '-') {
	if (i == argc-1) {
	   if ((fil = fopen(argv[i], "r")) == NULL) /* error exit with 1	*/
  	       MyError(stderr, 1, "%s::cannot open file %s", argv[0], argv[i]);
	}
	else
	   MyError(stderr, 5, "%s::%s is not a valable argument",argv[0], argv[i]);
      }
      else {				/* it is an argument		*/
         ptr = argv[i]+1;
         while(*ptr) { 
           switch(*ptr++) {
         
            case 'n':	split_chain   = 1;
            		break;
            
            case 'C':	specific_chain = *ptr;
	                /* Ajout Math pour pouvoir se servir de l'option -C qui p
			   permet aussi de n'avoir qu'un seul modle pdb aussi quand
			   pas de chaine*/	     
	                if( specific_chain == '_' ) 
			  specific_chain = ' ';
			ptr++;
            		break;
            
            case 'm':	if (sscanf(ptr,"%d",&minpdbnum) != 1) 
	                   MyError(stderr, 1, "cannot read option -m, exiting...");
			ptr++;
            		break;
            
            case 'M':	if (sscanf(ptr,"%d",&maxpdbnum) != 1) 
	                   MyError(stderr, 1, "cannot read option -M, exiting...");
			ptr++;
            		break;
            
	   case 't':               /* do not output if length <= threshold	*/
	                if (sscanf(optarg,"%d",&lengthThreshold) != 1)
			  MyError(stderr, 1, "cannot read option -t, exiting...");
			break;

            case 'p':	comp_phipsi   = 1;
            		break;
            
            case 'b':	binary_output   = 1;
            		break;
            
            case 'b':	read_binary   = 1;
            		break;
            
            case 'd':	comp_dist13   = 1;
            		break;
        
            case 'c':	comp_xyz   = 1;
            		break;
        
            case 'a':	comp_alphatau = 1;
            		break;

            case 'x':	comp_chi = 1;
            		break;

            case 'v':               /* verbose mode (on stderr)          */
                    verbose    = 1;
                    break;

            case 'h':	do_help();
		        exit(0);
            		break;

	    default:    MyError(stderr, 5, "%s::%s is not a valable argument",argv[0], argv[i]);
	    		break;
           }
	 }
      }
   }

   if (fil == NULL)
      fil = stdin;
#else  /* GETOPT */

    while ((carg = getopt(argc, argv, "paxdhbrcnvC:m:M:t:")) != -1) {

       switch (carg) {

                case 'n':               /* binary write and		*/
                    split_chain = 1;    /* output header for each	*/
					/* new chain			*/
                    break;              /* output only one NMR model	*/

                case 'C':               /* output only chain that user	*/
					/* asked for			*/
		    specific_chain = *optarg;
	                /* Ajout Math pour pouvoir se servir de l'option -C qui p
			   permet aussi de n'avoir qu'un seul modle pdb aussi quand
			   pas de chaine*/	     
	            if( specific_chain == '_' ) 
		      specific_chain = ' ';
                    break;              /* output only one NMR model	*/

                case 'm':               /* read minimum pdb number to output	*/
                    if (sscanf(optarg,"%d",&minpdbnum) != 1)
	                   MyError(stderr, 1, "cannot read option -m, exiting...");
                    break;

                case 'M':               /* read minimum pdb number to output	*/
                    if (sscanf(optarg,"%d",&maxpdbnum) != 1)
	                   MyError(stderr, 1, "cannot read option -M, exiting...");
                    break;

                case 't':               /* do not output if length <= threshold	*/
                    if (sscanf(optarg,"%d",&lengthThreshold) != 1)
	                   MyError(stderr, 1, "cannot read option -t, exiting...");
                    break;

                case 'p':               /* phipsi option        */
                    comp_phipsi = 1;
                    break;

                case 'b':               /* binary ouput option        */
		    binary_output = 1;
                    break;

                case 'r':               /* read binary option        */
		    read_binary = 1;
                    break;

                case 'a':               /* alpha-tau option     */
                    comp_alphatau = 1;
                    break;

                case 'd':               /* CA+1_CA-1 distance   */
                    comp_dist13 = 1;
                    break;

                case 'x':               /* chi option           */
                    comp_chi    = 1;
                    break;

                case 'c':               /* xyz of CA output option           */
                    comp_xyz    = 1;
                    break;

                case 'v':               /* verbose mode (on stderr)          */
                    verbose    = 1;
                    break;

                case 'h':               /* help                 */
		    do_help();
		    exit(0);
                    break;

                case '?':               /* bad option           */
                    errflag++;
             }

        }
        
	if (argc > optind + 1) {
           errflag++;
	}

        if (errflag) {
                fprintf(stderr, "usage: phipsi [-p] [-a] [-x] [-d] [-h] [-C <chain>] [pdb_file | < pdb_file]\n");
                fprintf(stderr, "       -h for Help\n");
                exit(1);
        }
#define NO_PATH	"no_path"
	if (optind == argc) {
	  fil = stdin;
	  strcpy(path,NO_PATH);
	}
	else if (argc == optind + 1) {
	   if ((fil = fopen(argv[optind], "r")) == NULL) /* error exit with 1	*/ 
  	       MyError(stderr, 1, "%s::%s: cannot open file", argv[0], argv[optind]);
           if (argv[optind][strlen(argv[optind])-2] == '.' && argv[optind][strlen(argv[optind])-1]=='Z') {
	     if(verbose)
	       fprintf(stderr,"Uncompressing file\n");
               if ((fol = tmpfile()) == NULL) {
		  MyError(stderr, 2, "Cannot open temporary file, exiting....\n");
	       }
	       if (unZ(fil, fol) < 10) {	/* pourquoi pas 10 caracteres...;-)	*/
		  fprintf(stderr, 
			  "Problem uncompressing file %s, exiting....\n", 
			  argv[optind]);
		  exit(3);
	       }
	       fclose(fil);
	       fil = fol;
	   }
           else if (argv[optind][strlen(argv[optind])-3] == '.' &&
                    argv[optind][strlen(argv[optind])-2]=='g' &&
                    argv[optind][strlen(argv[optind])-1]=='z' ) {
	     if(verbose)
	       fprintf(stderr,"Uncompressing gz file\n");
             if ((fol = tmpfile()) == NULL) {
                  fprintf(stderr, "Cannot open temporary file, exiting...\n");
                  exit(2);
	     }
             fclose(fil);
             if ((mygzfile = gzopen(argv[optind], "rb")) == NULL) {
                  fprintf(stderr, "Cannot open gz file %s, exiting....\n",argv[optind]);
                  exit(3);
             }
	     if (gz_uncompress(mygzfile, fol) != 0) {
		  fprintf(stderr, 
			  "Problem uncompressing gz file %s, exiting....\n", 
			  argv[optind]);
		  exit(3);
	     }
	     fil = fol;
	   }
           strcpy(path, argv[optind]);
	}
	else {
	     fprintf(stderr, "usage: phipsi [-p] [-a] [-x] [-d] [-h] [pdb_file | < pdb_file]\n");
             fprintf(stderr, "       -h for Help\n");
	     exit(1);
	}
#endif  /* GETOPT */

/*-----------------------------------*/
/* input reading                     */
/*-----------------------------------*/
   if (split_chain == 1 && (specific_chain != NOCHAIN_CHAR || minpdbnum != MINPDBNUM || maxpdbnum != MAXPDBNUM)) {
  	 MyError(stderr, 0, "%s:: option -n incompatible with -C, -m or -M, inactivating -C,-m or -M options", argv[0]);
	 minpdbnum = MINPDBNUM;
	 maxpdbnum = MAXPDBNUM;
	 specific_chain = NOCHAIN_CHAR;
	 
   }

   if (read_binary) {
      nread = 0;
      while((pbinres = readBinary(fil, header, path, &nb_res_tot, verbose))) {
         nread++;
         printf("HEADER %s\n",header);
         printf("PATH   %s\n",path);
         printf("NBRES   %d\n",nb_res_tot);
         for (i = 0; i < nb_res_tot; i++)
	   printf("%-4s %4d%c %7.1f %7.1f %7.1f %10.5f %10.5f %10.5f %c\n",
		pbinres[i].resName, 
		(int)(pbinres[i].numPDB.num[1]),
		pbinres[i].numPDB.inser[0], 
		pbinres[i].alpha, pbinres[i].tau, pbinres[i].dist,
		pbinres[i].x, pbinres[i].y, pbinres[i].z,
		pbinres[i].chain);
         free(pbinres);
      }
      fclose(fil);
      if (nread == 0 && pbinres == NULL)		/* error exit with 3	*/
   	MyError(stderr, 3, "error in reading binary file");
      exit(0); /* end, binary data have been read and printed in ascii*/
   }

   /* if(split_chain == 1 || specific_chain != NOCHAIN_CHAR){ */

   if (split_chain == 1) {
       first_res = readPdbResiduesChain(fil, &tChain, header, &nbChain);
       fclose(fil);
       if (tChain == NULL)		/* error exit with 2	*/
	  MyError(stderr, 2, "error in reading pdb file with readPdbResiduesChain (%s)", path); 
       if (first_res == NULL || first_res->next == NULL)		/* error exit with 2	*/
          MyError(stderr, 2, "error in reading pdb file (%s) with readPdbResidues, no amino acid residue found", path);
   }
   else {
					/* first res is a dummy res	*/
     first_res = readPdbResidues(fil, header, &nb_res_tot);
     fclose(fil);
     if (first_res == NULL || first_res->next == NULL)		/* error exit with 2	*/
          MyError(stderr, 2, "error in reading pdb file (%s) with readPdbResidues, no amino acid residue found", path);
   }

/*-----------------------------------*/
/* Searching for path                */
/*-----------------------------------*/
				/* output absolute path	*/
   if (path[0] != '/') {
     if (getcwd(base_path, PATH_MAX) == NULL) {
        MyError(stderr, 0, "base path cannot be found");
     }
     else {
        if (! strcmp(path,NO_PATH)) {
          	/* set the name to pdbXXXX.ent where XXXX	*/
		/* is the name found in HEADER	field		*/
           strncpy(path,header, 4);
           path[4]='\0';
           strcat(base_path, "/pdb");
           strcat(base_path, path);
           strcat(base_path, ".ent");
           strcpy(path, base_path);
        }
        else if (strlen(base_path) + strlen(path) < PATH_MAX+1) {
           strcat(base_path, "/");
           strcat(base_path, path);
           strcpy(path, base_path);
        }
	fprintf(stderr,"path->%s\nheader %s\n", path, header);
     }
   }


/*-----------------------------------*/
/* angles computation and output     */
/*-----------------------------------*/

   verifOut=BAD; /* Passe  TRUE des qu'on ecrit qq chose dans le fichier de sortie*/
   nbResOut=0;

			
		/* -- output loop 		*/


   /* if (split_chain ==1 || (binary_output && specific_chain == NOCHAIN_CHAR)) { */

   if (split_chain ==1) {

		/* split into one or		*/
		/* more chains  		*/
		/* each one with its header	*/
		/* and binary output		*/

      if(verbose)
         fprintf(stderr, "%d chain in %s\n", nbChain, header);

      if(verbose)
	fprintf(stderr, "Wanting all chains in binary format %d chain in %s\n", nbChain, header);

      for (iChain=0; iChain<nbChain; iChain++) {
			/* if all chain are to be output OR	*/
			/* if current chain is the wanted one	*/
         if (specific_chain == NOCHAIN_CHAR ||		/* split into all chains	*/
	     tChain[iChain].pChain->chain == specific_chain) { /* Si on veut cette chaine*/
		

	   /* Si on met une taille limite , on ne sort que les chaines de taille superieure a cette taille*/
	   /* Par contre, si on sort plusieurs chaines, on ne sort pas mais on ne sort pas la chaine quand mme*/
	   if (tChain[iChain].nbRes < lengthThreshold) {
	     if(verbose)
	         fprintf(stderr, "Chain %d %c, %d res, first res %s is too short \n", 
		   iChain+1, tChain[iChain].pChain->chain, 
		   tChain[iChain].nbRes, tChain[iChain].pChain->name);
	       continue; /* Si toutes les chaines sont trop petites, ne sort rien et verifOut reste  0*/
	                /* Sinon, verifOut contient la lg de la derniere chaine affiche */
	   }
	   else 
	     verifOut=GOOD;

	     if(verbose)
	         fprintf(stderr, "Chain %d %c, %d res, first res %s\n", 
		   iChain+1, tChain[iChain].pChain->chain, 
		   tChain[iChain].nbRes, tChain[iChain].pChain->name);
	     /* Pour que write binary marche, il	*/
	     /* faut que res->next du dernier	*/
	     /* residu de la chaine soit a NULL	*/
	     /* On va le faire temporairement	*/
	     if (iChain+1 < nbChain) {
	         /* Le precedent du 1er residu de	*/
	         /* la chaine suivante est le	*/
	         /* dernier residu de la chaine	*/
	         /* Son suivant doit etre mis a NULL*/
	         tChain[iChain+1].pChain->prev->next=NULL;
	     }
	     			/* write chain in header	*/
	     pad_with_blank(header,HEADERSIZE);
	     if (minpdbnum != MINPDBNUM || maxpdbnum != MAXPDBNUM) {
	        sprintf(numstr," CHAIN %c:%d-%d",tChain[iChain].pChain->chain,
				minpdbnum,maxpdbnum);
		numlen = (int)strlen(numstr);
	     }
	     else {
	        sprintf(numstr," CHAIN %c",tChain[iChain].pChain->chain);
		numlen = (int)strlen(numstr);
	     }
	     strncpy(header+HEADERSIZE-2-numlen,numstr,numlen);

	     nbResOut+=writeBinaryFile(stdout, header, path, tChain[iChain].nbRes,
	     		tChain[iChain].pChain, verbose, NOCHAIN_CHAR, minpdbnum, maxpdbnum);

	     if (verbose)
	         fprintf(stderr, "Nombre total de residus dans chaine %d %c : %d\n",
			 iChain, tChain[iChain].pChain->chain,
		 	tChain[iChain].nbRes );

	     if(iChain+1 < nbChain) {
	         /* Le precedent du 1er residu de la chaine	*/
	         /* suivante est le dernier residu de la	*/
	         /* chaine. Son suivant est remis a sa	*/
	         /* valeur initiale				*/
	         tChain[iChain+1].pChain->prev->next=tChain[iChain+1].pChain;
	     }
	 } 	/*fin    if (specific_chain == NOCHAIN_CHAR ||  tChain[iChain].pChain->chain == specific_chain) */

      }		/*fin for(iChain=0; iChain<nbChain; iChain++)*/

   }		/*if (split_chain ==1) */
   else {


                /********************************/
		/* other cases	       	        */
		/* only one chain wanted	*/
		/* ascii or binary output	*/
                /********************************/



     				/* Construction du header si	*/
				/* necessaire			*/
     pad_with_blank(header,HEADERSIZE);
	     			/* if specific chain asked for	*/
	     			/* write chain in header	*/
     if (specific_chain != NOCHAIN_CHAR) {
	   if (minpdbnum != MINPDBNUM || maxpdbnum != MAXPDBNUM) { 
	      sprintf(numstr," CHAIN %c:%d-%d",specific_chain,
	   			minpdbnum,maxpdbnum);
	      numlen = (int)strlen(numstr);
	   }
	   else {
	      sprintf(numstr," CHAIN %c",specific_chain);
	      numlen = (int)strlen(numstr);
	   }
	   strncpy(header+HEADERSIZE-2-numlen,numstr,numlen);
     }
     else if (minpdbnum != MINPDBNUM || maxpdbnum != MAXPDBNUM) {
	   sprintf(numstr," RES:%d-%d",minpdbnum,maxpdbnum);
	   numlen = (int)strlen(numstr);
	   strncpy(header+HEADERSIZE-2-numlen,numstr,numlen);
     }
     			/* calculer le nombre de residus a	*/
			/* "sortir" nb_res_output		*/
     nb_res_output = 0;
     for (cur_res = first_res->next ; cur_res ; cur_res = cur_res->next) {
	   if ((specific_chain == NOCHAIN_CHAR || cur_res->chain == specific_chain) &&
		(minpdbnum == MINPDBNUM || cur_res->pdbnum >= minpdbnum) &&
		(maxpdbnum == MAXPDBNUM || cur_res->pdbnum <= maxpdbnum))
	      nb_res_output++;
     }

     			/* a cette etape on sait (je crois)	*/
			/* le nombre de residus a sortir	*/
     			/* on peut donc mettre le test ici	*/

			/* Si taille > threshold */
   if(nb_res_output >= lengthThreshold) {
       verifOut=GOOD;
                                        /********************************/
					/* ascii output			*/
                                        /********************************/
       if (! binary_output) {
	 if(verbose)
	   fprintf(stderr, "ascii output	\n");

	 printf("HEADER %-45s\n", header);
	 printf("PATH   %s\n", path);
	 printf("NBRES  %d\n", nb_res_output);
	 for (cur_res = first_res->next ; cur_res ; cur_res = cur_res->next) {
	 
	/* debug fprintf(stderr,"RESIDU %-4s %4d %c\n",cur_res->name, (int)(cur_res->pdbnum), cur_res->chain); */
 
	   if ((specific_chain == NOCHAIN_CHAR || cur_res->chain == specific_chain) &&
	       (minpdbnum == MINPDBNUM || cur_res->pdbnum >= minpdbnum) &&
	       (maxpdbnum == MAXPDBNUM || cur_res->pdbnum <= maxpdbnum)) {
	     nbResOut++;
	     printf("%-4s %4d%c ", cur_res->name,
		    (int)(cur_res->pdbnum), cur_res->inser);

	     if (comp_phipsi) {
	       get_phi_psi(cur_res, verbose);
	       printf("%7.1f %7.1f %7.1f ",cur_res->phi , cur_res->psi,
		      cur_res->ome);
	     }
	
	     if (comp_alphatau) {
	       get_alpha_tau(cur_res, verbose);
	       if (get_distca_1_2(cur_res, verbose) == 1)
	          printf("%7.1f %7.1f %7.2f ", 500.0 ,500.0,
		      cur_res->dis12);
	       else
    	          printf("%7.1f %7.1f %7.2f ", cur_res->alpha, cur_res->tau,
		      cur_res->dis12);
	     }
	
	     if (comp_chi) {
	       get_chi(cur_res, verbose);
	       printf("%7.1f %7.1f %7.1f ", cur_res->x1, cur_res->x2,
		      cur_res->x3);
	     }
	
	     if (comp_dist13) {
	       get_distca_1_3(cur_res, verbose);
	       printf("%7.2f ", cur_res->dis13);
	     }

	     if (comp_xyz) {
	       get_CA_xyz(cur_res, &x, &y, &z, verbose);
	       printf("%10.5f %10.5f %10.5f ", x, y, z);
	     }
	   /* print the chain */
	     printf("%c\n", cur_res->chain);

	   }	/* end if (specific_chain == NOCHAIN_CHAR */
         }	/* end for	*/
   
         if (verbose)
	    fprintf(stderr, "Nombre total de residus nb_res_tot (sortie ascii) : %d\n", nb_res_output);

       }	/* end if (! binary)	*/
       else {
                                         /********************************/
                                 	 /* binary output	*/
                                        /********************************/
	 if(verbose)
	   fprintf(stderr, "binary output	\n");
    
	 nbResOut+=writeBinaryFile(stdout, header, path, nb_res_output, first_res->next, verbose,
			       specific_chain, minpdbnum, maxpdbnum);
     
	 if (verbose)
	   fprintf(stderr, "Nombre total de residus nb_res_tot (sortie binaire): %d\n", nbResOut);
 
       }
   } /* fin if (nb_res_tot >= lengthThreshold) */

   }	/* end if (split_chain ==1)	*/

   free_list_residus(first_res);

   if(verifOut == GOOD)  
     if(nbResOut > 0)
       return 0;
     else {
       if(verbose)
	 fprintf(stderr, "No residu outputed, only header threshold %d residu verifOut: %d nbResOut %d )\n", 
		 lengthThreshold, verifOut,nbResOut);
       return(ERR_OUTPUT_HEADER);
     }
   else {
     if(verbose)
       fprintf(stderr, "No output (maybe not enough residue (threshold %d residu verifOut: %d (GOOD = %d) nbResOut %d )\n", 
	       lengthThreshold, verifOut, GOOD, nbResOut);
     return(ERR_NOT_ENOUGH_RES);
   }
     
     
       
} /* FIN DU MAIN*/


/* -------------------------------------------------------------------- */
/* impression aide                                                      */
/* -------------------------------------------------------------------- */
static void do_help(void)
{
        printf("o-----------------------------------------------o\n");
        printf("|                                               |\n");
        printf("| phipsi :  Version 0.99 ABI       June   93    |\n");
        printf("|                                               |\n");
        printf("o-----------------------------------------------o\n");
        printf("| synopsis : phi psi omega alpha tau chi(s)     |\n");
        printf("| and CA-1 CA+1 distance computing              |\n");
        printf("|                                               |\n");
        printf("| usage: phipsi [options] [<] pdb_file          |\n");
        printf("|                                               |\n");
        printf("o-----------------------------------------------o\n");
        printf("| options:                                      |\n");
        printf("| -p  : computes phi, psi and omega             |\n");
        printf("| -a  : computes alpha,tau and CA_CA+1 distance |\n");
        printf("| -x  : computes chi1,chi2 and chi3             |\n");
        printf("| -d  : computes CA-1_CA+1 distance             |\n");
        printf("| -c  : output CA coordinates                   |\n");
        printf("| -b  : output binary file                      |\n");
        printf("| -r  : read binary file and output formatted   |\n");
	printf("|       file                                    |\n");
        printf("| -b  : computes alpha,tau and CA_CA+1 distance |\n");
        printf("|       and output CA coordinates in binary     |\n");
        printf("| -C <chain> : output only angles for chain     |\n");
	printf("|       <chain> (<chain> is one caracter)       |\n");
        printf("| -m <number> : output only angles for residues |\n");
	printf("|       whose PDB numbers are greater than      |\n");
	printf("|       specified number                        |\n");
        printf("| -M <number> : output only angles for residues |\n");
	printf("|       whose PDB numbers are lesser than       |\n");
	printf("|       specified number                        |\n");
        printf("| -t <number> : output only if number of        |\n");
	printf("|       residues to output is greater than      |\n");
	printf("|       specified number                        |\n");
        printf("|       to output is greater than number        |\n");
        printf("| -n  : split chains, outputs as many structures|\n"),
        printf("|       as chains and output CA coordinates     |\n"),
        printf("|       this option is intended for making      |\n");
        printf("|       a binary alpha bank without cutting the |\n");
        printf("|       PDB bank in chains before processing    |\n");
        printf("|       nor precising the chains                |\n");
        printf("|       (incompatible with other options)       |\n");
        printf("| -v  : verbose mode (on stderr)                |\n");
        printf("| -h  : displays this help                      |\n");
        printf("|                                               |\n");
        printf("|  The pdb_file can be in compressed format     |\n");
        printf("|    (i.e. .Z or .gz file)                      |\n");
        printf("o-----------------------------------------------o\n");
        printf("| note: phipsi gives only values for the 1st    |\n");
        printf("|        model (i.e. it stops at at ENDMDL)     |\n");
        printf("o-----------------------------------------------o\n");
        printf("| if for any reason (missing atoms, angle not   |\n");
        printf("| defined,...) the angle cannot be computed,    |\n");
        printf("| it is set to 500.0                            |\n");
        printf("|                                               |\n");
        printf("o-----------------------------------------------o\n");
        printf("| output              : on standard output      |\n");
        printf("| comments and errors : on standard error       |\n");
        printf("|                                               |\n");
        printf("o-----------------------------------------------o\n");
        printf("|                                               |\n");
        printf("| The ouput values are in order (on one line):  |\n");
        printf("| Atom_name Residue_name Residue_number [Phi \\  |\n");
        printf("| Psi Omega] [Alpha Tau CA_CA+1_Distance]    \\  |\n");
        printf("| [Chi1 Chi2 Chi3] [CA-1_CA+1_Distance]      \\  |\n");
        printf("| Residue_name [Chain character] Residue_    \\  |\n");
        printf("| number_inPDB[Insertion flag]                  |\n");
        printf("|                                               |\n");
        printf("| Note: For given CA, Alpha is dihedral between |\n");
	printf("|          CA-1,CA,CA+1 and CA+2                |\n");
        printf("| Note: For given CA, Tau is the angle between  |\n");
	printf("|          CA-1,CA,CA+1                         |\n");
        printf("| Note: No ouput on stdout if DNA               |\n");
        printf("o-----------------------------------------------o\n");


}

