/* ---------------------------------------------------------------------------- */
/* Yacool / Gloup / Jul 90							*/
/* Yacool Extensions / To be added into Yacool V1.0 /				*/
/* @File Nmatrix.h								*/
/* New matrix Library								*/
/* ---------------------------------------------------------------------------- */
/* -------------------------------------------- */
/* librairie acces aux matrices			*/
/* include file					*/
/* -------------------------------------------- */

#ifndef _H_Nmatrix

#define _H_Nmatrix

#define INDEL_CHAR              '-'

/* -------------------------------------------- */
/* constantes 									*/
/* -------------------------------------------- */

#define MAT_SIZE_MAX	32

#define MAT_MINVAL		0
#define MAT_MAXVAL		100
			/* unitary matrix values for PAM type */
#define MAT_PAM_MINVAL		-4
#define MAT_PAM_MAXVAL		7

#define DISTANCE_METHOD		1
#define SIMPLEDIST_METHOD	2
#define CHI2_METHOD		3

			/* gap for unitary PAM	*/
#define GAP_OPEN_RATIO        1.2
#define GAP_EXT_RATIO         0.8
#define GAP_GAP_RATIO         0.5

#define MIN2(a, b)       ((a) < (b) ? (a) : (b))

#define MAT_PAM_TYPE	0
#define MAT_PROX_TYPE	1
#define MAT_DIST_TYPE	2

/* -------------------------------------------- */
/* structures 					*/
/* -------------------------------------------- */

typedef struct {
	char	name[256], pathname[256];
	int		type, size, threshold;
	int	val[MAT_SIZE_MAX][MAT_SIZE_MAX], 
			classe[MAT_SIZE_MAX][MAT_SIZE_MAX],
			weight[MAT_SIZE_MAX];
        double	freq[MAT_SIZE_MAX][MAT_SIZE_MAX];
	char	vocab[MAT_SIZE_MAX];
	int	min_val, max_val;
} pam_matrix;


typedef struct {
	int		size;
	int		type;
	int		gap_ext;
	int	val[MAT_SIZE_MAX][MAT_SIZE_MAX];
	char	vocab[MAT_SIZE_MAX];
} pseudo_matrix;

/* -------------------------------------------- */
/* types 					*/
/* -------------------------------------------- */

typedef enum { faux = 0, vrai } Boolean;

typedef pam_matrix PamMatrix, *PamMatrixPtr; 

/* -------------------------------------------- */
/* prototypes 									*/
/* -------------------------------------------- */

Boolean load_matrix(pam_matrix *matrix, char *pamname);
Boolean read_blosum_matrix(FILE *filin, pam_matrix* matrix);
Boolean write_matrix(char *outname, pam_matrix *matrix);
Boolean reorder_matrix(pam_matrix *matrix, char *std_vocab);
Boolean diag_verify(pam_matrix *matrix);
Boolean sym_verify(pam_matrix *matrix);
Boolean val_verify(pam_matrix *matrix);
Boolean simple_verify(pam_matrix *matrix, Boolean diagtoo);
Boolean complain_pam_matrix(pam_matrix *matrix);
Boolean complain_fmt_matrix(pam_matrix *matrix);

Boolean simple_convert(pam_matrix *matrix, int totype, int verbose, int convert_method);
Boolean nodiag_convert(pam_matrix *matrix, int totype);
Boolean chi2_convert(pam_matrix *matrix);

void 	printinfo_matrix(FILE *dev, pam_matrix *matrix, char *head, 
			Boolean put_values, Boolean put_classes);
void 	unitary_matrix(pam_matrix *matrix, int type, char *vocab);
void	my_alert(FILE *stream, char *fmt, ...);
void	set_sIndelGapGapRatio(float value);
float	get_sIndelGapGapRatio(void);
void	set_sIndelExtRatio(float value);
float	get_sIndelExtRatio(void);
void	set_sIndelOpeRatio(float value);
float	get_sIndelOpeRatio(void);
double matcount(double *tab, int pos, int len, pam_matrix *mat, double val);
void compute_freq_matrix(pam_matrix *mat, int verbose);
#endif
