/*--------------------------------------------------------------*/
/* kontrol.c 	Decembre 1994					*/
/* controle sur les process, et eventuellement serial killer	*/
/*--------------------------------------------------------------*/
#include <stdio.h>

#ifdef SunOS
#include <sys/types.h>
#endif /*  SunOS	*/
#include <string.h>   	  /* for strcat, etc... */

#if (defined IRIX) || (defined SunOS)
#include <sys/statfs.h>		/* for struct statfs		*/
#include <limits.h>		/* for MAX_PATH		*/
#endif /* IRIX et SunOS */

#ifdef Darwin
#include <sys/syslimits.h> /* for MAX_PATH         */
#include <sys/time.h>
#include <sys/resource.h>
#endif

#ifdef OSF1     /* DEC */
#include <sys/mount.h>
#endif /* endif OSF1 DEC */

#ifdef Linux
#include <linux/limits.h> 	/* for MAX_PATH         */
#include <sys/time.h>
#include <sys/resource.h>
#endif /* Linux */

#include <time.h>		/* clock_t(sys/siginfo.h needs)	*/
#include <fcntl.h>		/* for O_RDONLY			*/
#include <errno.h>		/* for errno			*/
#include <setjmp.h>		/* for longjmp			*/
#include "kontrol.proto.h"	/* prototype file		*/


#if (defined IRIX) || (defined OSF1) || (defined SunOS) 
#include <sys/procfs.h>		/* for prpsinfo or prusage_t or psinfo_t */
#endif /* endif (defined IRIX) || (defined OSF1) */
#if  (defined SunOSXX)
#include <sys/old_procfs.h>		/* for prpsinfo or prusage_t or psinfo_t */
#endif
#undef DEBUGKONTROL

#ifdef DEBUGKONTROL

#define PRINT_DEBUG_STRING fprintf(stderr,"%s\n",sterr);

#else /* not DEBUGKONTROL */

#define PRINT_DEBUG_STRING  ;

#endif /* not DEBUGKONTROL */


#define MAX_KMR_MSG	512


/*--------------------------------------------------------------*/
/* prototypes locaux						*/
/*--------------------------------------------------------------*/

/*--------------------------------------------------------------*/
/* statiques							*/
/*--------------------------------------------------------------*/

#if (! defined Darwin) && (! defined Linux) && (! defined SunOS)
/*--------------------------------------------------------------*/
/* control_prog: controle le programme "prog_name", qui a	*/
/* un pid prog_pid, pour ne pas depasser maxmemKo Ko		*/
/* memoire, un message d'erreur de retour "err_msg",	*/
/*--------------------------------------------------------------*/
int control_prog(char *prog_name, pid_t prog_pid, int maxcputime,
		int maxmemKo, char **err_msg)
{
   static char	sterr[512];
   char		active_proc_path[PATH_MAX];
   int		fildes, retval;
#ifdef IRIX
   prusage_t	my_prusage, *my_prusagePtr = &my_prusage;
#endif
#if (defined OSF1) || (defined SunOSXX)     /* DEC */
   prpsinfo_t	my_prpsinfo, *my_prpsinfoPtr = &my_prpsinfo;
#endif /* OSF1 */


	/* cette routine controle le process "prog_name"	*/
	/* et de pid prog_pid					*/
	/* et retourne:						*/
	/* - UNK_ERROR		si erreur d'ouverture du ac-	*/
	/* 			tive_proc_path, i.e.		*/
	/*			/debug/"pid"			*/
	/*			ou plus recemment /proc/"pid"	*/
	/* - PROC_FINISHED	si le process est fini		*/
	/*			ou n'existe pas			*/
	/* - EXCEEDS_MEM_KO	si le process excede la taille	*/
	/*			memoire "maxmemKo" (en Ko)	*/
	/* - EXCEEDS_CPUTIME	si le process excede le temps	*/
	/* 			CPU "maxcputime"		*/
	/*							*/
	/* NOTE: *err_msg est alloue ici en statique		*/


				/* ---------------------------- */
				/* set the error message	*/
				/* ---------------------------- */
   *err_msg = sterr;



#if (defined IRIX) || (defined OSF1) || (defined SunOSXX)
				/* ---------------------------- */
				/* copie le nom du directory	*/
				/* des process			*/
				/* ---------------------------- */
   strcpy(active_proc_path,"/proc/");

				/* ---------------------------- */
				/* set the active proc name	*/
				/* it is the "id" file that is	*/
				/* in the /proc directory	*/
				/* ---------------------------- */
   if (strlen(prog_name) > (size_t)PATH_MAX) {
      sprintf(sterr,"control_prog:: program name too long");
      PRINT_DEBUG_STRING
      return UNK_ERROR;
   }

#endif /* (defined IRIX) || (defined OSF1) */

   sprintf(sterr,"%d", (int)prog_pid);
   strcat(active_proc_path, sterr);


#if (defined IRIX) || (defined OSF1) || (defined SunOSXX)

      fildes = open (active_proc_path, O_RDONLY);

				/* ---------------------------- */
      if (fildes == -1) {	/* open failed			*/ 
				/* ---------------------------- */
         if (errno == ENOENT) {
           sprintf(sterr,
		"control_prog:: program %s (id %d) has finished (not enough memory ?)\n",
		prog_name, (int)prog_pid);
	   PRINT_DEBUG_STRING
	   return PROC_FINISHED;	/* or non existent	*/
         }
         else {
            sprintf(sterr,
		"control_prog:: Can't open %s (errno:%d)",
		active_proc_path, errno);
	    return UNK_ERROR;		/* unknown error	*/
         }
      }
           
#if (defined IRIX) 
#ifdef IRIX
      retval = ioctl(fildes, PIOCUSAGE, my_prusagePtr);
#endif
#ifdef SunOSXX
      retval = ioctl(fildes, PIOCUSAGE, my_psinfoPtr);
#endif
      if (retval == -1) {
				/* ---------------------------- */
         if (errno == ENOENT) {	/* fildes non existent		*/
				/* ---------------------------- */
            sprintf(sterr,
		"control_prog:: program %s (id %d) has finished\n",
		prog_name, (int)prog_pid);
	   PRINT_DEBUG_STRING
	   return PROC_FINISHED;	/* or non existent	*/
         }
	 else {
           sprintf(sterr,
		"control_prog:: Can't ioctl PIOCUSAGE %s errno :%d",
		active_proc_path, errno);
	   PRINT_DEBUG_STRING
				/* ---------------------------- */
	   return UNK_ERROR;	/* cannot ioctl PIOCUSAGE	*/
				/* ---------------------------- */
         }
      }
      else {
	 if (my_prusagePtr->pu_utime.tv_sec > maxcputime) {
	    sprintf(sterr,
		"%s CPU User Time:: %d s exceeds %d s allowed",
		prog_name, (int)my_prusagePtr->pu_utime.tv_sec,
		(int)maxcputime);
	   PRINT_DEBUG_STRING
	   return EXCEEDS_CPUTIME;
	 }
	 if ((int)((float)(my_prusagePtr->pu_size)*4.096) > maxmemKo) {
	    sprintf(sterr,
		"%s Swappable Image (%d Ko) exceeds max allowed (%d Ko)",
		prog_name,
		(int)(((float)my_prusagePtr->pu_size)*4.096),
		maxmemKo);
	   PRINT_DEBUG_STRING
	   return EXCEEDS_MEM_KO;
	 }
      }
#else
#if (defined OSF1) || (defined SunOSXX)     /* DEC */
      retval = ioctl(fildes, PIOCPSINFO, my_prpsinfoPtr);

      if (retval == -1) {
				/* ---------------------------- */
         if (errno == ENOENT) {	/* fildes non existent		*/
				/* ---------------------------- */
            sprintf(sterr,
		"control_prog:: program %s (id %d) has finished\n",
		prog_name, (int)prog_pid);
	   PRINT_DEBUG_STRING
	   return PROC_FINISHED;	/* or non existent	*/
         }
	 else {
           sprintf(sterr,
		"control_prog:: Can't ioctl PIOCPSINFO %s errno :%d",
		active_proc_path, errno);
	   PRINT_DEBUG_STRING
				/* ---------------------------- */
	   return UNK_ERROR;	/* cannot ioctl PIOCPSINFO	*/
				/* ---------------------------- */
         }
      }
      else {
	 if (my_prpsinfoPtr->pr_time.tv_sec > maxcputime) {
	    sprintf(sterr,
		"%s CPU User Time:: %d s exceeds %d s allowed",
		prog_name, (int)my_prpsinfoPtr->pr_time.tv_sec,
		(int)maxcputime);
	   PRINT_DEBUG_STRING
	   return EXCEEDS_CPUTIME;
	 }
	 if ((int)((float)(my_prpsinfoPtr->pr_size)*4.096) > maxmemKo) {
	    sprintf(sterr,
		"%s Swappable Image (%d Ko) exceeds max allowed (%d Ko)",
		prog_name,
		(int)(((float)my_prpsinfoPtr->pr_size)*4.096),
		maxmemKo);
	   PRINT_DEBUG_STRING
	   return EXCEEDS_MEM_KO;
	 }
         if ((int)my_prpsinfoPtr->pr_zomb == 1) {
					/* zombie process	*/
	    sprintf(sterr,
              "%s is zombie process <%c>",
	      prog_name,
	      my_prpsinfoPtr->pr_sname);
	         PRINT_DEBUG_STRING
            return ZOMBIE_PROCESS;
         }
       }
#endif /* OSF1 */
#endif /* IRIX */

#if 1
#if (defined IRIX) 
         fprintf(stderr, "--------------------------------------\n");
         fprintf(stderr, "%s: CPU User Time : %8d sec %4d msec\n",
			prog_name,
			(int)my_prusagePtr->pu_utime.tv_sec,
			(int)(my_prusagePtr->pu_utime.tv_nsec/1000000));
					/* pu_size is unsigned long */
         fprintf(stderr, "%s: Swappable Image in Ko : %10d\n",
			prog_name,
			(int)((float)(my_prusagePtr->pu_size)*4.096));
#else
#if (defined OSF1) || (defined SunOSXX)     /* DEC */
         fprintf(stderr, "--------------------------------------\n");
     fprintf(stderr,"%s:ioctl PIOCPSINFO returns %d (zombie char=<%d>, task = <%d>=<%c>)\n",
                   prog_name, (int)retval, (int)(my_prpsinfoPtr->pr_zomb),
		 (int)(my_prpsinfoPtr->pr_state), my_prpsinfoPtr->pr_sname);
         fprintf(stderr, "%s: CPU User Time : %8d sec %4d msec\n",
			prog_name,
			(int)(my_prpsinfoPtr->pr_time).tv_sec,
			(int)(my_prpsinfoPtr->pr_time.tv_nsec/1000000));
					/* pu_size is unsigned long */
         fprintf(stderr, "%s: Swappable Image in Ko : %10d\n",
			prog_name,
			(int)((float)(my_prpsinfoPtr->pr_size)*4.096));
#endif /* OSF1     DEC */
#endif /* IRIX */
#endif /* 0 */

      close(fildes);

   return 0;

}
#endif /* end of defined IRIX || defined OSF1 */



#endif /* (not defined Darwin) */

#if (defined Darwin) || (defined Linux) || (defined SunOS)
/*--------------------------------------------------------------*/
/* control_prog: controle le programme "prog_name", qui a	*/
/* un pid prog_pid, pour ne pas depasser maxmemKo Ko		*/
/* memoire, un message d'erreur de retour "err_msg",	*/
/*--------------------------------------------------------------*/
int control_prog(char *prog_name, pid_t prog_pid, int maxcputime,
		int maxmemKo, char **err_msg)
{
/*
#define   RUSAGE_SELF     0
#define   RUSAGE_CHILDREN     -1
*/
   static char	sterr[512];
   /* struct proc *myproc; */
   struct rusage srusage, *rusage=&srusage;
   int who = RUSAGE_SELF;

	/* cette routine controle le process "prog_name"	*/
	/* et de pid prog_pid					*/
	/* et retourne:						*/
	/* - UNK_ERROR		si erreur d'ouverture du ac-	*/
	/* 			tive_proc_path, i.e.		*/
	/*			/debug/"pid"			*/
	/*			ou plus recemment /proc/"pid"	*/
	/* - PROC_FINISHED	si le process est fini		*/
	/*			ou n'existe pas			*/
	/* - EXCEEDS_MEM_KO	si le process excede la taille	*/
	/*			memoire "maxmemKo" (en Ko)	*/
	/* - EXCEEDS_CPUTIME	si le process excede le temps	*/
	/* 			CPU "maxcputime"		*/
	/*							*/
	/* NOTE: *err_msg est alloue ici en statique		*/


				/* ---------------------------- */
				/* set the error message	*/
				/* ---------------------------- */
   *err_msg = sterr;




   /* myproc = pfind(prog_pid); */

     if (getrusage(who, rusage) == -1) {
       strcpy(sterr, "Error in getrusage\n");
       return 1;
     }
     fprintf(stderr, "--------------------------------------\n");
     fprintf(stderr, "%s: CPU User Time : %8d sec %4d usec\n",
			prog_name,
			(int)rusage->ru_utime.tv_sec,
			(int)(rusage->ru_utime.tv_usec));
     
     fprintf(stderr, "%s: mem size in Ko : ru_minflt:%d ru_majflt:%d\n", 
		     prog_name, (int)(rusage->ru_minflt), (int)(rusage->ru_majflt));
     /* OLD jompo */
      fprintf(stderr, "%s: mem size in Ko : ru_maxrss:%d ru_ixrss:%d ru_idrss:%d ru_isrss:%d ru_minflt:%d ru_majflt:%d ru_inblock %d ru_oublock %d ru_nswap:%d\n",
			prog_name,
			(int)(rusage->ru_maxrss),
			(int)(rusage->ru_ixrss),
			(int)(rusage->ru_isrss),
			(int)(rusage->ru_idrss),
			(int)(rusage->ru_minflt),
			(int)(rusage->ru_majflt),
			(int)(rusage->ru_inblock),
			(int)(rusage->ru_oublock),
			(int)(rusage->ru_nswap));
	
   return 0;

}

#endif /* ifdef Darwin || Linux */


#if 0
/* Darwin*/
     struct rusage {
             struct timeval ru_utime; /* user time used */
             struct timeval ru_stime; /* system time used */
             long ru_maxrss;          /* integral max resident set size */
             long ru_ixrss;           /* integral shared text memory size */
             long ru_idrss;           /* integral unshared data size */
             long ru_isrss;           /* integral unshared stack size */
             long ru_minflt;          /* page reclaims */
             long ru_majflt;          /* page faults */
             long ru_nswap;           /* swaps */
             long ru_inblock;         /* block input operations */
             long ru_oublock;         /* block output operations */
             long ru_msgsnd;          /* messages sent */
             long ru_msgrcv;          /* messages received */
             long ru_nsignals;        /* signals received */
             long ru_nvcsw;           /* voluntary context switches */
             long ru_nivcsw;          /* involuntary context switches */
     };

#endif
/* CE QUI SUIT SONT DES COMMENTAIRES CONTENANT DES INFOS	*/
/* SUR PIOCUSAGE, fstatfs, etc...                       	*/
/*

 FAIRE man proc POUR AVOIR CES INFORMATIONS

   il y a la fonction prctl
     ptrdiff_t prctl (unsigned option, ...);

DESCRIPTION
     prctl provides information about processes and the ability to control
     certain of their attributes.  The return type ptrdiff_t is used so that
     prctl can return an integral value that is large enough to return a
     pointer/address.  option specifies one of the following actions:

     PR_MAXPROCS      returns the system imposed limit on the number of
                      processes per user.

     PR_MAXPPROCS     returns the maximum number of processors the calling
                      process can utilize.  If the caller is restricted (see
                      sysmp(2)) to run on a particular processor then
                      PR_MAXPPROCS will return 1.  If some of the processors
                      in the complex have been restricted (see mpadmin(1))
                      these will not be counted.

     PR_ISBLOCKED     returns 1 if the specified process is currently blocked.
                      The second argument, interpreted as type pid_t,
                      specifies the target process's pid.  Passing in 0
                      implies the caller.  Since other processes could have
                      subsequently unblocked the subject process, the result
                      should be considered as a snapshot only.

 fonction fstatfs:

      retval = fstatfs(fildes, statptr, sizeof(struct statfs), fstyp);
      if (retval == -1) {
         myerror(stderr,"control_prog::Can't fstatfs %s errno :%d",
	   active_proc_path, errno);
      }
      else {
         myerror(stderr, "Total number of blocks   :%ld\n",statptr->f_blocks);
         myerror(stderr, "Count of free blocks     :%ld\n",statptr->f_bfree);
      }
*/

/*
     The statfs structure pointed to by buf includes the following members:
 *//*       short   f_fstyp;    *//* File system type */
/*          long    f_bsize;    *//* Block size */
/*          long    f_frsize;   *//* Fragment size */
/*          long    f_blocks;   *//* Total number of blocks */
/*          long    f_bfree;    *//* Count of free blocks */
/*          long    f_files;    *//* Total number of file nodes */
/*          long    f_ffree;    *//* Count of free file nodes */
/*          char    f_fname[6]; *//* Volume name */
/*          char    f_fpack[6]; *//* Pack name */

/* 
     The argument p is a generic pointer whose type depends on the specific
     ioctl code.  Where not specifically mentioned below, its value should be
     zero.  <sys/procfs.h> contains definitions of ioctl codes and data
     structures used by the operations.

     Process information and control operations involve the use of sets of
     flags.  The set types sigset_t, fltset_t, and sysset_t correspond,
     respectively, to signal, fault, and system call enumerations defined in
     <sys/signal.h>, <sys/fault.h>, and <sys/syscall.h>.  Each set type is
     large enough to hold flags for its own enumeration.  Although they are of
     different sizes, they have a common structure and can be manipulated by
     these macros:
 */
/*          prfillset(&set);              *//* turn on all flags in set */
/*          premptyset(&set);             *//* turn off all flags in set */
/*          praddset(&set, flag);         *//* turn on the specified flag */
/*          prdelset(&set, flag);         *//* turn off the specified flag */
/*          r = prismember(&set, flag);   *//* != 0 iff flag is turned on */
/*
     One of prfillset() or premptyset() must be used to initialize set before
     it is used in any other operation.  flag must be a member of the
     enumeration corresponding to set.
 */


/*
   PIOCPSINFO
     This returns miscellaneous process information such as that reported by
     ps(1).  p is a pointer to a prpsinfo structure containing at least the
     following fields:
 */
/*    typedef struct prpsinfo {*/
/*             char        pr_state;     *//* numeric process state (see pr_sname) */
/*             char        pr_sname;     *//* printable character representing pr_state */
/*             char        pr_zomb;      *//* !=0: process terminated but not waited for */
/*             char        pr_nice;      *//* nice for cpu usage */
/*             u_long      pr_flag;      *//* process flags */
/*             uid_t       pr_uid;       *//* real user id */
/*             gid_t       pr_gid;       *//* real group id */
/*             pid_t       pr_pid;       *//* unique process id */
/*             pid_t       pr_ppid;      *//* process id of parent */
/*             pid_t       pr_pgrp;      *//* pid of process group leader */
/*             pid_t       pr_sid;       *//* session id */
/*             caddr_t     pr_addr;      *//* physical address of process */
/*             long        pr_size;      *//* size of process image in pages */
/*             long        pr_rssize;    *//* resident set size in pages */
/*             long        pr_pagesize;  *//* system page size, in bytes */
/*             caddr_t     pr_wchan;     *//* wait addr for sleeping process */
/*             timestruc_t pr_start;     *//* process start time, sec+nsec since epoch */
/*             timestruc_t pr_time;      *//* usr+sys cpu time for this process */
/*             long        pr_pri;       *//* priority, high value is high priority */
/*             char        pr_oldpri;    *//* pre-SVR4, low value is high priority */
/*             char        pr_cpu;       *//* pre-SVR4, cpu usage for scheduling */
/*             dev_t       pr_ttydev;    *//* controlling tty device (PRNODEV if none) */
/*             char        pr_clname[8]; *//* Scheduling class name */
/*             char        pr_fname[16]; *//* last component of exec()ed pathname */
/*             char        pr_psargs[PRARGSZ];  */ /* initial characters of arg list */
/*     } prpsinfo_t; */
/* prusage structure */
/*typedef struct prusage {*/
/*        timestruc_t pu_tstamp;  *//* time stamp */
/*        timestruc_t pu_starttime;       *//* time process was started */
/*        timestruc_t pu_utime;   *//* user CPU time */
/*        timestruc_t pu_stime;   *//* system CPU time */
/*        u_long pu_minf;         *//* minor (mapping) page faults */
/*        u_long pu_majf;         *//* major (disk) page faults */
/*        u_long pu_utlb;         *//* User TLB misses - always zero */
/*        u_long pu_nswap;       *//* swaps (process only) */
/*        u_long pu_gbread;       *//* gigabytes ... */
/*        u_long pu_bread;        *//* and bytes read */
/*        u_long pu_gbwrit;       *//* gigabytes ... */
/*        u_long pu_bwrit;        *//* and bytes written */
/*        u_long pu_sigs;         *//* signals received */
/*        u_long pu_vctx;         *//* voluntary context switches */
/*       u_long pu_vctx;         *//* voluntary context switches */
/*        u_long pu_ictx;        */ /* involuntary context switches */
/*        u_long pu_sysc;         *//* system calls */
/*        u_long pu_syscr;        *//* read() system calls */
/*        u_long pu_syscw;        *//* write() system calls */
/*        u_long pu_syscps;       *//* poll() or select() system calls */
/*        u_long pu_sysci;        *//* ioctl() system calls */
/*        u_long pu_graphfifo;    *//* graphics pipeline stalls */
/*        u_long pu_graph_req[8]; *//* graphics resource requests */
/*        u_long pu_graph_wait[8];*//* graphics resource waits */
/*        u_long pu_size;         *//* size of swappable image in pages */
/*        u_long pu_rsssize;      *//* resident size of swappable image */
/*        u_long pu_inblock;      *//* block input operations */
/*        u_long pu_oublock;      *//* block output operations */
/*        u_long pu_vfault;       *//* total number of vfaults */
/*        u_long pu_unused[1];    *//* currently unused */
/* } prusage_t; */
