/* ---------------------------------------------------- */
/* SpIMac V 0.0 					*/
/* <Gloup> Nov. 93					*/
/*							*/
/* @file: nr_util.c					*/
/*							*/
/* Mathematical Utilitary functions			*/
/*							*/
/* @globals:						*/
/*	<none>						*/
/* ---------------------------------------------------- */
/*							*/
/* taken "as is" from NUMERICAL RECIPES			*/
/* original file :  NRUTIL.C				*/
/*							*/
/*							*/
/* oops: slightly modified to fit on mac interface 	*/
/* ...and on ANSI compliance...				*/
/* ---------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>

#include "nr_util.h"

/* ------------------------------------ */
/* Num. Recipes error routine		*/
/* ------------------------------------ */
void nrerror(char *error_text)

{
	fprintf(stderr,"> Numerical Recipes run-time error...\n");
	fprintf(stderr,"> %s\n", error_text);
	exit(1);
}


/* ------------------------------------ */
/* float (default) vector allocation	*/
/* ------------------------------------ */
float *vector(int nl, int nh)

{
        float *v;

        v=(float *)malloc((unsigned) (nh-nl+1)*sizeof(float));
        if (!v) {
        	nrerror("allocation failure in vector()");
        	return NULL;
        }
        return v-nl;
}

/* ------------------------------------ */
/* integer vector allocation		*/
/* ------------------------------------ */
int *ivector(int nl, int nh)

{
        int *v;

        v=(int *)malloc((unsigned) (nh-nl+1)*sizeof(int));
        if (!v) {
        	nrerror("allocation failure in ivector()");
        	return NULL;
        }
        return v-nl;
}

/* ------------------------------------ */
/* double vector allocation		*/
/* ------------------------------------ */
double *dvector(int nl, int nh)

{
        double *v;

        v=(double *)malloc((unsigned) (nh-nl+1)*sizeof(double));
        if (!v) {
        	nrerror("allocation failure in dvector()");
        	return NULL;
        }
        return v-nl;
}

/* ------------------------------------ */
/* float (default) matrix allocation	*/
/* ------------------------------------ */
float **matrix(int nrl, int nrh, int ncl, int nch)

{
        int i, j;
        float **m;

        m=(float **) malloc((unsigned) (nrh-nrl+1)*sizeof(float*));
        if (!m) {
        	nrerror("allocation failure 1 in matrix()");
        	return NULL;
        }
        m -= nrl;

        for(i=nrl;i<=nrh;i++) {
                m[i]=(float *) malloc((unsigned) (nch-ncl+1)*sizeof(float));
                if (!m[i]) {
                	nrerror("allocation failure 2 in matrix()");
                	for (j = nrl ; j < i ; j++)
                		free(m[j] + ncl);
                	free(m + nrl);
		        	return NULL;
        		}
                m[i] -= ncl;
        }
        return m;
}

/* ------------------------------------ */
/* double matrix allocation		*/
/* ------------------------------------ */
double **dmatrix(int nrl, int nrh, int ncl, int nch)

{
        int i, j;
        double **m;

        m=(double **) malloc((unsigned) (nrh-nrl+1)*sizeof(double*));
        if (!m) {
        	nrerror("allocation failure 1 in dmatrix()");
	        return NULL;
        }
        m -= nrl;

        for(i=nrl;i<=nrh;i++) {
                m[i]=(double *) malloc((unsigned) (nch-ncl+1)*sizeof(double));
                if (!m[i]) {
                	nrerror("allocation failure 2 in dmatrix()");
                 	for (j = nrl ; j < i ; j++)
                		free(m[j] + ncl);
	               	free(m + nrl);
		        	return NULL;
        		}
                m[i] -= ncl;
        }
        return m;
}

/* ------------------------------------ */
/* integer matrix allocation		*/
/* ------------------------------------ */
int **imatrix(int nrl, int nrh, int ncl, int nch)

{
        int i,j, **m;

        m=(int **)malloc((unsigned) (nrh-nrl+1)*sizeof(int*));
        if (!m) {
        	nrerror("allocation failure 1 in imatrix()");
	        return NULL;
        }
        m -= nrl;

        for(i=nrl;i<=nrh;i++) {
                m[i]=(int *)malloc((unsigned) (nch-ncl+1)*sizeof(int));
                if (!m[i]) {
                	nrerror("allocation failure 2 in imatrix()");
                 	for (j = nrl ; j < i ; j++)
                		free(m[j] + ncl);
	               	free(m + nrl);
		        	return NULL;
        		}
                m[i] -= ncl;
        }
        return m;
}


/* ------------------------------------ */
/* sub-matrix allocation/extraction	*/
/* ------------------------------------ */
float **submatrix(float **a,
		int oldrl, int oldrh, int oldcl, int oldch, int newrl, int newcl)

{
        int i,j;
        float **m;

        m=(float **) malloc((unsigned) (oldrh-oldrl+1)*sizeof(float*));
        if (!m) {
        	nrerror("allocation failure in submatrix()");
			return NULL;
		}
        m -= newrl;

        for(i=oldrl,j=newrl;i<=oldrh;i++,j++) m[j]=a[i]+oldcl-newcl;

        return m;
}

/* ------------------------------------ */
/* float vector deallocation		*/
/* ------------------------------------ */
void free_vector(float *v, int nl, int nh)

{
        free((char*) (v+nl));
}

/* ------------------------------------ */
/* integer vector deallocation		*/
/* ------------------------------------ */
void free_ivector(int *v, int nl, int nh)

{
        free((char*) (v+nl));
}

/* ------------------------------------ */
/* double vector deallocation		*/
/* ------------------------------------ */
void free_dvector(double *v, int nl, int nh)

{
        free((char*) (v+nl));
}

/* ------------------------------------ */
/* float matrix deallocation		*/
/* ------------------------------------ */
void free_matrix(float **m, int nrl, int nrh, int ncl, int nch)

{
        int i;

        for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
        free((char*) (m+nrl));
}

/* ------------------------------------ */
/* double matrix deallocation		*/
/* ------------------------------------ */
void free_dmatrix(double **m, int nrl, int nrh, int ncl, int nch)

{
        int i;

        for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
        free((char*) (m+nrl));
}

/* ------------------------------------ */
/* integer matrix deallocation		*/
/* ------------------------------------ */
void free_imatrix(int **m, int nrl, int nrh, int ncl, int nch)

{
        int i;

        for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
        free((char*) (m+nrl));
}


/* ------------------------------------ */
/* sub-matrix deallocation		*/
/* ------------------------------------ */
void free_submatrix(float **b, int nrl, int nrh, int ncl, int nch)

{
        free((char*) (b+nrl));
}


/* ------------------------------------ */
/* vector -> matrix conversion		*/
/* ------------------------------------ */
float **convert_matrix(float *a, int nrl, int nrh, int ncl, int nch)

{
        int i,j,nrow,ncol;
        float **m;

        nrow=nrh-nrl+1;
        ncol=nch-ncl+1;
        m = (float **) malloc((unsigned) (nrow)*sizeof(float*));
        if (!m) {
        	nrerror("allocation failure in convert_matrix()");
			return NULL;
		}
        m -= nrl;
        for(i=0,j=nrl;i<=nrow-1;i++,j++) m[j]=a+ncol*i-ncl;
        return m;
}

/* ------------------------------------ */
/* converted matrix deallocation	*/
/* ------------------------------------ */
void free_convert_matrix(float **b, int nrl, int nrh, int ncl, int nch)

{
        free((char*) (b+nrl));
}
