/* This file is adapted from the sim.c program see below.
Some modifications have been done to fit it to serve as a function for swelfe
main program
(tunes for structures and other similarity scores, etc...)
*/
/* A PROGRAM FOR LOCAL SIMILARITIES WITH AFFINE WEIGHTS:

    copyright (c) 1990-1997 Xiaoqiu Huang and Webb Miller
    The distribution of the program is granted provided no charge is made
    and the copyright notice is included.
    E-mail: huang@cs.mtu.edu

IN NO EVENT SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    Please cite the paper
    "A Time-Efficient, Linear-Space Local Similarity Algorithm"
	(Advances in Applied Mathematics, 12: 337-357, 1991)

	      Xiaoqiu Huang
	      Department of Computer Science
	      Michigan Technological University
	      Houghton, MI 49931

	      Webb Miller
	      Department of Computer Science
	      The Pennsylvania State University
	      University Park, PA 16802

     An early version of this program is presented in the paper
     "A Space-Efficient Algorithm for Local Similarities"
	(Computer Applications in the Biosciences, 6(4): 373-381, 1990)

	      Xiaoqiu Huang
	      Department of Computer Science
	      Michigan Technological University
	      Houghton, MI 49931

	      Ross C. Hardison and Webb Miller
	      Department of Molecular and Cell Biology
	      Department of Computer Science
	      The Pennsylvania State University
	      University Park, PA 16802

    The SIM program finds k best non-intersecting alignments between
    two sequences or within one sequence. Using dynamic programming
    techniques, SIM is guaranteed to find optimal alignments. The
    alignments are reported in order of similarity score, with the
    highest scoring alignment first. The k best alignments share no
    aligned pairs. SIM requires space proportional to the sum of the
    input sequence lengths and the output alignment lengths. Thus
    SIM can handle sequences of tens of thousands, even a hundred of
    thousands, of base pairs on a workstation. For example,
    on 73,360-bp and 44,594-bp sequences, SIM took 15 hours to find
    100 best local alignments on a Sun4 workstation.
 */

#include   <stdio.h>
#include   <stdlib.h>
#include   <string.h>
#include   <math.h>

#include   "macros.h"
#include   "swelfe.h"
#include   "swalign.h"
#include   "sim.h"
/* OLD JOMPO #define  MATCHSC    10*/		/* match score */

static int match=0, mismh=0;		/* max and min substitution weights */
/* static int v[128][128];OLD JOMPO */			/* substitution scores */
/* static int isdna; OLD JOMPO */         		/* 1, DNA; 0, protein */

static void SIM(int *A,int *B,int M,int N,int K,int Q,int R,int nseq,int (*Igscore)(int a1, int a2, void *mat_thresh), void *mat_thresh, int SPN, tyProt *p0, tyProt *p1, int minlen, float minscore,float minrms, int vverbose, int elongation,float rrmsd, int which_rmsd, SWResult *swres, float taux_recouv, float cross_rms_seuil, float proba_limite, WV *water_ving, int Kwanted);
static void big_pass(int *A,int *B, int M, int N, int K, int nseq, int (*Igscore)(int a1, int a2, void *mat_thresh), void *mat_thresh);
static void display(int *A,int *B,int M,int N,int *S,int AP,int BP);
static void locate(int *A,int *B,int nseq, int (*Igscore)(int a1, int a2, void *mat_thresh), void *mat_thresh);
static void small_pass(int *A, int *B, int count, int nseq, int (*Igscore)(int a1, int a2, void *mat_thresh), void *mat_thresh);
static int no_cross(void);
static int diff(int *A,int *B, int M, int N, int tb, int te, int (*Igscore)(int a1, int a2, void *mat_thresh), void *mat_thresh);
static void fill_salign(int M,int N,int *S, char *sal);
static void ResetStaticVars(void);
void FreeAllStatics(int K, int M);

/* ------------------------------------------------------------------------- */
/* K: the number of local alignments to find */
SWResult *simalign(tyProt *p0, tyProt *p1, int automode, int K,
        float minscore, float proba_limite, float minrms, 
        int (*Igscore)(int a1, int a2, void *mat_thresh), void *mat_thresh, float gap_o, float gap_e,
        float taux_recouv, float cross_rms_seuil, int vverbose, int SPN, WV *water_ving, int minlen, int elongation, float rrmsd, int which_rmsd)
{

   static char *adn = "ACGT";
   static char *protein = "ACDEFGHIKLMNPQRSTVWY";
   SWResult             *swres = NULL;
   int                  i, j;
   int                  Kwanted;		/* number of wanted matches */
   int			q, r;		/* gap open and ext (integer)	*/
   /* OLD int			nn=0, nbpp=0, posd0=-1, posd1=-1, posf0=-1, posf1=-1; */
   /* OLD float                sn=0,  reverse_rms=0, recover=0; */
   /* OLD double		rms=0; */
   /* OLD DtMatrix4x4          matrix4x; */
   /* OLD char                 *salign = NULL; */
   /* OLD float		score, score_blosangle; */
   pam_matrix           *mat;
 int  M, N;				/* Sequence lengths and k     */
  /* OLD int  symbol;	*/			/* The next character	      */
  int  nseq;				/* Number of sequences        */
  /* FILE *Bp, *Ap, *Sp, *ckopen(); */
  /* OLD int  *A,  *B;*/			/* Storing two sequences      */
  /* OLD char  alph[129], *s;*/			/* alphabet */
  /* OLD int  size;*/				/* size of alphabet */

  ResetStaticVars();

  M = p0->nbRes;
  N = p1->nbRes;

  if (SPN == STRUCTURE){
     /* for structures, max diff between 2 angles is	*/
     /* 180*DIVANGLE, minimum diff is 0*DIVANGLE	*/
     match = Ianglescorenouveau(0,0,(float *)mat_thresh);
     mismh = Ianglescorenouveau(0,180*DIVANGLE,(float *)mat_thresh);
  } 
  else if (SPN == NUCLEIQUE) {
     /* for nucleic sequences			*/
     /* values are in freq array		*/
     /* take max and min value of similarity score	*/
     mat = (pam_matrix *)mat_thresh;
     for (i = 0; i < strlen(adn); i++) {
        for (j = i; j < strlen(adn); j++) {
          /* fprintf(stderr, "--->%c %c %f\n",adn[i],adn[j],mat->freq[adn[i]-'A'][adn[j]-'A']); */
          if (Igscore(adn[i]-'A',adn[j]-'A',mat_thresh) > match)
             match = Igscore(adn[i]-'A',adn[j]-'A',mat_thresh);
          if (Igscore(adn[i]-'A',adn[j]-'A',mat_thresh) < mismh)
             mismh = Igscore(adn[i]-'A',adn[j]-'A',mat_thresh);
        }
     }
  }
  else {
     /* for AA sequences			*/
     /* values are in val array			*/
     /* take max and min value of similarity score	*/
     mat = (pam_matrix *)mat_thresh;
     for (i = 0; i < strlen(protein); i++) {
        for (j = i; j < strlen(protein); j++) {
          /* fprintf(stderr, "--->%c %c %d\n",protein[i],protein[j],mat->val[protein[i]-'A'][protein[j]-'A']); */
          if (Igscore(protein[i]-'A',protein[j]-'A',mat_thresh) > match)
             match = Igscore(protein[i]-'A',protein[j]-'A',mat_thresh);
          if (Igscore(protein[i]-'A',protein[j]-'A',mat_thresh) < mismh)
             mismh = Igscore(protein[i]-'A',protein[j]-'A',mat_thresh);
        }
     }
  }  /*fin if SPN == */

  if ((swres = NewSWResult(p1)) == NULL)
     MyError(stderr, 4, "simalign:: not enough memory for swres!\n");


  if (p0 == p1 && automode) /* ceinture et bretelles ;-) */
 	   nseq = 1; /* self-alignment */
  else
	   nseq = 2; /* 2x2 alignment */

  /* p0/1->tAngles are: - indices in alphabet for sequences	*/
  /*                    - angle values*10 for structures      	*/
  /*  q: gap-open penalty is a nonnegative integer */
  /*  r: gap-extend penalty is a positive integer */
  if (SPN == NUCLEIQUE) {
    q = (int)((float)ADN_MULT*gap_o);
    r = (int)((float)ADN_MULT*gap_e);
  }
  else {
    q = (int)gap_o;
    r = (int)gap_e;
  }

  if (vverbose) {
     (void) printf("%s: match %d  mismatch %d  gap-open %d  gap-ext %d\n",
			(SPN==NUCLEIQUE ? "DNA:":(SPN==STRUCTURE?"3D-STRUCTURE:":"PROTEIN:")),
                        match, mismh, q, r);
  }
#ifdef PRINT_OLD
     (void) printf("Match   Mismatch   Gap-Open Penalty   Gap-Extension Penalty\n");
     (void) printf(" %d      %d            %d                 %d\n\n",
			match, mismh, q, r);
  if ( nseq == 2 ) {
      (void) printf("Upper Sequence: %s\n", p0->nomProt);
      (void) printf("        Length: %d\n", M);
      (void) printf("Lower Sequence: %s\n", p1->nomProt);
      (void) printf("        Length: %d\n", N);
  }
  else {
     (void) printf("Single Sequence: %s\n", p0->nomProt);
     (void) printf("         Length: %d\n", M);
  }

#endif

  /* !!!! ATTENTION: SIM commence la sequence a 1 et finit a M,N */
  /* OLD if ( nseq == 2 ), pas besoin si automode p0==p1 et nseq==1 */

  /* K ne peut pas etre nul ou negatif, si nul ou negatif cela veut dire qu'on */
  /* compte sur min score ou longueur min pour sortir de la boucle,  */
  /* donc il faut prevoir un grand nombre de repetitions de maniere a  */
  /* pouvoir toutes les sortir:        */
  /* on fixe ce nombre au max des 2 nombres : 100 ou (taille seq_max/10))      */
  if (K <= 0) {
     K = 100;
     if (K < M/5)
        K = M/5;
     if (K < N/5)
        K = N/5;
     Kwanted = K; /* great number of wanted matches */
  }
  else {
     Kwanted = K; /* real number of wanted matches */
     /* we must increment K because of restarts needed */
     K = 2*K > 10 ? 2*K : 10;
  }

  SIM(p0->tAngles-1,p1->tAngles-1,M,N,K,q,r,nseq,Igscore,mat_thresh, SPN, p0, p1, minlen, minscore, minrms, vverbose, elongation, rrmsd, which_rmsd, swres, taux_recouv, cross_rms_seuil, proba_limite, water_ving,Kwanted);

  /* free all statics */
  FreeAllStatics(K,M);

  return swres;
}

/* ------------------------------------------------------------------------- */
static int sQ, sR;			/* gap penalties */
static int sQR;				/* sQR = sQ + sR */

typedef struct ONE { int COL ;  struct ONE  *NEXT ;} pair, *pairptr;
pairptr *sROW, z; 			/* for saving used aligned pairs */
static short tt;

typedef struct NODE
	{ int  SCORE;
	  int  STARI;
	  int  STARJ;
	  int  ENDI;
	  int  ENDJ;
	  int  TOP;
	  int  BOT;
	  int  LEFT;
	  int  RIGHT; }  vertex, *vertexptr;
		
vertexptr  *LIST;			/* an array for saving k best scores */
vertexptr  sLOW = 0;			/* lowest score node in LIST */
vertexptr  sMOST = 0;			/* latestly accessed node in LIST */
static int sNUMNODE;			/* the number of nodes in LIST */

static int *CC, *DD;			/* saving matrix scores */
static int *RR, *SS, *EE, *FF;	 	/* saving start-points */
static int *HH, *WW;		 	/* saving matrix scores */
static int *II, *JJ, *XX, *YY;	 	/* saving start-points */
static int  sM1, sMM, sN1, sNN;		/* boundaries of recomputed area */
static int  sRL, sCL;			/* left and top boundaries */
static int  sMIN;			/* minimum score in LIST */
static short sFlag;			/* indicate if recomputation necessary*/

/* DIAG() assigns value to x if (ii,jj) is never used before */
#define DIAG(ii, jj, x, value)				\
{ for ( tt = 1, z = sROW[(ii)]; z != 0; z = z->NEXT )	\
    if ( z->COL == (jj) )				\
      { tt = 0; break; }				\
  if ( tt )						\
    x = ( value );					\
}

/* replace (ss1, xx1, yy1) by (ss2, xx2, yy2) if the latter is large */
#define ORDER(ss1, xx1, yy1, ss2, xx2, yy2)		\
{ if ( ss1 < ss2 )					\
    { ss1 = ss2; xx1 = xx2; yy1 = yy2; }		\
  else							\
    if ( ss1 == ss2 )					\
      { if ( xx1 < xx2 )				\
	  { xx1 = xx2; yy1 = yy2; }			\
	else						\
	  if ( xx1 == xx2 && yy1 < yy2 )		\
	    yy1 = yy2;					\
      }							\
}

/* The following definitions are for function diff() */

static int  zero = 0;				/* int type zero        */

#define gap(k)  ((k) <= 0 ? 0 : sQ+sR*(k))	/* k-symbol indel score */

static int sNo_mat,sNo_mis;
static int *sApp;				/* Current script append ptr */
static int  sLast;				/* Last script op appended */

static int sI, sJ;				/* current positions of A ,B */
static int sAl_len; 				/* length of alignment */
						/* Append "Delete k" op */
#define DEL(k)				\
{ sI += k;				\
  sAl_len += k;				\
  if (sLast < 0)				\
    sLast = sApp[-1] -= (k);		\
  else					\
    sLast = *sApp++ = -(k);		\
}
						/* Append "Insert k" op */
#define INS(k)				\
{ sJ += k;				\
  sAl_len += k;				\
  if (sLast < 0)				\
    { sApp[-1] = (k); *sApp++ = sLast; }	\
  else					\
    sLast = *sApp++ = (k);		\
}

						/* Append "Replace" op */
#define REP 				\
{ sLast = *sApp++ = 0; 			\
  sAl_len += 1;				\
}

/* ------------------------------------------------------------------------- */
static void ResetStaticVars(void)
{
   match = mismh = 0;
   CC = DD = RR = SS = EE = FF = HH = WW = II = JJ = XX = YY = NULL;
   LIST = NULL;
   sROW = NULL;
   sLOW = 0;
   sMOST = 0;                   /* latestly accessed node in LIST */
   sFlag = 0;                   /* indicate if recomputation necessary*/

   return;
}

/* ------------------------------------------------------------------------- */
void FreeAllStatics(int K, int M)
{
   int i;
   pairptr z, suiv;
   

   MYFREE(CC);
   MYFREE(DD);
   MYFREE(RR);
   MYFREE(SS);
   MYFREE(EE);
   MYFREE(FF);
   MYFREE(HH);
   MYFREE(WW);
   MYFREE(II);
   MYFREE(JJ);
   MYFREE(XX);
   MYFREE(YY);

   for ( i = 0; i < K ; i++ )
      MYFREE(LIST[i]);
   MYFREE(LIST);

   for ( i = 1; i <= M; i++ ) {
      if (sROW[i] != 0) {
         z = sROW[i];
         /* MYFREE(z); */
         while (z) {
           suiv = z->NEXT;
           MYFREE(z);
           z = suiv;
         }
         sROW[i] = 0;
      }
   }
/*    for ( i = 1; i <= M; i++ ) {
      if (sROW[i] != 0) {
         z = sROW[i];
         MYFREE(z);
*/
/*
         while (z) {
           suiv = z->NEXT;
           MYFREE(z);
           z = suiv;
         }
 */
/*     }
   }
*/
   MYFREE(sROW);
 
   return;
}
/* ------------------------------------------------------------------------- */
/* SIM(A,B,M,N,K,Q,R,nseq) reports K best non-intersecting alignments of
   the segments of A and B in order of similarity scores, where
   v[a][b] is the score of aligning a and b, and -(Q+R*i) is the score
   of an i-symbol indel.  						*/

static void SIM(int *A,int *B,int M,int N,int K,int Q,int R,int nseq,int (*Igscore)(int a1, int a2, void *mat_thresh), void *mat_thresh, int SPN, tyProt *p0, tyProt *p1, int minlen, float minscore,float minrms, int vverbose, int elongation,float rrmsd, int which_rmsd, SWResult *swres, float taux_recouv, float cross_rms_seuil, float proba_limite, WV *water_ving, int Kwanted)
{ int              endi, endj, stari, starj;	/* endpoint and startpoint */ 
  int              score;   			/* the max score in LIST */
  int              count;			/* maximum size of list */	
  int              last_len_found;		/* maximum size of list */	
  register int     i, j;			/* row and column indices */
  int              *S;				/* saving operations for diff */
  vertexptr        cur; 			/* temporary pointer */
  vertexptr        findmax();	 		/* return the largest score node */
  double           rms = 0.0L;
  float            reverse_rms=0.0L, fscore=0.0L, fscore_blosangle=0.0L;
  int              posd0=0,posd1=0,posf0=0,posf1=0,nresult = 0, limcount;
  DtMatrix4x4      matrix4x;
  char             *salign = NULL;
  float            recover = 0.0; /* taux de recouvrement  */
  float            proba = 1.0, last_score_found;  /* attention last_score_found est exprime en score/ADN_MULT */
  COORDS               **pp0 = NULL,**pp1 = NULL;
  int                  *lensub = NULL, nbpp = 0;
	
  if (SPN == STRUCTURE){
     /* for structures				*/
     /* allocate for coords for rms             */
     lensub = NEWN(int, INCPP);
     pp0 = NEWN(COORDS *, INCPP);
     pp1 = NEWN(COORDS *, INCPP);

     if (!lensub || !pp0 || !pp1)
           MyError(stderr,3,"SIM:: not enough memory for RMS computations (pp0,pp1)\n");
     nbpp = INCPP;
  }
   /* allocate space for all vectors */
   j = (N + 1);
   NEWY(CC, int, j);
   NEWY(DD, int, j);
   NEWY(RR, int, j);
   NEWY(SS, int, j);
   NEWY(EE, int, j);
   NEWY(FF, int, j);
   i = (M + 1);
   NEWY(HH, int, i);
   NEWY(WW, int, i);
   NEWY(II, int, i);
   NEWY(JJ, int, i);
   NEWY(XX, int, i);
   NEWY(YY, int, i);
   NEWY(S, int, i+j);
   /* sROW = ( pairptr * ) ckalloc( (M + 1) * sizeof(pairptr)); OLD JOMPO */
   NEWY(sROW, pairptr, M+1);

   if (! (salign = NEWN(char, p0->nbRes+p1->nbRes)) )
            MyError(stderr, 2, "SIM: not enough memory for salign\n");

   /* set up list for each row */
   for ( i = 1; i <= M; i++ ) {
      if ( nseq == 2 )
         sROW[i] = 0;
      else {
	 /* sROW[i] = z = ( pairptr ) ckalloc( (int) sizeof(pair)); OLD JOMPO */
	 NEWY(sROW[i], pair, 1);
	 z = sROW[i];
         z->COL = i;			
         z->NEXT = 0;
      }
   }
   sQ = Q;
   sR = R;
   sQR = sQ + sR;


   /* LIST = ( vertexptr * ) ckalloc( K * sizeof(vertexptr)); OLD JOMPO */
   NEWY(LIST, vertexptr, K);
   for ( i = 0; i < K ; i++ )
      /* LIST[i] = ( vertexptr ) ckalloc( (int) sizeof(vertex)); OLD JOMPO */
      NEWY(LIST[i], vertex, 1);

   if (vverbose)
      fprintf(stderr, "--> K = %d Kwanted = %d\n",K,Kwanted);
   sNUMNODE = sMIN = 0;
   big_pass(A,B,M,N,K,nseq,Igscore,mat_thresh);


   /* Report the Kwanted best alignments one by one. After each alignment is
       output, recompute part of the matrix. First determine the size
       of the area to be recomputed, then do the recomputation         */
   nresult = 0; /* nb results */
   last_len_found = minlen + 1;       /* dummy to begin loop */
   last_score_found = minscore + 1;   /* dummy to begin loop */
   count = K-1;
   limcount = K-Kwanted;
   while ( count >= limcount && (minlen <= 0 || last_len_found >= minlen) &&
           (minscore <= 0.0 || last_score_found >= minscore) ) {
      /* OLD for ( count = K - 1; count >= 0 ; count-- ) { */
      if ( sNUMNODE == 0 ) {
	 MyError(stderr,0,"There are no %d alignments between two sequences", Kwanted);
         break;
      }
      cur = findmax();	/* Return a pointer to a node with max score*/
      if (vverbose)
         fprintf(stderr, "--> match count = %d score = %d\n",K-count,cur->SCORE);
      score = cur->SCORE;
      if (SPN==NUCLEIQUE)
         last_score_found = (float)score/ADN_MULT;
      else
         last_score_found = (float)score;
      stari = ++cur->STARI;
      starj = ++cur->STARJ;
      endi = cur->ENDI;
      endj = cur->ENDJ;
      sM1 = cur->TOP;
      sMM = cur->BOT;
      sN1 = cur->LEFT;
      sNN = cur->RIGHT;
      sRL = endi - stari + 1;
      sCL = endj - starj + 1;
      sI = stari - 1;
      sJ = starj - 1;
      sApp = S;
      sLast = 0;
      sAl_len = 0;
      /* raz nb match/mismatch */
      sNo_mat = sNo_mis = 0;
      (void) diff(&A[stari]-1, &B[starj]-1,sRL,sCL,sQ,sQ,Igscore,mat_thresh);
      /* !!!! ATTENTION: SIM commence la sequence a 1 et finit a M,N */
      /* donc mettre posd0 et posd1 a stari-1 et starj-1 */
      posd0=stari-1;
      posd1=starj-1;
      /* donc aussi mettre posf0 et posf1 a endi-1 et endj-1 */
      posf0=endi-1;
      posf1=endj-1;
      /* length of real alignment, i.e. nb matches+nb mismatches	*/
      last_len_found = sNo_mat+sNo_mis;
      if (nseq == 1) {	/* if automode */
         recover = 0.0;
         if ((posd0 > posd1) && (posd0 < posf1) ) {
               recover = (float)((posf1)-(posd0))/(float)(sNo_mat+sNo_mis);
         }
         else if ((posf0 > posd1) && (posf0 < posf1) ) {
               recover = (float)((posf0)-(posd1))/(float)(sNo_mat+sNo_mis);
         }
      }
      /* keep alignment only if it satify conditions */
      if ((nseq==2 || recover <= taux_recouv) &&  (minlen <= 0 || last_len_found >= minlen) &&
            (minscore <= 0.0 || last_score_found >= minscore) ) {
#ifdef PRINT_OLD
         /* Output the best alignment */
         (void) printf("\n*********************************************************\n");
         (void) printf("      Number %d Local Alignment\n", K - count);
         (void) printf("      Similarity Score : %d\n",last_score_found);
         (void) printf("      Match Percentage : %d%%\n", (100*sNo_mat)/sAl_len);
         (void) printf("      Number of Matches : %d\n", sNo_mat);
         (void) printf("      Number of Mismatches : %d\n", sNo_mis);
         (void) printf("      Total Length of Gaps : %d\n", sAl_len-sNo_mat-sNo_mis);
         (void) printf("      Begins at (%d, %d) and Ends at (%d, %d)\n",
	                stari,starj, endi,endj);
         if (SPN != STRUCTURE)
            display(&A[stari]-1,&B[starj]-1,sRL,sCL,S,stari,starj);
#endif
         /* remplir salign */
         fill_salign(sRL, sCL, S, salign);
         /* fprintf(stderr, "salign:\n%s\n\n",salign); */
/*         PrintAliSeq(stdout, p0, posd0, salign, INSER);
         PrintAliSeq(stdout, p1, posd1, salign, DELET);
 */
         /* JOMPO A FINIR */
         (void) fflush(stdout);
         /* fill up swres	*/
         if (SPN == STRUCTURE){
            fscore = (float)last_score_found;
            fscore_blosangle = 0.0; /* NOT USED */
            /* AddSWResult est appele dans calculRMS */
            /* nresult est aussi incremente si le resultat est garde */
#undef DEBJOMPO
#ifdef DEBJOMPO
fprintf(stderr,"avant calculRMS: pp0=%X\n",(unsigned int)pp0);
#endif
            if (calculRMS(&nbpp, &lensub, &pp0, &pp1, p0, p1, salign, &rms, &reverse_rms , swres, &fscore, &fscore_blosangle, recover, &posd0, &posd1, posf0, posf1, matrix4x, &nresult, minscore , minrms, vverbose, elongation, rrmsd, which_rmsd, nseq) == 0)
                continue;
#ifdef DEBJOMPO
fprintf(stderr,"nbpp=%d nresult=%d\n",nbpp,nresult);
fprintf(stderr,"calculRMS: pp0=%X pp0[%d]=%X\n",(unsigned int)pp0, (nresult-1),(unsigned int)(pp0[(nresult-1)]));
#endif
         }
         else {
            /* JOMPO A FINIR CALCULER LA PROBA */
            if (water_ving != NULL) {
               if ((proba = proba_water_ving(water_ving, last_score_found, nresult) )< 0)
                  fprintf(stderr," probleme proba waterman_vingron\n");
            }
            else {
               proba = 1.0;
            }
            /* attention la proba est 1-p-value c'est pourquoi on teste si < proba_limite */
            if (nresult >= NB_SCORES_MAX || (minscore > 0.0 && last_score_found < minscore) ||
                  (water_ving != NULL && proba < proba_limite) ||
                  (minlen > 0 && last_len_found < minlen) ) {
               fprintf(stderr, "break: limit score, probability or length given is reached\n");
               break;
            }
            AddSWResult(swres, last_score_found, proba, -1.0, recover, posd0, posd1, posf0, posf1, salign, NULL, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
            nresult++;
            if (vverbose)
               fprintf(stderr,
                 "added match (%4d,%4d) (%4d,%4d) (score=%.2f len=%d)\n",
                  posd0,posf0,posd1,posf1,last_score_found,last_len_found);
         }
      } /* end if recover... */
      /* fprintf(stderr, "BOUCLE--->nresult=%d swres->nmatch=%d\n",nresult, swres->nmatch);  */

      /* do not compute next pass if we have already Kwanted best scores */
      if ( count-limcount ) { /* old if (count) */
                sFlag = 0;
                locate(A,B,nseq,Igscore,mat_thresh);
                if ( sFlag )
		   small_pass(A,B,count,nseq,Igscore,mat_thresh);
      }

      /* on passe le tour si taux recouvrement pas ok */
      if (nseq == 1 && recover > taux_recouv) {
         if (vverbose)
            fprintf(stderr,
               "alignment discarded too much overlap (%.3f), restarting... (%4d,%4d) (%4d,%4d) (score=%f len=%d)\n",
               recover,posd0,posf0,posd1,posf1,last_score_found,last_len_found);
         count++;
      }

      count--;
   } /* end while count... */

   if (vverbose) {
      if (minlen > 0 && last_len_found < minlen)
        fprintf(stderr,"sim: break cause length (%d) < minlength (%d)\n",last_len_found,minlen);
      else if (minscore >  0.0 && last_score_found < minscore)
        fprintf(stderr,"sim: break cause score (%f) < minscore (%f)\n",last_score_found,minscore);
      else if (water_ving != NULL && proba < proba_limite)
        fprintf(stderr,"sim: break cause proba (%f) < proba limit (%f)\n",proba,proba_limite);
      else if (count < limcount)
        fprintf(stderr,"sim: break cause number of matches reached\n");
      else if (sNUMNODE == 0)
        fprintf(stderr,"sim: break cause no more matches\n");
      else
        fprintf(stderr,"sim: break unknown cause !!!\n");
   }

   free(salign);

   if (SPN == STRUCTURE && swres->nmatch >= 1 ) {
     /* compute crossed RMS over an Swres	*/
     /* debug jompo if (! (swres->crossrms = newcrossrmstab(swres->nmatch)) )  */
     if (! (swres->crossrms = NEWN(float *,swres->nmatch)) )
       MyError(stderr,3,"swalign:: not enough memory for crossed rms\n");
     
     for (i=0;i<swres->nmatch; i++) {
       /* debug jompo if (! (swres->crossrms[i] = newcrossrms(swres->nmatch)) )  */
       if (! (swres->crossrms[i] = NEWN(float,swres->nmatch)) )
	 MyError(stderr,3,"swalign:: not enough memory for crossed rms\n");
     }
     
     for (i = 0; i < swres->nmatch ; i++) {
       for (j = 0; j < swres->nmatch; j++) {
	 
	 /* rms_error returns the RMS with the	*/
	 /* actual RT matrix			*/
	 if (j==i) {
	   swres->crossrms[i][i] = swres->rms[i];
         }
	 else {
#ifdef DEBJOMPO
fprintf(stderr,"pp0[%d]=%X len=%d\n",j,(unsigned int)(pp0[(j)]),lensub[j]);
#endif
	   swres->crossrms[i][j] = (float)sqrt(zuker_superpose(pp0[j], pp1[j], lensub[j], swres->matrix4x[i]));
         }
       }
     }

     /* debug jompo if (! (swres->group = newgroup(swres->nmatch)) ) */
     if (! (swres->group = NEWN(int, swres->nmatch)) )
       MyError(stderr,3,"swalign:: not enough memory for group\n");
     
     /* compute group of compatibles	*/
     makegroup(0, swres->nmatch, swres->crossrms, swres->group, &(swres->ngroup), cross_rms_seuil);
     /* fprintf(stderr, "nombre groupe = %d\n",swres->ngroup); */
     
   }
   else {
     
     swres->crossrms = NULL;
     swres->group = NULL;
   }

   /* fprintf(stderr, "--->nresult=%d swres->nmatch=%d\n",nresult, swres->nmatch);   */
   if(SPN == STRUCTURE){
     /* free only pp's which have been allocated */
     for (i = 0; i < swres->nmatch; i++) {
        /* fprintf(stderr, "--->nresult=%d i=%d\n",nresult, i);   */
        if(pp0[i])
            free(pp0[i]);
       if(pp1[i])
           free(pp1[i]);
     }
     free(pp0);
     free(pp1);
     free(lensub);
   }
   
   MYFREE(S);

   return;
}

/* ------------------------------------------------------------------------- */
/* A big pass to compute K best classes */

static void big_pass(int *A,int *B, int M, int N, int K, int nseq, int (*Igscore)(int a1, int a2, void *mat_thresh), void *mat_thresh)
{ register  int  i, j;			/* row and column indices */
  register  int  c;			/* best score at current point */
  register  int  f;			/* best score ending with insertion */
  register  int  d;			/* best score ending with deletion */
  register  int  p;			/* best score at (i-1, j-1) */
  register  int  ci, cj;		/* end-point associated with c */ 
  register  int  di, dj;		/* end-point associated with d */
  register  int  fi, fj;		/* end-point associated with f */
  register  int  pi, pj;		/* end-point associated with p */
  /* int  *va;*/				/* pointer to v(A[i], B[j]) */
  int   addnode();			/* function for inserting a node */

	
	/* Compute the matrix and save the top K best scores in LIST
	   CC : the scores of the current row
	   RR and EE : the starting point that leads to score CC
	   DD : the scores of the current row, ending with deletion
	   SS and FF : the starting point that leads to score DD        */
 	/* Initialize the 0 th row */
	for ( j = 1; j <= N ; j++ ) {
             CC[j] = 0;
	     RR[j] = 0;
	     EE[j] = j;
	     DD[j] = - (sQ);
	     SS[j] = 0;
	     FF[j] = j;
	}
	for ( i = 1; i <= M; i++) {
             c = 0;				/* Initialize column 0 */
	     f = - (sQ);
	     ci = fi = i;
	     /* va = v[A[i]]; OLD JOMPO */
	     if ( nseq == 2 ) {
                 p = 0;
	         pi = i - 1;
	         cj = fj = pj = 0;
	     }
	     else {
                 p = CC[i];
		 pi = RR[i];
		 pj = EE[i];
	         cj = fj = i;
	     }
	     for ( j = (nseq == 2 ? 1 : (i+1)) ; j <= N ; j++ )  {
                  f = f - sR;
		  c = c - sQR;
		  ORDER(f, fi, fj, c, ci, cj)
		  c = CC[j] - sQR; 
		  ci = RR[j];
		  cj = EE[j];
		  d = DD[j] - sR;
		  di = SS[j];
		  dj = FF[j];
		  ORDER(d, di, dj, c, ci, cj)
		  c = 0;
#if 0
fprintf(stderr,"i=%4d j=%4d (%5d,%5d) score %4d\n", i,j, A[i],B[j], Igscore(A[i],B[j],mat_thresh));
fflush(stderr);
#endif
		  DIAG(i, j, c, p+Igscore(A[i],B[j],mat_thresh))	/* diagonal */
		  if ( c <= 0 )
		    { c = 0; ci = i; cj = j; }
		  else
		    { ci = pi; cj = pj; }
		  ORDER(c, ci, cj, d, di, dj)
		  ORDER(c, ci, cj, f, fi, fj)
		  p = CC[j];
		  CC[j] = c;
		  pi = RR[j];
		  pj = EE[j];
		  RR[j] = ci;
		  EE[j] = cj;
		  DD[j] = d;
		  SS[j] = di;
		  FF[j] = dj;
		  if ( c > sMIN ) {	/* add the score into list */
#if 0
fprintf(stderr,"i=%4d j=%4d max %4d sMIN %4d\n", i,j,c,sMIN);
fflush(stderr);
#endif
		    sMIN = addnode(c, ci, cj, i, j, K, sMIN);
                 }
	     }
         }
}

/* ------------------------------------------------------------------------- */
/* Determine the left and top boundaries of the recomputed area */

static void locate(int *A,int *B,int nseq, int (*Igscore)(int a1, int a2, void *mat_thresh), void *mat_thresh)
/* OLD JOMPO  char A[],B[]; int nseq; */
{ register  int  i, j;			/* row and column indices */
  register  int  c;			/* best score at current point */
  register  int  f;			/* best score ending with insertion */
  register  int  d;			/* best score ending with deletion */
  register  int  p;			/* best score at (i-1, j-1) */
  register  int  ci, cj;		/* end-point associated with c */ 
  register  int  di, dj;		/* end-point associated with d */
  register  int  fi, fj;		/* end-point associated with f */
  register  int  pi, pj;		/* end-point associated with p */
  short  cflag, rflag;			/* for recomputation */
  /* int  *va;	*/			/* pointer to v(A[i], B[j]) */
  int   addnode();			/* function for inserting a node */
  int  limit;				/* the bound on j */

  di = 0; dj = 0;
	/* Reverse pass
	   rows
	   CC : the scores on the current row
	   RR and EE : the endpoints that lead to CC
	   DD : the deletion scores 
	   SS and FF : the endpoints that lead to DD

	   columns
	   HH : the scores on the current columns
	   II and JJ : the endpoints that lead to HH
	   WW : the deletion scores
	   XX and YY : the endpoints that lead to WW
	*/
	for ( j = sNN; j >= sN1 ; j-- )
          {  CC[j] = 0;
	     EE[j] = j;
	     DD[j] = - (sQ);
	     FF[j] = j;
	     if ( nseq == 2 || j > sMM )
                RR[j] = SS[j] = sMM + 1;
	     else
                RR[j] = SS[j] = j;
	  }

        for ( i = sMM; i >= sM1; i-- )
	  {  c = p = 0;
	     f = - (sQ);
	     ci = fi = i;
	     pi = i + 1;
	     cj = fj = pj = sNN + 1;
	     /* va = v[A[i]]; OLD JOMPO */
	     if ( nseq == 2 || sN1 > i )
		limit = sN1;
	     else
		limit = i + 1;
	     for ( j = sNN; j >= limit ; j-- )  
	       {  f = f - sR;
		  c = c - sQR;
		  ORDER(f, fi, fj, c, ci, cj)
		  c = CC[j] - sQR; 
		  ci = RR[j];
		  cj = EE[j];
		  d = DD[j] - sR;
		  di = SS[j];
		  dj = FF[j];
		  ORDER(d, di, dj, c, ci, cj)
		  c = 0;
		  DIAG(i, j, c, p+Igscore(A[i],B[j],mat_thresh))		/* diagonal */
		  if ( c <= 0 )
		    { c = 0; ci = i; cj = j; }
		  else
		    { ci = pi; cj = pj; }
		  ORDER(c, ci, cj, d, di, dj)
		  ORDER(c, ci, cj, f, fi, fj)
		  p = CC[j];
		  CC[j] = c;
		  pi = RR[j];
		  pj = EE[j];
		  RR[j] = ci;
		  EE[j] = cj;
		  DD[j] = d;
		  SS[j] = di;
		  FF[j] = dj;
		  if ( c > sMIN )
		    sFlag = 1;
	        }
	     if ( nseq == 2 || i < sN1 )
	       { HH[i] = CC[sN1];
	         II[i] = RR[sN1];
	         JJ[i] = EE[sN1];
	         WW[i] = f;
	         XX[i] = fi;
	         YY[i] = fj;
	       }
	  }
      
  for ( sRL = sM1, sCL = sN1; ; ) {
      for ( rflag = cflag = 1; ( rflag && sM1 > 1 ) || ( cflag && sN1 > 1 ) ;  ) {
#if 0
          fprintf(stderr,"locate: LIST[0]=%x sM1=%d sN1=%d\n",(unsigned int)(LIST[0]),sM1,sN1);
          fflush(stderr);
#endif
          if ( rflag && sM1 > 1 ) {	/* Compute one row */
              rflag = 0;
	      sM1--;
      	      c = p = 0;
	      f = - (sQ);
	      ci = fi = sM1;
	      pi = sM1 + 1;
	      cj = fj = pj = sNN + 1;
	      /* va = v[A[sM1]]; OLD JOMPO */
	      for ( j = sNN; j >= sN1 ; j-- ) {
	          f = f - sR;
		  c = c - sQR;
		  ORDER(f, fi, fj, c, ci, cj)
		  c = CC[j] - sQR; 
		  ci = RR[j];
		  cj = EE[j];
		  d = DD[j] - sR;
		  di = SS[j];
		  dj = FF[j];
		  ORDER(d, di, dj, c, ci, cj)
		  c = 0;
		  DIAG(sM1, j, c, p+Igscore(A[sM1],B[j],mat_thresh))		/* diagonal */
		  if ( c <= 0 )
		    { c = 0; ci = sM1; cj = j; }
		  else
		    { ci = pi; cj = pj; }
		  ORDER(c, ci, cj, d, di, dj)
		  ORDER(c, ci, cj, f, fi, fj)
		  p = CC[j];
		  CC[j] = c;
		  pi = RR[j];
		  pj = EE[j];
		  RR[j] = ci;
		  EE[j] = cj;
		  DD[j] = d;
		  SS[j] = di;
		  FF[j] = dj;
		  if ( c > sMIN )
		     sFlag = 1;
		  if ( ! rflag && ( (ci > sRL && cj > sCL) || (di > sRL && dj > sCL)
	 		                            || (fi > sRL && fj > sCL) ) )
		      rflag = 1;
	      }
	      HH[sM1] = CC[sN1];
	      II[sM1] = RR[sN1];
	      JJ[sM1] = EE[sN1];
	      WW[sM1] = f;
	      XX[sM1] = fi;
	      YY[sM1] = fj;
	      if ( ! cflag && ( (ci > sRL && cj > sCL) || (di > sRL && dj > sCL)
			     || (fi > sRL && fj > sCL) ) )
	         cflag = 1;
	}

	if ( nseq == 1 && sN1 == (sM1 + 1) && ! rflag )
	     cflag = 0;
	if ( cflag && sN1 > 1 )	{ /* Compute one column */
	      cflag = 0;
	      sN1--;
	      c = 0;
	      f = - (sQ);
	      cj = fj = sN1;
	      /* va = v[B[sN1]]; OLD JOMPO */
	      if ( nseq == 2 || sMM < sN1 ) {
		  p = 0;
	          ci = fi = pi = sMM + 1;
	          pj = sN1 + 1;
		  limit = sMM;
              }
	      else {
		  p = HH[sN1];
		  pi = II[sN1];
		  pj = JJ[sN1];
	          ci = fi = sN1;
		  limit = sN1 - 1;
	      }
	      for ( i = limit; i >= sM1 ; i-- ) {
	          f = f - sR;
		  c = c - sQR;
		  ORDER(f, fi, fj, c, ci, cj)
		  c = HH[i] - sQR; 
		  ci = II[i];
		  cj = JJ[i];
		  d = WW[i] - sR;
		  di = XX[i];
		  dj = YY[i];
		  ORDER(d, di, dj, c, ci, cj)
		  c = 0;
	          DIAG(i, sN1, c, p+Igscore(B[sN1],A[i],mat_thresh))
		  if ( c <= 0 )
		    { c = 0; ci = i; cj = sN1; }
		  else
		    { ci = pi; cj = pj; }
		  ORDER(c, ci, cj, d, di, dj)
		  ORDER(c, ci, cj, f, fi, fj)
		  p = HH[i];
		  HH[i] = c;
		  pi = II[i];
		  pj = JJ[i];
		  II[i] = ci;
		  JJ[i] = cj;
		  WW[i] = d;
		  XX[i] = di;
		  YY[i] = dj;
		  if ( c > sMIN )
		     sFlag = 1;
	          if ( ! cflag && ( (ci > sRL && cj > sCL) || (di > sRL && dj > sCL)
		               || (fi > sRL && fj > sCL) ) )
		     cflag = 1;
	      }
	      CC[sN1] = HH[sM1];
	      RR[sN1] = II[sM1];
	      EE[sN1] = JJ[sM1];
	      DD[sN1] = f;
	      SS[sN1] = fi;
	      FF[sN1] = fj;
	      if ( ! rflag && ( (ci > sRL && cj > sCL) || (di > sRL && dj > sCL)
		                                 || (fi > sRL && fj > sCL) ) )
	         rflag = 1;
	    }
	}
      if ( (sM1 == 1 && sN1 == 1) || no_cross() )
	 break;
   }
  sM1--;
  sN1--;
}

/* ------------------------------------------------------------------------- */
/* recompute the area on forward pass */
static void small_pass(int *A, int *B, int count, int nseq, int (*Igscore)(int a1, int a2, void *mat_thresh), void *mat_thresh)
{ register  int  i, j;			/* row and column indices */
  register  int  c;			/* best score at current point */
  register  int  f;			/* best score ending with insertion */
  register  int  d;			/* best score ending with deletion */
  register  int  p;			/* best score at (i-1, j-1) */
  register  int  ci, cj;		/* end-point associated with c */ 
  register  int  di, dj;		/* end-point associated with d */
  register  int  fi, fj;		/* end-point associated with f */
  register  int  pi, pj;		/* end-point associated with p */
  /* int  *va;	*/			/* pointer to v(A[i], B[j]) */
  int   addnode();			/* function for inserting a node */
  int  limit;				/* lower bound on j */

	for ( j = sN1 + 1; j <= sNN ; j++ )
	  {  CC[j] = 0;
	     RR[j] = sM1;
	     EE[j] = j;
	     DD[j] = - (sQ);
	     SS[j] = sM1;
	     FF[j] = j;
	  }
	for ( i = sM1 + 1; i <= sMM; i++) 
	  {  c = 0;				/* Initialize column 0 */
	     f = - (sQ);
	     ci = fi = i;
	     /* va = v[A[i]]; OLD JOMPO */
	     if ( nseq == 2 || i <= sN1 )
	       { p = 0;
	         pi = i - 1;
	         cj = fj = pj = sN1;
		 limit = sN1 + 1;
	       }
	     else
	       { p = CC[i];
		 pi = RR[i];
		 pj = EE[i];
	         cj = fj = i;
		 limit = i + 1;
	       }
	     for ( j = limit ; j <= sNN ; j++ )  
	       {  f = f - sR;
		  c = c - sQR;
		  ORDER(f, fi, fj, c, ci, cj)
		  c = CC[j] - sQR; 
		  ci = RR[j];
		  cj = EE[j];
		  d = DD[j] - sR;
		  di = SS[j];
		  dj = FF[j];
		  ORDER(d, di, dj, c, ci, cj)
		  c = 0;
		  DIAG(i, j, c, p+Igscore(A[i],B[j],mat_thresh))		/* diagonal */
		  if ( c <= 0 )
		    { c = 0; ci = i; cj = j; }
		  else
		    { ci = pi; cj = pj; }
		  ORDER(c, ci, cj, d, di, dj)
		  ORDER(c, ci, cj, f, fi, fj)
		  p = CC[j];
		  CC[j] = c;
		  pi = RR[j];
		  pj = EE[j];
		  RR[j] = ci;
		  EE[j] = cj;
		  DD[j] = d;
		  SS[j] = di;
		  FF[j] = dj;
		  if ( c > sMIN )	/* add the score into list */
		    sMIN = addnode(c, ci, cj, i, j, count, sMIN);
	        }
	  }
}

/* ------------------------------------------------------------------------- */
/* Add a new node into list.  */

int addnode(int c, int ci, int cj, int i, int j, int K, int cost)
{
  short found;				/* 1 if the node is in LIST */
  register int d;

  found = 0;
  if ( sMOST != 0 && sMOST->STARI == ci && sMOST->STARJ == cj )
    found = 1;
  else
     for ( d = 0; d < sNUMNODE ; d++ )
	{ sMOST = LIST[d];
#if 0
fprintf(stderr,"LIST[%4d]->SCORE=%d\n",d,LIST[d]->SCORE);
#endif
	  if ( sMOST->STARI == ci && sMOST->STARJ == cj )
	    { found = 1;
	      break;
	    }
        }
  if ( found )
    { if ( sMOST->SCORE < c )
        { sMOST->SCORE = c;
          sMOST->ENDI = i;
          sMOST->ENDJ = j;
        }
      if ( sMOST->TOP > i ) sMOST->TOP = i;
      if ( sMOST->BOT < i ) sMOST->BOT = i;
      if ( sMOST->LEFT > j ) sMOST->LEFT = j;
      if ( sMOST->RIGHT < j ) sMOST->RIGHT = j;
    }
  else
    { if ( sNUMNODE == K )	/* list full */
	 sMOST = sLOW;
      else
         sMOST = LIST[sNUMNODE++];
      sMOST->SCORE = c;
      sMOST->STARI = ci;
      sMOST->STARJ = cj;
      sMOST->ENDI = i;
      sMOST->ENDJ = j;
      sMOST->TOP = sMOST->BOT = i;
      sMOST->LEFT = sMOST->RIGHT = j;
    }
  if ( sNUMNODE == K )
    { if ( sLOW == sMOST || ! sLOW ) 
        { for ( sLOW = LIST[0], d = 1; d < sNUMNODE ; d++ )
            if ( LIST[d]->SCORE < sLOW->SCORE )
              sLOW = LIST[d];
	}
      return ( sLOW->SCORE ) ;
    }
  else
    return cost;
}

/* ------------------------------------------------------------------------- */
/* Find and remove the largest score in list */

vertexptr findmax()
{ vertexptr  cur;
  register int i, j;

  for ( j = 0, i = 1; i < sNUMNODE ; i++ )
    if ( LIST[i]->SCORE > LIST[j]->SCORE )
       j = i;
  cur = LIST[j];
  if ( j != --sNUMNODE )
    { LIST[j] = LIST[sNUMNODE];
      LIST[sNUMNODE] =  cur;
    }
  sMOST = LIST[0];
  if ( sLOW == cur ) sLOW = LIST[0];
  return ( cur );
}

/* ------------------------------------------------------------------------- */
/* return 1 if no node in LIST share vertices with the area */

static int no_cross(void)
{ vertexptr  cur;
  register int i;
      for ( i = 0; i < sNUMNODE; i++ ) { 
#if 0
          fprintf(stderr,"LIST[%d]=%x\n",i,(unsigned int)(LIST[i]));
          fflush(stderr);
#endif
          cur = LIST[i];
	  if ( cur->STARI <= sMM && cur->STARJ <= sNN && cur->BOT >= sM1-1 && 
	       cur->RIGHT >= sN1-1 && ( cur->STARI < sRL || cur->STARJ < sCL ))
	     { if ( cur->STARI < sRL ) sRL = cur->STARI;
	       if ( cur->STARJ < sCL ) sCL = cur->STARJ;
	       sFlag = 1;
	       break;
	     }
      }
      if ( i == sNUMNODE )
	return 1;
      else
	return 0;
}

/* ------------------------------------------------------------------------- */
/* diff(A,B,M,N,tb,te) returns the score of an optimum conversion between
   A[1..M] and B[1..N] that begins(ends) with a delete if tb(te) is zero
   and appends such a conversion to the current script.                   */

static int diff(int *A,int *B, int M, int N, int tb, int te, int (*Igscore)(int a1, int a2, void *mat_thresh), void *mat_thresh)

{ int   midi, midj, type;	/* Midpoint, type, and cost */
  int midc;

{ register int   i, j;
  register int c, e, d, s;
           int t;

/* Boundary cases: M <= 1 or N == 0 */
  if (N <= 0)
    { if (M > 0) DEL(M)
      return - gap(M);
    }
  if (M <= 1)
    { if (M <= 0)
        { INS(N);
          return - gap(N);
        }
      if (tb > te) tb = te;
      midc = - (tb + sR + gap(N) );
      midj = 0;
      /* va = v[A[1]]; */
      for (j = 1; j <= N; j++)
        {  for ( tt = 1, z = sROW[sI+1]; z != 0; z = z->NEXT )	
              if ( z->COL == j+sJ )			
	         { tt = 0; break; }		
           if ( tt )			
            { c = Igscore(A[1],B[j],mat_thresh) - ( gap(j-1) + gap(N-j) );
              if (c > midc)
               { midc = c;
                 midj = j;
               }
	    }
	}
      if (midj == 0)
        { INS(N) DEL(1) }
      else
        { if (midj > 1) INS(midj-1)
          REP
	  /* if ( A[1] == B[midj] || isdna && va[A[1],B[midj]] == MATCHSC ) OLD JOMPO */
	  if ( A[1] == B[midj])
	     sNo_mat += 1;
	  else
	     sNo_mis += 1;
	  /* mark (A[sI],B[sJ]) as used: put sJ into list sROW[sI] */	
          sI++; sJ++;
	  NEWY(z, pair, 1);
          z->COL = sJ;			
          z->NEXT = sROW[sI];				
	  sROW[sI] = z;
          if (midj < N) INS(N-midj)
        }
      return midc;
    }

/* Divide: Find optimum midpoint (midi,midj) of cost midc */

  midi = M/2;			/* Forward phase:                          */
  CC[0] = 0;			/*   Compute C(M/2,k) & D(M/2,k) for all k */
  t = -sQ;
  for (j = 1; j <= N; j++)
    { CC[j] = t = t-sR;
      DD[j] = t-sQ;
    }
  t = -tb;
  for (i = 1; i <= midi; i++)
    { s = CC[0];
      CC[0] = c = t = t-sR;
      e = t-sQ;
      /* va = v[A[i]]; OLD JOMPO */
      for (j = 1; j <= N; j++)
        { if ((c = c - sQR) > (e = e - sR)) e = c;
          if ((c = CC[j] - sQR) > (d = DD[j] - sR)) d = c;
	  DIAG(i+sI, j+sJ, c, s+Igscore(A[i],B[j],mat_thresh))
          if (c < d) c = d;
          if (c < e) c = e;
          s = CC[j];
          CC[j] = c;
          DD[j] = d;
        }
    }
  DD[0] = CC[0];

  RR[N] = 0;			/* Reverse phase:                          */
  t = -sQ;			/*   Compute R(M/2,k) & S(M/2,k) for all k */
  for (j = N-1; j >= 0; j--)
    { RR[j] = t = t-sR;
      SS[j] = t-sQ;
    }
  t = -te;
  for (i = M-1; i >= midi; i--)
    { s = RR[N];
      RR[N] = c = t = t-sR;
      e = t-sQ;
      /* va = v[A[i+1]]; OLD JOMPO */
      for (j = N-1; j >= 0; j--)
        { if ((c = c - sQR) > (e = e - sR)) e = c;
          if ((c = RR[j] - sQR) > (d = SS[j] - sR)) d = c;
	  DIAG(i+1+sI, j+1+sJ, c, s+Igscore(A[i+1],B[j+1],mat_thresh))
          if (c < d) c = d;
          if (c < e) c = e;
          s = RR[j];
          RR[j] = c;
          SS[j] = d;
        }
    }
  SS[N] = RR[N];

  midc = CC[0]+RR[0];		/* Find optimal midpoint */
  midj = 0;
  type = 1;
  for (j = 0; j <= N; j++)
    if ((c = CC[j] + RR[j]) >= midc)
      if ( c > midc || (CC[j] != DD[j] && RR[j] == SS[j]) ) {
	midc = c;
        midj = j;
      }
  for (j = N; j >= 0; j--)
    if ((c = DD[j] + SS[j] + sQ) > midc) {
	midc = c;
        midj = j;
        type = 2;
     }
}

/* Conquer: recursively around midpoint */

  if (type == 1)
    { (void) diff(A,B,midi,midj,tb,sQ,Igscore,mat_thresh);
      (void) diff(A+midi,B+midj,M-midi,N-midj,sQ,te,Igscore,mat_thresh);
    }
  else
    { (void) diff(A,B,midi-1,midj,tb,zero,Igscore,mat_thresh);
      DEL(2);
      (void) diff(A+midi+1,B+midj,M-midi-1,N-midj,zero,te,Igscore,mat_thresh);
    }
  return midc;
}

/* ------------------------------------------------------------------------- */
/* Alignment display routine */

static char ALINE[51], BLINE[51], CLINE[51];

static void display(int *A,int *B,int M,int N,int *S,int AP,int BP)
{ register char *a, *b, *c;
  register int   i,  j, op;
           int   lines, ap, bp;

  i = j = op = lines = 0;
  ap = AP;
  bp = BP;
  a = ALINE;
  b = BLINE;
  c = CLINE;
  while (i < M || j < N) {
      /* fprintf(stderr, "S--->%3d (i=%3d j=%3d)\n",*S,i,j); */
      if (op == 0 && *S == 0)
        { op = *S++;
          *a = A[++i]+'A';
          *b = B[++j]+'A';
          /* *c++ = (*a == *b || isdna && v[*a][*b] == MATCHSC ) ? '|' : ' '; OLD JOMPO */
          *c++ = (*a == *b) ? '|' : ' ';
	  a++;
	  b++;
        }
      else
        { if (op == 0)
            op = *S++;
          if (op > 0)
            { *a++ = ' ';
              *b++ = B[++j]+'A';
              op--;
            }
          else
            { *a++ = A[++i]+'A';
              *b++ = ' ';
              op++;
            }
          *c++ = '-';
        }
      if (a >= ALINE+50 || (i >= M && j >= N) )
        { *a = *b = *c = '\0';
          (void) printf("\n%5d ",50*lines++);
          for (b = ALINE+10; b <= a; b += 10)
            (void) printf("    .    :");
          if (b <= a+5)
            (void) printf("    .");
          (void) printf("\n%5d %s\n      %s\n%5d %s\n",ap,ALINE,CLINE,bp,BLINE);
	  ap = AP + i;
	  bp = BP + j;
          a = ALINE;
          b = BLINE;
          c = CLINE;
        }
    }
}

static void fill_salign(int M,int N,int *S, char *sal)
{
  register int  i, j, op;

  i = j = op = 0;
  while (i < M || j < N) {
      /* fprintf(stderr, "S--->%3d (i=%3d j=%3d)\n",*S,i,j); */
      if (op == 0 && *S == 0) {
          op = *S++;
          i++;
          j++;
          *sal = 's';
      }
      else {
          if (op == 0)
            op = *S++;
          if (op > 0) {
              *sal = 'd'; 
              j++;
              op--;
          }
          else {
              *sal = 'i';
              i++;
              op++;
          }
      }
      sal++;
  }
  *sal ='\0';
}

