/*Fonctions d'ecriture des resultats*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h> /*Pour exponentiel de mtd proba*/

#include "swelfe.h"
#include "swalign.h"
#include "util.h"





/* 14 couleurs pour midas (total de 0 a 65): rouge vert jaune bleu //erreur violet 22 orange 36// cyan magenta gris blanc (pas surs : vert-clair orange-clair bleu-clair violet-clair 54, 41, 12, 27)*/
static int couleurs[] ={32, 1, 48, 16, 8, 24, 65, 0};
/***********************************************************************/
/*  fichier pymol
 */
/*************************************************************************/
int EcritTousPymol(tyProt *p0,SWResult *swres, FILE *pFiOut, tyParametres *pParms,  int nbStructVoulues) {

    int i, j;
    int lg1=0, lg2=0;
    int deb1, deb2, fin1, fin2;

    if (pFiOut==NULL || p0==NULL || pParms == NULL ) {
        fprintf(stderr, "Ecriture du fichier pour pymol impossibles (un des parametres est null)\n");
        return(PASBON);
    }


    for (i=0; i<nbStructVoulues && i < swres->nbprint; i++) {
        deb1 = swres->posd0[i];
        deb2 = swres->posd1[i];

        for (j = 0 ; j < strlen(swres->sali[i]); j++) {
            if (swres->sali[i][j] == SUBST) {
                lg1++;
                lg2++;
            } else if (swres->sali[i][j] == INSER) {
                lg1 ++;
            } else {
                lg2 ++;
            }
        }
        fin1 = deb1 + lg1;
        fin2 = deb2 + lg2;


        fprintf(pFiOut, "load %s.pdb , struct1\n", p0->nomProt);
        fprintf(pFiOut, "hide everything, struct1 \n show ribbon, struct1\n color blue, struct1\n");
        fprintf(pFiOut, "load %s.pdb , struct2\n", p0->nomProt);
        fprintf(pFiOut, "hide everything,  struct2 \n show ribbon,  struct2 \n color red,  struct2\n");

        fprintf(pFiOut, "select  sel11, struct1 and resi %d-%d \n", deb1, fin1);
        fprintf(pFiOut, "color forest, sel11\n");
        fprintf(pFiOut, "select  sel12, struct1 and resi %d-%d\n", deb2, fin2);
        fprintf(pFiOut, "color yellow, sel12 \n");
        fprintf(pFiOut, "select  sel21, struct2 and resi %d-%d\n", deb1, fin1);
        fprintf(pFiOut, "color purple, sel21 \n");
        fprintf(pFiOut, "select  sel22, struct2 and resi %d-%d\n", deb2, fin2);
        fprintf(pFiOut, "color orange, sel22 \n");

        fprintf(pFiOut, "run fitting.py \n");
        fprintf(pFiOut, "fitting struct1, i. %d-%d , struct2, i. %d-%d\n\n\n", deb1, fin1, deb2, fin2);
        fprintf(stderr, "impossible d'afficher la proteine %d\n", i+1);
    }

    return(BON);
}


#if 0

/* ----------------------------------------------------------- */
/*Get_Prot: RENVOYER LA PROTINE DE RANG VOULU rang            */
/*Attention, ne pas l'utiliser si il y a encore des protines  */
/* classer: je ne sais pas ce que a fait de trier l'arbre    */
/*en plein milieu ( priori rien mais bon)                     */
/* ----------------------------------------------------------- */
/*Renvoie un pointeur sur la proteine de rang rang,            */
/*NULL si pas/plus de proteine  ce rang*/
/* ----------------------------------------------------------- */
tyProt *Get_Prot_Fin(int rang) {

    if (sArbreTrie==PASBON)
        sort_arbre();

    if (rang < skeep_struct && sArbreScan[rang].score != -1)
        return(sArbreScan[rang].pProt);
    else
        return(NULL);
} /*Fin de Get_Prot*/


#endif
/************************************************/
/*Ecrire une des proteines de l'enTete de fichier set pour gok*/
/************************************************/
static int EcritProtEnTeteSet( tyProt *pProt, FILE *pFiOut, int adresse, int color) {



    if (pProt==NULL || pFiOut==NULL) {
        fprintf(stderr, "a parameter of EcritEnTeteSet is NULL\n");
        return(PASBON);
    }



    fprintf(pFiOut, "name %s%d\n", pProt->nomProt, adresse);
    fprintf(pFiOut, "visible 1\n");
    fprintf(pFiOut, "color %d\n", couleurs[color]);
    fprintf(pFiOut, "path %s\n", pProt->pdbpath);
    fprintf(pFiOut, "adress %d\n", adresse);
    fprintf(pFiOut, "end\n");

    return(BON);


}



/************************************************/
/*Ecrire un enTete de fichier set pour gok*/
/************************************************/
static int EcritProtSet( tyProt *p0, tyProt *p1, SWResult *swres, FILE *pFiOut, int nombre, int color) {
    int i;
    int ip0, ip1;
    int lgmot;


    if (p1==NULL || pFiOut==NULL || p1==NULL) {
        fprintf(stderr, "un des parametres de EcritEnTeteSet est NULL\n");
        return(PASBON);
    }



    for (i=0; i<nombre; i++) {

        fprintf(pFiOut, "name match%d \n", i);

        ip0=swres->posd0[i];
        ip1=swres->posd1[i];
        lgmot = swres->posf0[i] - swres->posd0[i];

        fprintf(pFiOut, "pdbref 0 %d %c-%d %c %d-%d %d-%d\n",
                (int)(p0->tNumPDB[ip0].num[1]),
                p0->tFrag[0].name,
                (int)(p0->tNumPDB[ip0+lgmot-1].num[1]),
                p0->tFrag[0].name,
                (int)(p0->tNumPDB[ip0].num[1]),
                (int)(p0->tNumPDB[ip0+lgmot-1].num[1]),
                (int)(ip0),
                (int)(ip0+lgmot-1));

        fprintf(pFiOut, "pdbref 1 %d %c-%d %c %d-%d %d-%d\n",
                (int)(p1->tNumPDB[ip1].num[1]),
                p1->tFrag[0].name,
                (int)(p1->tNumPDB[ip1+lgmot-1].num[1]),
                p1->tFrag[0].name,
                (int)(p1->tNumPDB[ip1].num[1]),
                (int)(p1->tNumPDB[ip1+lgmot-1].num[1]),
                (int)(ip1),
                (int)(ip1+lgmot-1));

        fprintf(pFiOut, "visible 1\n");
        fprintf(pFiOut, "color %d\n", couleurs[(color+i)%8]);
        fprintf(pFiOut, "symbol alpha\n");
        fprintf(pFiOut, "end\n");

    }

    return(BON);
}



/*********************************************************************/
/*EcritSet()
  Ecrire un fichier .set lisible par gok contenant les mots communs trouves
  pour toutes les  proteines
*/
/***********************************************************************/
int EcritToutDansSet(tyProt *p0, tyProt *p1, SWResult *swres, FILE *pFiOut, int nbStructVoulues) {

    if (p0 == NULL || p1 == NULL || swres == NULL || pFiOut == NULL) {
        fprintf(stderr, "a parameter of EcritToutDansSet is NULL\n");
        fprintf(pFiOut, "Cannot create \"set\" file\n");
        return(PASBON);
    }

    fprintf(pFiOut, "*-PROTEIN-SET---ASCII-FILE--*\n");

    fprintf(stderr, "*  suite2  *\n");

    if (EcritProtEnTeteSet( p0, pFiOut, 0,  0) == PASBON)   {
        fprintf(stderr, "Cannot create \"set\" file\n");
        return(PASBON);
    }

    if (EcritProtEnTeteSet( p1, pFiOut, 1, 1) == PASBON)  {
        fprintf(stderr, "Cannot create \"set\" file\n");
        return(PASBON);
    }

    fprintf(pFiOut, "END_PDB_REC\n");

    if (nbStructVoulues > swres->nmatch)
        nbStructVoulues =   swres->nmatch;

    if (EcritProtSet(p0, p1, swres, pFiOut,nbStructVoulues,2 ) == PASBON)    {
        fprintf(stderr, "Impossible de creer le fichier set\n");
        return(PASBON);
    }


    fprintf(pFiOut, "END_WORD\n");

    return(BON);
}

/*recupere la fin de la proteine*/



#if 0
/***********************************************************************/
/*EcritTousSHSP(tyProt *pProt, tyProt *pQuery, tyParametres *pParms, FILE *pFiOut)
  Ecrire les SHSP trouves dans le fichier de sorti pourt que R puisseu en faire un graph
*/
/*************************************************************************/
int EcritTousSHSP_R(tyProt *pQuery, FILE *pFiOut, tyParametres *pParms) {

    int i, iP, iQ, iSHSP;
    tyProt *pProt=NULL;

    if (pFiOut==NULL || pQuery==NULL || pParms == NULL ) {
        fprintf(stderr, "Ecriture des SHSP pour  R impossibles (un des parametres est null)\n");
        return(PASBON);
    }

    /*Ecriture des indices de tous les CA de la query*/

    fprintf(pFiOut, "%10s ", pQuery->nomProt);
    for (i=0; i<pQuery->nbRes; i++) {
        fprintf(pFiOut, " %4d ", (int)(pQuery->tNumPDB[i].num[i]));
    }
    fprintf(pFiOut, "\n");
    i=0;
    pProt=Get_Prot_Fin(i);
    if (pProt==NULL) {
        fprintf(stderr, "EcritTousSHSP_R:: No protein returned from the minimier\n");
        return(PASBON);
    }

    while ( pProt !=NULL) {
        iQ=0;
        fprintf(pFiOut, "%10s ", pProt->nomProt);
        for (iSHSP=0; iSHSP < pProt->pResult->nbMots; iSHSP++) {
            iP= pProt->pResult->tMots[iSHSP].posQuery-pProt->pResult->tMots[iSHSP].decalage;
            for (; iQ<pProt->pResult->tMots[iSHSP].posQuery; iQ++)
                fprintf(pFiOut, " -100 ");/*Met des espaces entre les SHSPs*/
            for (; iQ<pProt->pResult->tMots[iSHSP].posQuery+pProt->pResult->tMots[iSHSP].lgMot; iQ++) {
                fprintf(pFiOut, " %4d ", iP);
                iP++;
            }
        }
        for (; iQ<pQuery->nbRes; iQ++)
            fprintf(pFiOut, " -100 ");/*Met des espaces entre les SHSPs*/

        fprintf(pFiOut, "\n");
        i++;
        pProt=Get_Prot_Fin(i);
    }
    return(BON);
}

/*****************************************/
/*Ecriture des resultats*/
/*****************************************/
int EcritUnResult(tyProt *pQuery, tyProt *pProtBank,
                  FILE *pFiOut, int classement, tyStatResult *pStat, tyParametres *pParms) {
    tyResult *pMot;
    tyCouleurRGB *pCouleursMot = NULL;
    int iProt, iQuery; /*indice du result dans le tableau*/
    /* contenant tous les angles de la prot ou query*/
    int k;
    int iFragP=0, iFragQ=0;

    if (pQuery == NULL || pFiOut == NULL || pProtBank == NULL) {
        fprintf(stderr,"a parameter of EcritUnResult is NULL\n");
        return(PASBON);
    }


    fprintf(pFiOut, "%s %d %s %10.2f %s %10.2f %s %s : %s \n",
            NAME_KWORD_NEW_MATCH, classement+1, NAME_SCORE,
            pProtBank->scoreClassement, NAME_SCORE_Z,
            (pProtBank->scoreClassement-pStat->avgScores)/pStat->sigmaScores,
            NAME_SEQ_MATCH,
            pProtBank->nomProt, pProtBank->desc  );
    fprintf(pFiOut, "%s %d residues, %d/%d residus aligned (%5.1f%%)\n",
            NAME_LENGTH_ALIGN, pProtBank->nbRes,
            pProtBank->pResult->lgChemin,
            pProtBank->nbRes,(float)(pProtBank->pResult->lgChemin)/(float)(pProtBank->nbRes)*(float)100);
    fprintf(pFiOut, "%s %s\n", NAME_FULL_PATHNAME, pProtBank->pdbpath);

    if (pParms->printScores == BON) {
        fprintf(pFiOut,"Computed scores : \n"
                "  score : %11.8f score Z= %11.8f\n"
                "  Sum(Log(ProbaIndep(SHSP)))= %10f  with spatially compatible SHSPs only: %10f\n"
                "  Sum(Log(ProbaBi(SHSP)))=    %10f  with spatially compatible SHSPs only: %10f\n"
                "  Sum(Log(ProbaMTD(SHSP)))=   %10f  with spatially compatible SHSPs only: %10f\n",
                (double)pProtBank->pResult->scoreTot,
                ((double)pProtBank->pResult->scoreTot - pStat->avgScores) / pStat->sigmaScores,
                pProtBank->pResult->sumLogProbaIndep,
                pProtBank->pResult->sumLogProbaIndep_Compatible,
                pProtBank->pResult->sumLogProbaBi,
                pProtBank->pResult->sumLogProbaBi_Compatible,
                pProtBank->pResult->sumLogProbaMTD,
                pProtBank->pResult->sumLogProbaMTD_Compatible);
    }

    fprintf(pFiOut, "%s %d SHSPs found\n", NAME_SHSP, pProtBank->pResult->nbMots);

    /* imprime l'entete	*/
    fprintf(pFiOut,"Id    %11s  chain  %11s  chain  %6s  %5s  %6s %7s  ",
            POSQUERY_STRING, POSBANK_STRING, SCORE_STRING, SHIFT_STRING,
            LENGTH_STRING, RMSD_STRING);
    if (pProtBank->pResult->tProbaIndep != NULL)
        fprintf(pFiOut, "%12s  ",PROBAMONO_STRING);
    if (pProtBank->pResult->tProbaBi != NULL)
        fprintf(pFiOut, "%12s  ",PROBABI_STRING);
    if (pProtBank->pResult->tProbaMTD != NULL)
        fprintf(pFiOut, "%12s  ",MTDPROBA_STRING);
    if (pParms->RasmolWeb == BON)
        fprintf(pFiOut, "%7s ",COLOR_STRING);
    fprintf(pFiOut, "\n");

    for (k=0; k<pProtBank->pResult->nbMots; k++) {
        if (pParms->RasmolWeb == BON) {
            if (pProtBank->pResult->tCouleursMots==NULL) {
                if (pProtBank->pResult->nbMots >1) {
                    fprintf(stderr, "EcritUnResultWeb:: No RGB color for SHSP, no web output\n");
                    exit(ERR_SORTIES);
                } else {

                }
            }
            pCouleursMot=&(pProtBank->pResult->tCouleursMots[k]);
        }

        pMot=&(pProtBank->pResult->tMots[k]);

        iQuery=pMot->posQuery;
        iProt=pMot->posQuery-pMot->decalage;

        while (iQuery > pQuery->tFrag[iFragQ].fin) {
            iFragQ++;
            if (iFragQ>=pQuery->nbFrag)
                MyError(stderr, ERR_SORTIES, "Pb avec frag de query: iQuery: %d, debut fdernier frag %d (iFragQ: %d\n",
                        iQuery,  pQuery->tFrag[iFragQ-1].debut, iFragQ);
        }
        while (iProt > pProtBank->tFrag[iFragP].fin) {
            iFragP++;
            if (iFragP>=pProtBank->nbFrag)
                MyError(stderr, ERR_SORTIES, "Pb avec frag de prot: iProt: %d, debut dernier frag %d (iFragp: %d\n",
                        iProt,  pProtBank->tFrag[iFragP-1].debut, iFragP);
        }
        if (iProt<0 || iQuery <0) {
            fprintf(stderr, "EcritUnResult:: iP %d or iQ %d is negative, it shouldn't append (dec: %d, num %d)\n",
                    iProt, iQuery,
                    pMot->decalage, k );
            return(PASBON);
        }

        /*       fprintf(pFiOut, "frag %d/%d, %c, frag %d/%d, %c\n", */
        /* 	      iFragQ, pQuery->nbFrag, */
        /* 	      (pQuery->tFrag[iFragQ].name),  */
        /* 	      iFragP, pProtBank->nbFrag, */
        /* 	      (pProtBank->tFrag[iFragP].name)); */

        fprintf(pFiOut,
                "[ %2d] %4d%c - %4d%c (%c)  %4d%c - %4d%c (%c)  %6d  %5d  %6d ",
                k+1,
                (int)(pQuery->tNumPDB[iQuery].num[1]),
                pQuery->tNumPDB[iQuery].inser[0],
                (int)(pQuery->tNumPDB[iQuery+pMot->lgMot-1].num[1]),
                pQuery->tNumPDB[iQuery+pMot->lgMot-1].inser[0],
                (pQuery->tFrag[iFragQ].name),
                (int)(pProtBank->tNumPDB[iProt].num[1]),
                pProtBank->tNumPDB[iProt].inser[0],
                (int)(pProtBank->tNumPDB[iProt+pMot->lgMot-1].num[1]),
                pProtBank->tNumPDB[iProt+pMot->lgMot-1].inser[0],
                (pProtBank->tFrag[iFragP].name),
                pMot->score,
                (int)(pQuery->tNumPDB[iQuery].num[1]-pProtBank->tNumPDB[iProt].num[1]),
                pMot->lgMot);

        if (pProtBank->pResult->tRMSD != NULL) {
            fprintf(pFiOut,"%7.1f  ", pProtBank->pResult->tRMSD[k]);
        } else if (pProtBank->pResult->tRMSDCroises != NULL) {
            fprintf(pFiOut,"%7.1f  ",
                    pProtBank->pResult->tRMSDCroises[k*pProtBank->pResult->nbMots+k]);
        } else {
            fprintf(pFiOut,"  none  ");
        }

        if (pProtBank->pResult->tProbaIndep != NULL)
            fprintf(pFiOut, "%10E  ",pProtBank->pResult->tProbaIndep[k]);

        if (pProtBank->pResult->tProbaBi != NULL)
            fprintf(pFiOut, "%10E  ",pProtBank->pResult->tProbaBi[k]);

        if (pProtBank->pResult->tProbaMTD != NULL)
            fprintf(pFiOut, "%10E  ", pProtBank->pResult->tProbaMTD[k]);

        if (pParms->RasmolWeb == BON) {
            if (pProtBank->pResult->tCouleursMots!=NULL)
                fprintf(pFiOut,"#%2X%2X%2X ",
                        (unsigned int)(pCouleursMot->r),
                        (unsigned int)(pCouleursMot->g),
                        (unsigned int)(pCouleursMot->b));
            else
                fprintf(pFiOut,"#%2X%2X%2X ",
                        (unsigned int)(255),
                        (unsigned int)(255),
                        (unsigned int)(255));
        }


        fputc((int)'\n', pFiOut);

#ifdef ECRITANGLES
        /* 	  fprintf(pFiOut,"---> iQuery=%d iProt=%d",iQuery,iProt); */
        nbLignes=pMot->lgMot/10;
        if (nbLignes*10<pMot->lgMot)
            nbLignes++;

        for (l=0; l<nbLignes; l++) {
            fprintf(pFiOut, "query ");
            for (i=iQuery+l*10; i<iQuery+pMot->lgMot && i<iQuery+(l+1)*10; i++) {
                /*  		  fprintf(pFiOut, "%4d ", pQuery->tAnglesRecodes[i]);  */
                fprintf(pFiOut, "%6.1f ", pQuery->tAngles[i]/(float)DIVANGLE);
            }
            fputc((int)'\n', pFiOut);

            fprintf(pFiOut, "prot  ");
            for (i=iProt+l*10; i<iProt+pMot->lgMot && i<iProt+(l+1)*10; i++) {
                /*  		  fprintf(pFiOut, "%4d ", pProtBank->tAnglesRecodes[i]);  */
                fprintf(pFiOut, "%6.1f ", pProtBank->tAngles[i]/(float)DIVANGLE);
            }
            fputc((int)'\n', pFiOut);

        }
        fputc((int)'\n', pFiOut);
#endif

    } /*fin for k<nbMots donc fin d'une prot*/


    if (pProtBank->pResult->tSHSPCompatibles != NULL)
        EcritCompatiblesSpatiaux(pProtBank, pFiOut, pParms->classement);
    else
        fprintf(pFiOut, "No array of \"spatially compatible\" SHSP (NULL)\n");

    if (pProtBank->pResult->tRMSDCroises!=NULL)
        EcritRMSCroises(pFiOut, pProtBank);

    fprintf(pFiOut, "%s\n", SHOW_SHSP);
    for (k=0; k<pProtBank->pResult->nbMots; k++) {
        pMot=&(pProtBank->pResult->tMots[k]);
        iQuery=pMot->posQuery;
        iProt=pMot->posQuery-pMot->decalage;
        fprintf(pFiOut, "%s %d\n", STRINGSHSP, k+1);
        EcritSeqSHSP(iQuery, iProt, pMot->lgMot, pQuery->tNameRes, pProtBank->tNameRes, pFiOut);
    }
    fputc((int)'\n', pFiOut);
    return(BON);
}

/*************************************************/
/* Ecrire les sequences*/
/************************************************/
static void EcritSeqSHSP(int iQuery,  int iProt, int lgMot, char *tResQuery, char *tResProt, FILE *pFiOut) {

    int i, finLigne, debutLigne;
    int lgLigne=40;

    if (lgMot<lgLigne) {
        fprintf(pFiOut, "%s    ",STRINGQUERY);
        for (i=0; i<lgMot; i++)
            fputc((int)tResQuery[iQuery+i], pFiOut);
        fprintf(pFiOut, "\n         ");
        for (i=0; i<lgMot; i++)
            fputc((int)IdChar(tResQuery[iQuery+i], tResProt[iProt+i]), pFiOut);

        fprintf(pFiOut, "\n%s ",STRINGDATABASE);
        for (i=0; i<lgMot; i++)
            fputc((int)tResProt[iProt+i], pFiOut);
        fputc((int)'\n', pFiOut);
        fputc((int)'\n', pFiOut);
    } else {
        i=0;
        while (i<lgMot) {
            finLigne=i+lgLigne;
            debutLigne=i;
            fprintf(pFiOut, "%s    ",STRINGQUERY);
            for (i=debutLigne; i<finLigne && i<lgMot; i++)
                fputc((int)tResQuery[iQuery+i], pFiOut);
            fprintf(pFiOut, "\n         ");
            for (i=debutLigne; i<finLigne && i<lgMot; i++)
                fputc((int)IdChar(tResQuery[iQuery+i], tResProt[iProt+i]), pFiOut);

            fprintf(pFiOut, "\n%s ",STRINGDATABASE);
            for (i=debutLigne; i<finLigne && i<lgMot; i++)
                fputc((int)tResProt[iProt+i], pFiOut);
            fputc((int)'\n', pFiOut);
            fputc((int)'\n', pFiOut);
        }
    }

    return;
}

/*---------------------------------------------	*/
/* return an "Id" character, i.e. '|' when	*/
/* c1 == c2, or similarity ":" or "."		*/
/*---------------------------------------------	*/
static char IdChar(char c1, char c2) {
    /* BLOSUM62 matrix	*/
    static int mat[24][24]= {{4,-1,-2,-2,0,-1,-1,0,-2,-1,-1,-1,-1,-2,-1,1,0,-3,-2,0,-2,-1,0,-4},
        {-1,5,0,-2,-3,1,0,-2,0,-3,-2,2,-1,-3,-2,-1,-1,-3,-2,-3,-1,0,-1,-4},
        {-2,0,6,1,-3,0,0,0,1,-3,-3,0,-2,-3,-2,1,0,-4,-2,-3,3,0,-1,-4},
        {-2,-2,1,6,-3,0,2,-1,-1,-3,-4,-1,-3,-3,-1,0,-1,-4,-3,-3,4,1,-1,-4},
        {0,-3,-3,-3,9,-3,-4,-3,-3,-1,-1,-3,-1,-2,-3,-1,-1,-2,-2,-1,-3,-3,-2,-4},
        {-1,1,0,0,-3,5,2,-2,0,-3,-2,1,0,-3,-1,0,-1,-2,-1,-2,0,3,-1,-4},
        {-1,0,0,2,-4,2,5,-2,0,-3,-3,1,-2,-3,-1,0,-1,-3,-2,-2,1,4,-1,-4},
        {0,-2,0,-1,-3,-2,-2,6,-2,-4,-4,-2,-3,-3,-2,0,-2,-2,-3,-3,-1,-2,-1,-4},
        {-2,0,1,-1,-3,0,0,-2,8,-3,-3,-1,-2,-1,-2,-1,-2,-2,2,-3,0,0,-1,-4},
        {-1,-3,-3,-3,-1,-3,-3,-4,-3,4,2,-3,1,0,-3,-2,-1,-3,-1,3,-3,-3,-1,-4},
        {-1,-2,-3,-4,-1,-2,-3,-4,-3,2,4,-2,2,0,-3,-2,-1,-2,-1,1,-4,-3,-1,-4},
        {-1,2,0,-1,-3,1,1,-2,-1,-3,-2,5,-1,-3,-1,0,-1,-3,-2,-2,0,1,-1,-4},
        {-1,-1,-2,-3,-1,0,-2,-3,-2,1,2,-1,5,0,-2,-1,-1,-1,-1,1,-3,-1,-1,-4},
        {-2,-3,-3,-3,-2,-3,-3,-3,-1,0,0,-3,0,6,-4,-2,-2,1,3,-1,-3,-3,-1,-4},
        {-1,-2,-2,-1,-3,-1,-1,-2,-2,-3,-3,-1,-2,-4,7,-1,-1,-4,-3,-2,-2,-1,-2,-4},
        {1,-1,1,0,-1,0,0,0,-1,-2,-2,0,-1,-2,-1,4,1,-3,-2,-2,0,0,0,-4},
        {0,-1,0,-1,-1,-1,-1,-2,-2,-1,-1,-1,-1,-2,-1,1,5,-2,-2,0,-1,-1,0,-4},
        {-3,-3,-4,-4,-2,-2,-3,-2,-2,-3,-2,-3,-1,1,-4,-3,-2,11,2,-3,-4,-3,-2,-4},
        {-2,-2,-2,-3,-2,-1,-2,-3,2,-1,-1,-2,-1,3,-3,-2,-2,2,7,-1,-3,-2,-1,-4},
        {0,-3,-3,-3,-1,-2,-2,-3,-3,3,1,-2,1,-1,-2,-2,0,-3,-1,4,-3,-2,-1,-4},
        {-2,-1,3,4,-3,0,1,-1,0,-3,-4,0,-3,-3,-2,0,-1,-4,-3,-3,4,1,-1,-4},
        {-1,0,0,1,-3,3,4,-2,0,-3,-3,1,-1,-3,-1,0,-1,-3,-2,-2,1,4,-1,-4},
        {0,-1,-1,-1,-2,-1,-1,-1,-1,-1,-1,-1,-1,-1,-2,0,0,-2,-1,-1,-1,-1,-1,-4},
        {-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,1}
    };
    /* index of c-'A' in BLOSUM62 matrix	*/
    /* A  R  N  D  C  Q  E  G  H  I  L  K  M  F  P  S  T  W  Y  V  B  Z  X  * */
    /* 0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 */
    /* indx contient les index de lalphabet de 'A' a 'Z' dans mat	*/
    static int indx[26]={0,20,4,3,6,13,7,8,9,23,11,10,12,2,23,14,5,1,15,16,23,19,17,22,18,21};

    if (mat[indx[(int)(c1-'A')]][indx[(int)(c2-'A')]] >= 4)
        return '|';
    else if (mat[indx[(int)(c1-'A')]][indx[(int)(c2-'A')]] >= 2)
        return ':';
    else if (mat[indx[(int)(c1-'A')]][indx[(int)(c2-'A')]] >= 1)
        return '.';
    else
        return ' ';

}
#endif
#if 0   /*rien a partir de la pour le moment!!!!!!!!!!!*/

/****************************************************/
/*Ecrire les compatibilits spatiales des SHSP
Entres: pProtBank: la proteine de la banque
         pFiOut: ptr sur fichier de sortie

Sortie: Bon si OK, PASBON sinon*/
/****************************************************/
int EcritCompatiblesSpatiaux(tyProt *pProtBank, FILE* pFiOut, char classement) {

    int i, j, k, lgCheminCompatibles;
    double logProbaGpCourant=0;
    double *tProba=NULL;

    /* Le tableau tCompatibles contient les compatibilits spatiales des SHSPs*/
    /* si  c'est le 1er d'un groupe, toujours l'indice du suivant multipli par -1*/
    /* ou le numero du SHSP suivant du groupe si c'est 1 SHSP d'un groupe hors 1er et dernier*/
    /*ou l'indice de la case si c'est la fin d'un groupe*/
    /* Et si le SHSP est seul, la case contiendra sont numero*-1*/
    switch (classement) {
    case CLASS_PROBA_INDEP_COMPATIBLES :
        tProba = pProtBank->pResult->tProbaIndep;
        break;
    case CLASS_PROBA_BI_COMPATIBLES :
        tProba = pProtBank->pResult->tProbaBi;
        break;
    case CLASS_PROBA_MTD_COMPATIBLES :
        tProba = pProtBank->pResult->tProbaMTD;
        break;
    }/*fin switch*/
    if (tProba == NULL) {
        fprintf(stderr, "EcritCompatiblesSpatiaux:: tProba is empty (NULL), should not append!!!\n");
        return(PASBON);
    }


    lgCheminCompatibles=0; /*Nobre de residus pour ce groupe*/
    k=0; /*Nombre de SHSP dj "vus"*/
    i=0; /*numero/indice du 1er SHSP du groupe (indice linaire)*/
    j=0; /*numero du SHSP courant*/
    while (k<pProtBank->pResult->nbMots) {
        if (pProtBank->pResult->tSHSPCompatibles[j] <= 0 ) {
            if (i!=j) {
                fprintf(pFiOut, "EcritUnResult:: Ya 1 pb avec le parcours ou le tableau des SHSP compatibles\n");
                fprintf(pFiOut, "Un SHSP incompatibles avec tous les autres fait parte d'un groupe: i: %d, j: %d\n", i, j);
                exit(ERR_SORTIES);
            }
            /*Si les numeros sont negatifs*/
            /*Le SHSP est seul ou le 1er d'un groupe*/
            if ( pProtBank->pResult->tSHSPCompatibles[j] == j*-1) {
                /* Il est tout seul*/
                lgCheminCompatibles=pProtBank->pResult->tMots[j].lgMot;
                if (tProba[j]<0) /*Cas des mtd: c'est dj un log10, donc <0*/
                    logProbaGpCourant=tProba[j];
                else
                    logProbaGpCourant=log10(tProba[j]);
                j=pProtBank->pResult->tSHSPCompatibles[j]*-1; /*indice du SHSP suivant*/
                fprintf(pFiOut, "SHSP %d is not spatially compatible with others SHSPs (score %8.2f, %5.2f%% residues aligned)\n",
                        j+1,logProbaGpCourant*-1, (float)lgCheminCompatibles/pProtBank->nbRes*100);
                logProbaGpCourant=0;
                lgCheminCompatibles=0;
                k++;
                i++;
                j++;
            } else { /*C'est le premier d'un groupe */
                fprintf(pFiOut, "Group of SHSPs spatially compatible: %d ", j+1);
                lgCheminCompatibles=pProtBank->pResult->tMots[j].lgMot;
                if (tProba[j]<0) /*Cas des mtd: c'est dj un log10, donc <0*/
                    logProbaGpCourant=tProba[j];
                else
                    logProbaGpCourant=log10(tProba[j]);
                j=pProtBank->pResult->tSHSPCompatibles[j]*-1; /*indice du SHSP suivant*/
                k++;
            }

        } else {
            if (i!=j) { /*Au tour precedent, on tait dans un groupe*/
                lgCheminCompatibles+=pProtBank->pResult->tMots[j].lgMot;
                if (tProba[j]<0) /*Cas des mtd: c'est dj un log10, donc <0*/
                    logProbaGpCourant+=tProba[j];
                else
                    logProbaGpCourant+=log10(tProba[j]);
                if ( pProtBank->pResult->tSHSPCompatibles[j] == j) { /*C'est la fin d'un groupe*/
                    fprintf(pFiOut, " %d (score %8.2f, %5.2f%% residues aligned) \n",
                            j+1, logProbaGpCourant*-1, (float)lgCheminCompatibles/pProtBank->nbRes*100);
                    k++; /*Incrmente le nombre de SHSP vus*/
                    i++; /* On va reprendre le parcours du tableau  partir du suivant du 1er du groupe */
                    j=i; /*cf ci dessus*/
                } else { /*On est toujours dans un groupe*/
                    fprintf(pFiOut, " %d ", j+1);
                    k++; /*Incrmente le nombre de SHSP vus*/
                    j=pProtBank->pResult->tSHSPCompatibles[j]; /*indice du SHSP suivant*/
                }
            } else {/* Au tour precedent, soit on atteint la fin d'un groupe, soit pas dans 1 groupe*/
                /*On cerche le prochain debut de groupe, ie tSHSPCompatibles[j]<0, pas le cas ici*/
                j++;
                i++;
            }

        }/*fin if(pProtBank->pResult->tSHSPCompatibles[j] <= 0 )else*/
    }/*fin while*/
    return(BON);
}


/*************************************************/
/*
*/
/***************************************************/
static int EcritRMSCroises(FILE *pFiOut, tyProt *pProt) {
    int j, k;
    double somRMS=0;
    /*   int i; */

    if (pProt==NULL || pFiOut==NULL) {
        fprintf(stderr, "One parameter of EcritRMSCroises is NULL\n");
        return(PASBON);
    }

    fprintf(pFiOut, "Cross RMSD of the SHSP\n");

    /*   for (i=0; i<skeep_struct; i++) */
    /*     { */
    /*       fprintf(pFiOut, "Protein %s\n", pProt->nomProt); */

    /*ce if pourrait etre enleve : ce test est fait aussi par la fonction mere*/
    if (pProt->pResult->tRMSDCroises!=NULL) {

        fprintf(pFiOut, "                 | num HSSP for RMSD:\n");
        fprintf(pFiOut, "num HSSP for     |\n");
        fprintf(pFiOut, "Rotation matrix  |");

        for (j=0; j<pProt->pResult->nbMots; j++) {
            fprintf(pFiOut, " %6d |", j+1);
        }
        fprintf(pFiOut, "    avg |\n");
        fprintf(pFiOut, "------------------");

        for (j=0; j<pProt->pResult->nbMots+1; j++)
            fprintf(pFiOut, "---------");
        fprintf(pFiOut, "\n");

        for (j=0; j<pProt->pResult->nbMots; j++) {
            fprintf(pFiOut, "%16d |", j+1);
            somRMS=0;
            for (k=0; k<pProt->pResult->nbMots; k++) {
                if (pProt->pResult
                        ->tRMSDCroises[j*pProt->pResult->nbMots+k] < 9999)
                    fprintf(pFiOut, " %6.1f |",
                            pProt->pResult
                            ->tRMSDCroises[j*pProt->pResult->nbMots+k]);
                else
                    fprintf(pFiOut, "  >9999 |");
                if (k!=j) { /*si ce n'est pas le RMS avec la matrice du mot*/
                    somRMS+=pProt->pResult
                            ->tRMSDCroises[j*pProt->pResult->nbMots+k];
                }
            }
            fprintf(pFiOut, " %6.1f |\n", somRMS/(double)(pProt->pResult->nbMots-1));
        }/*fin for j*/
    } /*fin if pProt->pResult->tRMSDCroises!=NULL)*/
    else
        fprintf(pFiOut, "no RMSD\n");

    fprintf(pFiOut, "\n");
    /*    }*//*fin for i*/

    return(BON);
}

/*************************************************/
/*Ecrire  un fichier pour molscript
 */
/***************************************************/

static int EcritUnMolscript(tyProt *pQuery, tyProt *pProt, FILE *pFiOut, tyParametres *pPar, int rang) {

    int iP, iQ, iPPrev, iQPrev; /*indice du result dans le tableau*/
    /* contenant tous les angles de la prot ou query*/
    int k, i, j, longestSHSP, lengthLongestSHSP, iQuery, iProt;
    /*     float colorQ=0.45, colorP=0.55, colorSHSP=0.0, saturationSHSP=1; */
    float ratio;
    int r,g,b;
    tyResult *pMot;

    tyStatResult stat; /*temp*/

    stat.avgScores=1;
    stat.sigmaScores=1;

    /*      { */
    /* 	int i; */
    /* 	fprintf(stderr, "Protein %s\n", pProt->nomProt); */
    /* 	for(i=0; i<pProt->pResult->nbMots;i++){ */
    /* 	  fprintf(stderr, "SHSP %d lg %d tCompatibles: %d\n", i, pProt->pResult->tMots[i].lgMot, pProt->pResult->tSHSPCompatibles[i]); */
    /* 	} */
    /*       } */

    if (pQuery==NULL || pFiOut==NULL || pProt==NULL) {
        fprintf(stderr, "un des parametres de EcritUnMolscript est NULL: pQuery %p pFiOut %p pProt %p\n", pQuery, pFiOut, pProt);
        return(PASBON);
    }
    if (pProt->pResult->tCouleursMots==NULL) {
        NEWY(pProt->pResult->tCouleursMots, tyCouleurRGB, pProt->pResult->nbMots);
        for (i=0;i<pProt->pResult->nbMots; i++) {
            if (pProt->pResult->nbMots > 1)
                ratio = (float)i/(pProt->pResult->nbMots-1);
            else
                ratio = 1.0;
#define BASECOLOR	170
            if (i%2 == 0)
                r = 255 - (int)((255-BASECOLOR)*ratio);
            else
                r = BASECOLOR + (int)((255-BASECOLOR)*ratio);
            g = BASECOLOR + (int)((255-BASECOLOR)*ratio);

            if (i%2 == 1)
                b = 255 - (int)((255-BASECOLOR)*ratio);
            else
                b = BASECOLOR + (int)((255-BASECOLOR)*ratio);
#if 0
            if (i%2 == 0)
                r = 255 - (int)((255-BASECOLOR)*(float)i/pProt->pResult->nbMots);
            else
                r = 255;
            g = BASECOLOR + (int)((255-BASECOLOR)*(float)i/pProt->pResult->nbMots);
            if (i%2 == 1)
                b = 255 - (int)((255-BASECOLOR)*(float)i/pProt->pResult->nbMots);
            else
                b = 255;
#endif
            pProt->pResult->tCouleursMots[i].r=r;
            pProt->pResult->tCouleursMots[i].g=g;
            pProt->pResult->tCouleursMots[i].b=b;

        }
        /* 	fprintf(stderr, "No color for EcritUnMolscript, can't draw\n"); return(PASBON); */
    }


    /*Par defaut, aligne sur le plus long des SHSP des SHSPs compatibles du mailleur gp -> le trouver*/

    if (pProt->pResult->tSHSPCompatibles[pProt->pResult->num1erSHSPMeilleurGp] > 0) {
        fprintf(stderr, "1er meilleur %d -> %d au milieu d'un groupe !!! \n",pProt->pResult->num1erSHSPMeilleurGp,
                pProt->pResult->tSHSPCompatibles[pProt->pResult->num1erSHSPMeilleurGp]);
        return(PASBON);
    }
    /*     fprintf(stderr, "1er meilleur %d -> %d lg %d\n",pProt->pResult->num1erSHSPMeilleurGp,  */
    /* 	    pProt->pResult->tSHSPCompatibles[pProt->pResult->num1erSHSPMeilleurGp], */
    /* 	    pProt->pResult->tMots[pProt->pResult->num1erSHSPMeilleurGp].lgMot); */
    longestSHSP=pProt->pResult->num1erSHSPMeilleurGp;
    lengthLongestSHSP=pProt->pResult->tMots[pProt->pResult->num1erSHSPMeilleurGp].lgMot;
    if (pProt->pResult->tSHSPCompatibles[pProt->pResult->num1erSHSPMeilleurGp]*-1 !=
            pProt->pResult->num1erSHSPMeilleurGp) { /* S'il n'est pas tout seul*/
        i=-1*pProt->pResult->tSHSPCompatibles[pProt->pResult->num1erSHSPMeilleurGp];
        /*     fprintf(stderr, "1er %d\n",i1erSHSPMeilleur);  */
        while (i!=pProt->pResult->tSHSPCompatibles[i]) {
            /* 	fprintf(stderr, "suivant: %d  -> %d lg %d\n", i,  */
            /* 	    pProt->pResult->tSHSPCompatibles[i], */
            /* 	    pProt->pResult->tMots[i].lgMot); */
            /*       fprintf(stderr, "i %d, tSHSPCompatibles[i]: %d\n", i, pProtBank->pResult->tSHSPCompatibles[i]); */

            if (lengthLongestSHSP < pProt->pResult->tMots[i].lgMot) {
                longestSHSP=i;
                lengthLongestSHSP=pProt->pResult->tMots[i].lgMot;
            }
            i=pProt->pResult->tSHSPCompatibles[i];

        }
        /* 	fprintf(stderr, "dernier: %d  -> %d lg %d\n", i,  */
        /* 	    pProt->pResult->tSHSPCompatibles[i], */
        /* 	    pProt->pResult->tMots[i].lgMot); */
        if (lengthLongestSHSP < pProt->pResult->tMots[i].lgMot) {
            longestSHSP=i;
            lengthLongestSHSP=pProt->pResult->tMots[i].lgMot;
        }
    }

    /*     for(i=0; i<pProt->pResult->nbMots;i++){ */
    /*       fprintf(stderr, "SHSP %d lg %d", i, pProt->pResult->tMots[i].lgMot); */
    /*       if(i == longestSHSP) */
    /* 	fprintf(stderr, " MEILLEUR LE PLUS LONG\n"); */
    /*       else */
    /* 	fprintf(stderr, "\n"); */
    /*     } */

    /*     longestSHSP=0; */
    /*     lengthLongestSHSP=pProt->pResult->tMots[0].lgMot; */
    /*     for(k=1; k<pProt->pResult->nbMots; k++){ */
    /*       pMot=&(pProt->pResult->tMots[k]); */
    /*       fprintf(stderr, "compare %d et %d\n", lengthLongestSHSP,  pMot->lgMot); */
    /*       if(lengthLongestSHSP < pMot->lgMot) { */
    /* 	longestSHSP=k; */
    /* 	lengthLongestSHSP=pMot->lgMot; */
    /*       } */
    /*     } */
    fprintf(pFiOut, "! PROTEIN %d %s\n", rang, pProt->nomProt);
    fprintf(pFiOut, "! title \"query%s database protein %s\"\n", pQuery->nomProt, pProt->nomProt);
    fprintf(pFiOut, "plot\n");
    fprintf(pFiOut, "     background black;\n");
    fprintf(pFiOut, "     read query \"%s\";\n", pQuery->pdbpath);
    fprintf(pFiOut, "     read database \"%s\";\n\n", pProt->pdbpath);


    /*Ecrire les matrices de rotations translations, la premiere peut etre directement utilisee*/
    /*Les suivantes sont en commentaire Molscript*/
    if (pProt->pResult->tMatrixRT!=NULL) {
        /*       fprintf(stderr, "EcritUnMolscript: longestSHSP: %d / %d SHSP\n", longestSHSP,  pProt->pResult->nbMots); */
        pMot=&(pProt->pResult->tMots[longestSHSP]);
        iQuery=pMot->posQuery;
        iProt=pMot->posQuery-pMot->decalage;
        /*       iQuery=pProt->pResult->tMots[longestSHSP].posQuery; */

        /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
               iProt=pProt->pResult->tMots[longestSHSP].posQuery-!!!!!!!!=!!!!!pProt->pResult->tMots[longestSHSP].decalage; */
        /*dur  retrouver comme erreur!!!!!!!!!!!!!*/
        if (iProt<0 || iQuery <0) {
            fprintf(stderr, "EcritUnMolscript 000:: iP %d or iQ %d is negative, it shouldn't append (dec: %d, SHSP num %d prot %s)\n", iProt, iQuery,
                    pMot->decalage, longestSHSP, pProt->nomProt );
            EcritUnResult(pQuery, pProt, stderr, rang, &stat, pPar );
            return(PASBON);
        }
        fprintf(pFiOut, "! SHSP %3d iQuery: %4d - %4d iProt: %4d - %4d\n",
                longestSHSP+1, (int)(pQuery->tNumPDB[iQuery].num[1]),
                (int)(pQuery->tNumPDB[iQuery+pProt->pResult->tMots[longestSHSP].lgMot-1].num[1]),
                (int)(pProt->tNumPDB[iProt].num[1]),
                (int)(pProt->tNumPDB[iProt+pProt->pResult->tMots[longestSHSP].lgMot-1].num[1]));
        /*1er SHSP*/
        fprintf(pFiOut, "       transform in molecule database\n");
        /*Ecrit Le vecteur de translation de la structure de la banque*/
        fprintf(pFiOut, "            by centre %f %f %f\n",
                pProt->pResult->tMatrixRT[longestSHSP][3*3],
                pProt->pResult->tMatrixRT[longestSHSP][3*3+1],
                pProt->pResult->tMatrixRT[longestSHSP][3*3+2]);
        /*Ecrit la matrice de rotation de la structure de la banque*/
        fprintf(pFiOut, "            by rotation\n");
        for (i=0; i<3; i++) {
            fprintf(pFiOut, "                 ");
            for (j=0; j<3; j++)
                fprintf(pFiOut, "%f ", pProt->pResult->tMatrixRT[longestSHSP][i*3+j]);
            fprintf(pFiOut, "\n");
        }
        fprintf(pFiOut, "       ;\n");
        /*Ecrit Le vecteur de translation de la structure requete*/
        fprintf(pFiOut, "       transform in molecule query\n");
        /*Ecrit Le vecteur de translation de la structure de la requete*/
        fprintf(pFiOut, "            by centre %f %f %f;\n",
                pProt->pResult->tMatrixRT[longestSHSP][4*3],
                pProt->pResult->tMatrixRT[longestSHSP][4*3+1],
                pProt->pResult->tMatrixRT[longestSHSP][4*3+2]);
        fprintf(pFiOut, "! ENDSHSP \n");

        /*SHSP suivants*/
        for (k=0; k<pProt->pResult->nbMots; k++) {
            if (k==longestSHSP) continue;
            iQuery=pProt->pResult->tMots[k].posQuery;
            iProt=pProt->pResult->tMots[k].posQuery-pProt->pResult->tMots[k].decalage;

            if (iProt<0 || iQuery <0) {
                fprintf(stderr, "EcritUnMolscript 111:: iP %d or iQ %d is negative, it shouldn't append (dec: %d, SHSP num %d prot %s)\n", iProt, iQuery,
                        pMot->decalage, longestSHSP, pProt->nomProt );
                EcritUnResult(pQuery, pProt, stderr, rang, &stat, pPar);
                return(PASBON);
            }

            fprintf(pFiOut, "! SHSP %3d iQuery: %4d - %4d iProt: %4d - %4d\n",
                    k+1, (int)(pQuery->tNumPDB[iQuery].num[1]),
                    (int)(pQuery->tNumPDB[iQuery+pProt->pResult->tMots[k].lgMot-1].num[1]),
                    (int)(pProt->tNumPDB[iProt].num[1]),
                    (int)(pProt->tNumPDB[iProt+pProt->pResult->tMots[k].lgMot-1].num[1]));
            /*             fprintf(pFiOut, "! SHSP %3d\n", k+1); */
            fprintf(pFiOut, "!       transform in molecule database\n");
            /*Ecrit Le vecteur de translation de la structure de la banque*/
            fprintf(pFiOut, "!            by centre %f %f %f\n",
                    pProt->pResult->tMatrixRT[k][3*3],
                    pProt->pResult->tMatrixRT[k][3*3+1],
                    pProt->pResult->tMatrixRT[k][3*3+2]);
            /*Ecrit la matrice de rotation de la structure de la banque*/
            fprintf(pFiOut, "!            by rotation\n");
            for (i=0; i<3; i++) {
                fprintf(pFiOut, "!                 ");
                for (j=0; j<3; j++)
                    fprintf(pFiOut, "%f ", pProt->pResult->tMatrixRT[k][i*3+j]);
                fprintf(pFiOut, "\n");
            }
            fprintf(pFiOut, "!       ;\n");

            /*Ecrit Le vecteur de translation de la structure requete*/
            fprintf(pFiOut, "!       transform in molecule query\n");
            /*Ecrit Le vecteur de translation de la structure de la banque*/
            fprintf(pFiOut, "!            by centre %f %f %f;\n",
                    pProt->pResult->tMatrixRT[k][4*3],
                    pProt->pResult->tMatrixRT[k][4*3+1],
                    pProt->pResult->tMatrixRT[k][4*3+2]);
            fprintf(pFiOut, "! ENDSHSP \n");

        }
    } else
        fprintf(stderr, "Problem: There is no translation rotation matrix for Molscript\n");

    /*Centrer le tout au milieu de l'image*/
    fprintf(pFiOut, "     transform atom * by centre position atom *;\n\n");

    fprintf(pFiOut, "     set linewidth 0.0; !grosseur de la ligne bordant le trait\n\
     set coilradius 0.2; !grosseur du trait\n\
     set segments 10; !nb segments entre CA a pas l'air de changer grand chose\n\
     set linecolour black;\n\
     pointlight 0 0 20; !source de lumire\n\
     set lightambientintensity 0.9; !Ne modifie qq chose que si a cree une source de mumire\n\
     set lightattenuation  1 0.5 0.5; !dft: 1 0 0\n\
     set lightintensity 0.5; !dft 1 (max) \n\
     set lightradius 1000; ! en Angstrom dft=100\n\n");


    iQPrev=0;
    iPPrev=0;

    for (k=0; k<pProt->pResult->nbMots; k++) {
        pMot=&(pProt->pResult->tMots[k]);

        iQ=pMot->posQuery;
        iP=pMot->posQuery-pMot->decalage;

        /*        iQ=pProt->pResult->tMots[k].posQuery; */
        /*         iP=pProt->pResult->tMots[k].posQuery-pProt->pResult->tMots[k].decalage; */

        if (iP<0 || iQ <0) {
            fprintf(stderr, "EcritUnMolscript:: iP %d or iQ %d is negative, it shouldn't append (dec: %d, SHSP num %d prot %s)\n", iP, iQ,
                    pMot->decalage, k, pProt->nomProt );
            EcritUnResult(pQuery, pProt, stderr, rang, &stat, pPar);
            return(PASBON);
        }


        /*Tracer le morceau de la query qui n'est pas un SHSP*/
        /*Rouge*/
        /*         fprintf(pFiOut, "     set planecolour rgb 1 0 0;\n"); */
        /* query rouge sombre	*/
        fprintf(pFiOut, "     set planecolour rgb %3.1f %3.1f %3.1f;\n",
                (float)240/255, (float)40/255, (float)40/255);
        /*         fprintf(pFiOut, "     set planecolour hsb %f 0.6 0.6;\n", colorQ); */
        /*Rq: Il faut le nom de la chaine s'il y en a un*/
        fprintf(pFiOut, "     turn require molecule query and from %c%d to %c%d;\n",
                pQuery->tFrag[0].name, (int)(pQuery->tNumPDB[iQPrev].num[1]),
                pQuery->tFrag[0].name, (int)(pQuery->tNumPDB[iQ].num[1]));
        /*Tracer le morceau de la proteine banque qui n'est pas un SHSP*/
        /*bleu */
        fprintf(pFiOut, "     set planecolour rgb 0 0 1;\n");
        /* bank  bleu sombre	*/
        fprintf(pFiOut, "     set planecolour rgb %3.1f %3.1f %3.1f;\n",
                (float)40/255, (float)40/255, (float)240/255);
        /*        fprintf(pFiOut, "     set planecolour hsb %f 0.6 0.6;\n", colorP);*/
        fprintf(pFiOut, "     turn require molecule database and from %c%d to %c%d;\n\n",
                pProt->tFrag[0].name, (int)(pProt->tNumPDB[iPPrev].num[1]),
                pProt->tFrag[0].name, (int)(pProt->tNumPDB[iP].num[1]));

        /*Tracer le SHSP */
        fprintf(pFiOut, "     set planecolour rgb %3.1f %3.1f %3.1f;\n",
                (float)pProt->pResult->tCouleursMots[k].r/255,
                (float)pProt->pResult->tCouleursMots[k].g/255,
                (float)pProt->pResult->tCouleursMots[k].b/255);
        /*         fprintf(pFiOut, "     set planecolour hsb %f %f 1.0;\n", colorSHSP, saturationSHSP); */
        /*query*/
        fprintf(pFiOut, "     turn require molecule query and from %c%d to %c%d;\n",
                pQuery->tFrag[0].name, (int)(pQuery->tNumPDB[iQ].num[1]),
                pQuery->tFrag[0].name, (int)(pQuery->tNumPDB[iQ+pProt->pResult->tMots[k].lgMot-1].num[1]));
        /*database*/
        fprintf(pFiOut, "     turn require molecule database and from %c%d to %c%d;\n\n",
                pProt->tFrag[0].name, (int)(pProt->tNumPDB[iP].num[1]),
                pProt->tFrag[0].name, (int)(pProt->tNumPDB[iP+pProt->pResult->tMots[k].lgMot-1].num[1]));


        /*Mise a jour des couleurs des SHSP*/
        /*Il y a 20 couleurs: color va de 0  0.8 par pas de 0.2 */
        /*, puis de 0.1  0.9 */
        /*et la saturation peut etre de 1 ou de 0.5*/
        /*         if(colorSHSP < 0.8) */
        /*             colorSHSP+=0.2; */
        /*         else */
        /*           { */
        /*             if(colorSHSP==0.8) */
        /*                 colorSHSP=0.1; */
        /*             else */
        /*                 colorSHSP=0.0; */
        /*             if (saturationSHSP == 1) */
        /*                 saturationSHSP=0.5; */
        /*             else */
        /*                 saturationSHSP=1; */
        /*           } */

        iPPrev=iP+pProt->pResult->tMots[k].lgMot-1;
        iQPrev=iQ+pProt->pResult->tMots[k].lgMot-1;

    } /*fin for k*/

    /*Tracer les derniers morceaux des proteines*/
    if (iQPrev < pQuery->nbRes-1) {
        fprintf(pFiOut, "     set planecolour rgb %3.1f %3.1f %3.1f;\n",
                (float)240/255, (float)40/255, (float)40/255);
        /* 	fprintf(pFiOut, "     set planecolour rgb 1 0 0;\n"); */
        /*         fprintf(pFiOut, "     set planecolour hsb %f 0.6 0.6;\n", colorQ); */
        fprintf(pFiOut, "     turn require molecule query and from %c%d to %c%d;\n",
                pQuery->tFrag[0].name, (int)(pQuery->tNumPDB[iQPrev].num[1]),
                pQuery->tFrag[0].name, (int)(pQuery->tNumPDB[pQuery->nbRes-1].num[1]));

    }
    if (iPPrev < pProt->nbRes-1) {
        fprintf(pFiOut, "     set planecolour rgb %3.1f %3.1f %3.1f;\n",
                (float)40/255, (float)40/255, (float)240/255);
        /*         fprintf(pFiOut, "     set planecolour hsb %f 0.6 0.6;\n", colorP); */
        fprintf(pFiOut, "     turn require molecule database and from %c%d to %c%d;\n",
                pProt->tFrag[0].name, (int)(pProt->tNumPDB[iPPrev].num[1]),
                pProt->tFrag[0].name, (int)(pProt->tNumPDB[pProt->nbRes-1].num[1]));

    }

    fprintf(pFiOut, "end_plot\n");
    fprintf(pFiOut, "! ENDPROTEIN\n\n");

    /* 	fprintf(stderr, "1er Meilleur: %d Plus long %d (%d)\n", pProt->pResult->num1erSHSPMeilleurGp,longestSHSP, lengthLongestSHSP ); */

    return(BON);
}



/* --------------------------------------------	*/
/*Ecrit un fichier script pour Rasmol 		*/
/* --------------------------------------------	*/
static int EcritRasmolScript(tyProt *pQuery, tyProt *pProt, FILE *script, FILE *pdb, int shspnum) {

    double	*tmat, x, y, z, x2, y2, z2, ratio;
    tyResult	*pMot;
    int		iProt, iQuery, iMot, i, j, r, g, b;

    char		color[80];

    InitNamRes();

    if (pProt==NULL || script==NULL || pQuery==NULL) {
        fprintf(stderr,
                "EcritRasmolScript:: error: one of the parameter of EcritRasmolScript is NULL\n");
        return(PASBON);
    }

    /* si pdb est NULL, on ecrit tout dans script	*/
    if (pdb == NULL)
        pdb = script;

    fprintf(script, "load pdb inline\n");
    /* query rouge sombre	*/
    fprintf(script, "select *A\ncolor [240,40,40]\n");
    /* bank  bleu sombre	*/
    fprintf(script, "select *B\ncolor [40,40,240]\n");

    /* choix des couleurs	*/

    /***********************************************************/
    /*  Allocation du tableau contenant les couleurs des SHPS */
    /***********************************************************/
    NEWY(pProt->pResult->tCouleursMots, tyCouleurRGB, pProt->pResult->nbMots);

    for (iMot=0; iMot<pProt->pResult->nbMots; iMot++) {
        pMot=&(pProt->pResult->tMots[iMot]);


        iQuery=pMot->posQuery;
        iProt=pMot->posQuery-pMot->decalage;
        if (pProt->pResult->nbMots > 1)
            ratio = (float)iMot/(pProt->pResult->nbMots-1);
        else
            ratio = 1.0;
#define BASECOLOR	170
        if (iMot%2 == 0)
            r = 255 - (int)((255-BASECOLOR)*ratio);
        else
            r = BASECOLOR + (int)((255-BASECOLOR)*ratio);
        g = BASECOLOR + (int)((255-BASECOLOR)*ratio);

        if (iMot%2 == 1)
            b = 255 - (int)((255-BASECOLOR)*ratio);
        else
            b = BASECOLOR + (int)((255-BASECOLOR)*ratio);
#if 0
        if (iMot%2 == 0)
            r = 255 - (int)((255-BASECOLOR)*(float)iMot/pProt->pResult->nbMots);
        else
            r = 255;
        g = BASECOLOR + (int)((255-BASECOLOR)*(float)iMot/pProt->pResult->nbMots);
        if (iMot%2 == 1)
            b = 255 - (int)((255-BASECOLOR)*(float)iMot/pProt->pResult->nbMots);
        else
            b = 255;
#endif
        pProt->pResult->tCouleursMots[iMot].r=r;
        pProt->pResult->tCouleursMots[iMot].g=g;
        pProt->pResult->tCouleursMots[iMot].b=b;

        sprintf(color,"color [%d,%d,%d]\n", r,g,b);

        fprintf(script, "select *A & %d-%d\n",
                (int)(pQuery->tNumPDB[iQuery].num[1]),
                (int)(pQuery->tNumPDB[iQuery+pMot->lgMot-1].num[1]));
        fprintf(script, "%s\n", color);

        fprintf(script, "select *B & %d-%d\n",
                (int)(pProt->tNumPDB[iProt].num[1]),
                (int)(pProt->tNumPDB[iProt+pMot->lgMot-1].num[1]));
        fprintf(script, "%s\n", color);

    }

    fprintf(script, "select all\n");
    fprintf(script, "wireframe off\nbackbone 50\nselect none\nexit\nexit\n");

    fflush(script);

    fprintf(pdb, "HEADER    PROTEIN STRUCTURE ALIGNMENT BY YAKUSA\n");
    fprintf(pdb, "COMPND    %s [%d-%d] (chain A)  ; %s [%d-%d] (chain B)\n",
            pQuery->nomProt, (int)(pQuery->tNumPDB[0].num[1]),
            (int)(pQuery->tNumPDB[pQuery->nbRes].num[1]),
            pProt->nomProt, (int)(pProt->tNumPDB[0].num[1]),
            (int)(pProt->tNumPDB[pProt->nbRes].num[1]));


    /* Ecrire les matrices de rotations translations,	*/
    /* en REMARK la premiere est directement utilisee	*/
    /* pour le SHSP le plus long				*/
    /* Les suivantes sont en aussi en commentaire (REMARK)	*/
    if (pProt->pResult->tMatrixRT!=NULL) {
        /* fprintf(stderr, "EcritRasmolScript:: shspnum: %d / %d SHSP\n", shspnum,  pProt->pResult->nbMots); */
        pMot=&(pProt->pResult->tMots[shspnum]);
        iQuery=pMot->posQuery;
        iProt=pMot->posQuery-pMot->decalage;
        /* iQuery=pProt->pResult->tMots[shspnum].posQuery; */

        if (iProt<0 || iQuery <0) {
            fprintf(stderr,
                    "EcritRasmolScript 000:: iProt %d or iQuery %d is negative, it shouldn't append (dec: %d, SHSP num %d prot %s)\n",
                    iProt, iQuery, pMot->decalage, shspnum, pProt->nomProt );
            return(PASBON);
        }
        fprintf(pdb, "REMARK SHSP %3d iQuery: %4d - %4d iProt: %4d - %4d\n",
                shspnum+1, (int)(pQuery->tNumPDB[iQuery].num[1]),
                (int)(pQuery->tNumPDB[iQuery+pProt->pResult->tMots[shspnum].lgMot-1].num[1]),
                (int)(pProt->tNumPDB[iProt].num[1]),
                (int)(pProt->tNumPDB[iProt+pProt->pResult->tMots[shspnum].lgMot-1].num[1]));
        /*1er SHSP*/
        fprintf(pdb, "REMARK       transform in molecule database\n");
        /* keep tMatrixRT in pointer	*/
        tmat = pProt->pResult->tMatrixRT[shspnum];
        /*Ecrit Le vecteur de translation de la structure de la banque*/
        fprintf(pdb, "REMARK            by centre %f %f %f\n",
                tmat[3*3], tmat[3*3+1], tmat[3*3+2]);

        /*Ecrit la matrice de rotation de la structure de la banque*/
        fprintf(pdb, "REMARK        by rotation\n");
        for (i=0; i<3; i++) {
            fprintf(pdb, "REMARK           ");
            for (j=0; j<3; j++)
                fprintf(pdb, "%f ", tmat[i*3+j]);
            fprintf(pdb, "\n");
        }
        fprintf(pdb, "REMARK       ;\n");
        /*Ecrit Le vecteur de translation de la structure requete*/
        fprintf(pdb, "REMARK       transform in molecule query\n");
        /*Ecrit Le vecteur de translation de la structure de la requete*/
        fprintf(pdb, "REMARK            by centre %f %f %f;\n",
                tmat[4*3], tmat[4*3+1], tmat[4*3+2]);
        fprintf(pdb, "REMARK ENDSHSP \n");

#ifdef MORESHSP
        /*SHSP suivants*/
        for (k=0; k<pProt->pResult->nbMots; k++) {
            if (k==shspnum)
                continue;
            iQuery=pProt->pResult->tMots[k].posQuery;
            iProt=pProt->pResult->tMots[k].posQuery-pProt->pResult->tMots[k].decalage;

            if (iProt<0 || iQuery <0) {
                fprintf(stderr,
                        "EcritRasmolScript 111:: iP %d or iQ %d is negative, it shouldn't append (dec: %d, SHSP num %d prot %s)\n",
                        iProt, iQuery, pMot->decalage, k, pProt->nomProt );
                return(PASBON);
            }

            fprintf(pdb, "REMARK SHSP %3d iQuery: %4d - %4d iProt: %4d - %4d\n",
                    k+1, (int)(pQuery->tNumPDB[iQuery].num[1]),
                    (int)(pQuery->tNumPDB[iQuery+pProt->pResult->tMots[k].lgMot-1].num[1]),
                    (int)(pProt->tNumPDB[iProt].num[1]),
                    (int)(pProt->tNumPDB[iProt+pProt->pResult->tMots[k].lgMot-1].num[1]));
            /* fprintf(pFiOut, "! SHSP %3d\n", k+1); */

            /* keep tMatrixRT in pointer	*/
            tmat = pProt->pResult->tMatrixRT[k];
            fprintf(pdb, "REMARK       transform in molecule database\n");
            /*Ecrit Le vecteur de translation de la structure de la banque*/
            fprintf(pdb, "REMARK            by centre %f %f %f\n",
                    tmat[3*3], tmat[3*3+1], tmat[3*3+2]);

            /*Ecrit la matrice de rotation de la structure de la banque*/
            fprintf(pdb, "REMARK            by rotation\n");
            for (i=0; i<3; i++) {
                fprintf(pdb, "REMARK        ");
                for (j=0; j<3; j++)
                    fprintf(pdb, "%f ", tmat[i*3+j]);
                fprintf(pdb, "\n");
            }
            fprintf(pdb, "REMARK       ;\n");

            /*Ecrit Le vecteur de translation de la structure requete*/
            fprintf(pdb, "REMARK       transform in molecule query\n");
            /*Ecrit Le vecteur de translation de la structure de la banque*/
            fprintf(pdb, "REMARK            by centre %f %f %f;\n",
                    tmat[4*3], tmat[4*3+1], tmat[4*3+2]);
            fprintf(pdb, "REMARK ENDSHSP \n");

        }	/* end for k */

#endif	/* ifdef MORESHSP */
    } else {	/* pProt->pResult->tMatrixRT == NULL */
        fprintf(stderr, "EcritRasmolScript:: pb: no translation rotation matrix for RasmolScript\n");
        return(PASBON);
    }

    /* ----------------------------	*/
    /* set matrix to those of the	*/
    /* wanted shsp 			*/
    /* ----------------------------	*/


    tmat = pProt->pResult->tMatrixRT[shspnum];

    /* ----------------------------	*/
    /* imprime query == chaine A	*/
    /* ----------------------------	*/

    for (i = 0; i < pQuery->nbRes; i++) {
        fprintf(pdb, "ATOM  %5d  CA  %3s A%4d    %8.3f%8.3f%8.3f\n",
                i, GetNamRes(pQuery->tNameRes[i]), (int)(pQuery->tNumPDB[i].num[1]),
                pQuery->coords[i].x, pQuery->coords[i].y, pQuery->coords[i].z);
    }
    /*
    ATOM      0  N   ASN A  10      48.053  26.067  32.355
        12345678 1234
    */

    /* ----------------------------	*/
    /* imprime protbank == chaine B	*/
    /* ----------------------------	*/
    for (i = 0; i < pProt->nbRes; i++) {

        /* Translation 1 */
        x = pProt->coords[i].x - tmat[3*3];
        y = pProt->coords[i].y - tmat[3*3+1];
        z = pProt->coords[i].z - tmat[3*3+2];

        /* Rotation 2	*/
        x2 = x*tmat[0] + y*tmat[1] + z*tmat[2];
        y2 = x*tmat[3] + y*tmat[4] + z*tmat[5];
        z2 = x*tmat[6] + y*tmat[7] + z*tmat[8];

        /* Translation 3 */
        x2 = x2 + tmat[4*3];
        y2 = y2 + tmat[4*3+1];
        z2 = z2 + tmat[4*3+2];

        fprintf(pdb, "ATOM  %5d  CA  %3s B%4d    %8.3f%8.3f%8.3f\n",
                i, GetNamRes(pProt->tNameRes[i]), (int)(pProt->tNumPDB[i].num[1]),
                x2,y2,z2);

    }

    fprintf(pdb, "END\n");

    return(BON);

}
/**************************************************************************/
/* Temporaire*/
/**************************************************************************/
int EcritTousResultPourStat(tyProt *pQuery, FILE *pFiOut, tyParametres *pParms, tyStatResult *pStat) {
    int i;
    tyProt *pProt=NULL;

    if (pFiOut==NULL || pQuery==NULL || pParms == NULL || pStat == NULL) {
        fprintf(stderr,
                "Cannot write results (one parameters is NULL)\n");
        return(PASBON);
    }



    fprintf(pFiOut, "QUERY %s \n", pQuery->nomProt);


    i=0;
    pProt=Get_Prot_Fin(i);
    if (pProt==NULL) {
        fprintf(stderr, "EcritTousResultPourStat:: No protein returned from minimier function\n");
        return(PASBON);
    }
    while ( pProt !=NULL) {
        if (EcritUnResultStat(pQuery, pProt, pFiOut, i, pStat, pParms)==PASBON)
            fprintf(pFiOut, "Cannot ouput protein %d\n", i+1);
        i++;
        pProt=Get_Prot_Fin(i);
    }
    return(BON);

}


/*****************************************/
/*Ecriture des resultats*/
/*****************************************/
static int EcritUnResultStat(tyProt *pQuery, tyProt *pProtBank,
                             FILE *pFiOut, int rang, tyStatResult *pStat, tyParametres *pParms) {
    int k, i, j, iSHSP1, iSHSP2;

    int i1erSHSPMeilleur=pProtBank->pResult->num1erSHSPMeilleurGp; /*Indice dans tSHSPCompatibles du 1er SHSP du meilleur groupe*/

    int nbSHSPsMeilleur=0;

    int scoreSHSPsMeilleur=0;

    int lgCheminSHSPsMeilleur=0;

    double avgRMSCroisesMeilleur=-1;

    double logProbaMeilleur=0;

    double *tProba=NULL;

    int tSHSPsMeilleur[pProtBank->pResult->nbMots];
    int tSHSPsAutres[pProtBank->pResult->nbMots];

    tyResult *pMot;
    int iQuery, iProt;


    if (pQuery == NULL || pFiOut == NULL || pProtBank == NULL) {
        fprintf(stderr,"a parameter of EcritUnResultStat is NULL\n");
        return(PASBON);
    }

    logProbaMeilleur=pProtBank->scoreClassement;

    /*   fprintf(pFiOut, "Protein %d %6s score %10.2f Zscore %10.2f  ",  */
    /* 	  rang+1, pProtBank->nomProt, pProtBank->scoreClassement,  	 */
    /* 	  (pProtBank->scoreClassement-pStat->avgScores)/pStat->sigmaScores);  */


    /* Le tableau tCompatibles contient les compatibilits spatiales des SHSPs*/
    /* si  c'est le 1er d'un groupe, toujours l'indice du suivant multipli par -1*/
    /* ou le numero du SHSP suivant du groupe si c'est 1 SHSP d'un groupe hors 1er et dernier*/
    /*ou l'indice de la case si c'est la fin d'un groupe*/
    /* Et si le SHSP est seul, la case contiendra sont numero*-1*/
    switch (pParms->classement) {
    case CLASS_PROBA_INDEP_COMPATIBLES :
        tProba = pProtBank->pResult->tProbaIndep;
        break;
    case CLASS_PROBA_BI_COMPATIBLES :
        tProba = pProtBank->pResult->tProbaBi;
        break;
    case CLASS_PROBA_MTD_COMPATIBLES :
        tProba = pProtBank->pResult->tProbaMTD;
        break;
    }/*fin switch*/
    if (tProba == NULL) {
        fprintf(stderr, "EcritCompatiblesSpatiaux:: tProba is empty (NULL), should not append!!!\n");
        return(PASBON);
    }




    /****************************/
    /*tSHSPsMeilleur contient les indices*/
    /* des SHSP du meilleur gp dans l'ordre*/
    /****************************/
    /*Range les numeros du meilleur groupe dans tSHSPsMeilleur*/
    lgCheminSHSPsMeilleur=pProtBank->pResult->tMots[i1erSHSPMeilleur].lgMot;
    nbSHSPsMeilleur=1;
    tSHSPsMeilleur[0]=i1erSHSPMeilleur;


    if (pProtBank->pResult->tSHSPCompatibles[i1erSHSPMeilleur]*-1 != i1erSHSPMeilleur) { /* S'il n'est pas tout seul*/
        i=-1*pProtBank->pResult->tSHSPCompatibles[i1erSHSPMeilleur];
        k=1;

        /*     fprintf(stderr, "1er %d\n",i1erSHSPMeilleur);  */
        while (i!=pProtBank->pResult->tSHSPCompatibles[i]) {
            /*        fprintf(stderr, "i %d, tSHSPCompatibles[i]: %d\n", i, pProtBank->pResult->tSHSPCompatibles[i]); */
            tSHSPsMeilleur[k]=i;
            lgCheminSHSPsMeilleur+=pProtBank->pResult->tMots[i].lgMot;
            nbSHSPsMeilleur++;
            k++;
            i=pProtBank->pResult->tSHSPCompatibles[i];

        }
        tSHSPsMeilleur[k]=i;
        lgCheminSHSPsMeilleur+=pProtBank->pResult->tMots[i].lgMot;
        nbSHSPsMeilleur++;
        qsort(tSHSPsMeilleur,nbSHSPsMeilleur,sizeof(int), (int (*)(const void * , const void * ))sCompar);
    }
    /*   fprintf(stderr, "1er meilleur SHSP: %d -> %d, nb SHSP %d\n", i1erSHSPMeilleur,  */
    /* 	  pProtBank->pResult->tSHSPCompatibles[i1erSHSPMeilleur], nbSHSPsMeilleur); */
    scoreSHSPsMeilleur=0;
    avgRMSCroisesMeilleur=0;
    if (nbSHSPsMeilleur > 1) {
        for (iSHSP1=0; iSHSP1<nbSHSPsMeilleur-1; iSHSP1++) {
            scoreSHSPsMeilleur+=pProtBank->pResult->tMots[tSHSPsMeilleur[iSHSP1]].score;
            for (iSHSP2=iSHSP1+1; iSHSP2<nbSHSPsMeilleur; iSHSP2++) {
                avgRMSCroisesMeilleur+=pProtBank->pResult->tRMSDCroises[tSHSPsMeilleur[iSHSP1]*pProtBank->pResult->nbMots+tSHSPsMeilleur[iSHSP2]];
                avgRMSCroisesMeilleur+=pProtBank->pResult->tRMSDCroises[tSHSPsMeilleur[iSHSP1]+pProtBank->pResult->nbMots*tSHSPsMeilleur[iSHSP2]];
            }
        }
        avgRMSCroisesMeilleur=avgRMSCroisesMeilleur/((nbSHSPsMeilleur-1)*2);
    } else {
        avgRMSCroisesMeilleur=-1;
    }
    /****************************/
    /*tSHSPsAutres contient les indices*/
    /* des SHSP hors meilleur gp, dans l'ordre*/
    /****************************/
    k=0;
    i=0;
    j=0;
    while (i< pProtBank->pResult->nbMots) {
        if (i<tSHSPsMeilleur[k] || i>=nbSHSPsMeilleur) {
            tSHSPsAutres[j]=i;
            j++;
            i++;
        } else {
            k++;
            i++;
        }
    }


    /*     for(i=0; i< nbSHSPsMeilleur; i++) */
    /*       fprintf(pFiOut, " %d ", tSHSPsMeilleur[i]+1); */
    /*     fprintf(pFiOut, "\n"); */
    /*     for(i=0; i< pProtBank->pResult->nbMots-nbSHSPsMeilleur; i++) */
    /*       fprintf(pFiOut, " %d ", tSHSPsAutres[i]+1); */
    /*     fprintf(pFiOut, "\n"); */

    /*      fprintf(stderr, "Meilleur Groupe nbSHSP: %d score %f score classement %f avg RMS %f\n", */
    /* 	     nbSHSPMeilleurGroupe, logProbaGpCourant, logProbaMeilleur, avgRMSCroises); */
    /* fprintf(pFiOut, " Meilleur groupe nbSHSPCompatibles %2d  avgRMSCroises %8.2f, score: %d logProba %f i 1er %d\n", */
    /* 	nbSHSPsMeilleur, avgRMSCroisesMeilleur, scoreSHSPsMeilleur, logProbaMeilleur, i1erSHSPMeilleur); */


    /*Format de sortie: */
    /*   Ligne 1 Info proteine:  */
    /* PROT <rang> <nomProt> <lg prot> <codePDB et desc>  */
    /* Ligne 2 Divers info si on veut des support vector machines*/
    /* INFO <nbSHSPs compatibles> <avg des RMS croises des compatibles> <Somme des scores S des SHSPs compatibles> <lg ali SHSP compatibles>*/
    /* Ligne 3 score compatibles:  */
    /* SCORESC <score log MTD (de classement)> <score centr reduit> <score reduit> <score / dist inter quartile> */
    /* Ligne 4 score avec tous les SHSPs (pas reduit... car ecart type et autres ne sont pas calcules*/
    /* SCOREST <score Log MTD>*/
    /* Ligne 5: SHSP compatibles, pour chaque SHSP */
    /* COMPATIBLES <num PDB 1er CA du SHSP dans Query>-<num PDB dernier CA dans Query>:<1er CA dans Prot>-<dernier CA dans Prot>:<score MTD>*/
    /* Ligne 6: autres SHSP , pour chaque SHSP */
    /* AUTRES <num PDB 1er CA du SHSP dans Query>-<num PDB dernier CA dans Query>:<1er CA dans Prot>-<dernier CA dans Prot>:<score MTD>*/



    fprintf(pFiOut, "PROT %d %s %d %s\n", rang+1, pProtBank->nomProt, pProtBank->nbRes, pProtBank->desc);

    fprintf(pFiOut, "INFO %d %5.2f %d %d\n", nbSHSPsMeilleur, avgRMSCroisesMeilleur, scoreSHSPsMeilleur, lgCheminSHSPsMeilleur);

    fprintf(pFiOut, "SCORESC %10.2f %10.2f %10.2f %10.2f\n",
            pProtBank->scoreClassement,
            (pProtBank->scoreClassement-pStat->avgScores)/pStat->sigmaScores,
            pProtBank->scoreClassement/pStat->sigmaScores,
            pProtBank->scoreClassement/pStat->interQuartiles*-1);

    fprintf(pFiOut, "SCOREST %10.2f\n",
            pProtBank->pResult->sumLogProbaMTD*-1);

    fprintf(pFiOut, "COMP   ");
    for (i=0; i<nbSHSPsMeilleur; i++) {
        pMot=&(pProtBank->pResult->tMots[tSHSPsMeilleur[i]]);
        iQuery=pMot->posQuery;
        iProt=pMot->posQuery-pMot->decalage;
        fprintf(pFiOut, "%d%c %d%c : %d%c %d%c : %.2E | ",
                pQuery->tNumPDB[iQuery].num[1], pQuery->tNumPDB[iQuery].inser[0],
                pQuery->tNumPDB[iQuery+pMot->lgMot-1].num[1], pQuery->tNumPDB[iQuery+pMot->lgMot-1].inser[0],
                pProtBank->tNumPDB[iProt].num[1], pProtBank->tNumPDB[iProt].inser[0],
                pProtBank->tNumPDB[iProt+pMot->lgMot-1].num[1], pProtBank->tNumPDB[iProt+pMot->lgMot-1].inser[0],
                pProtBank->pResult->tProbaMTD[tSHSPsMeilleur[i]]);
    }
    fprintf(pFiOut, "\n");

    fprintf(pFiOut, "AUTRES ");
    for (i=0; i<pProtBank->pResult->nbMots-nbSHSPsMeilleur; i++) {
        pMot=&(pProtBank->pResult->tMots[tSHSPsAutres[i]]);
        iQuery=pMot->posQuery;
        iProt=pMot->posQuery-pMot->decalage;
        fprintf(pFiOut, "%d%c %d%c : %d%c %d%c :%.2E | ",
                pQuery->tNumPDB[iQuery].num[1], pQuery->tNumPDB[iQuery].inser[0],
                pQuery->tNumPDB[iQuery+pMot->lgMot-1].num[1], pQuery->tNumPDB[iQuery+pMot->lgMot-1].inser[0],
                pProtBank->tNumPDB[iProt].num[1], pProtBank->tNumPDB[iProt].inser[0],
                pProtBank->tNumPDB[iProt+pMot->lgMot-1].num[1], pProtBank->tNumPDB[iProt+pMot->lgMot-1].inser[0],
                pProtBank->pResult->tProbaMTD[tSHSPsAutres[i]]);
    }
    fprintf(pFiOut, "\n");
    fprintf(pFiOut, "\n");

    return(BON);
}




/*********************************************************************/
/* Fonction de comparaison pour qsort*/
/* classe en ordre croissant         */
/*********************************************************************/
static int sCompar(int *x, int *y) {
    return( (*x < *y) ? -1 : (*x > *y ? +1 : 0));


} /* end of int sCompar(x, y) */




/* --------------------------------------------	*/
/* init the 3 letter code array indexed		*/
/* on  1 letter code				*/
/* --------------------------------------------	*/
static void InitNamRes(void) {
    int	i;

    if (sNamRes == NULL) {
        NEWY(sNamRes, cquatre, 255);
    }

    for (i = 0; i < 255; i++) {
        strcpy(sNamRes[i], "XXX");
    }
    strcpy(sNamRes[(int)'A'],"ALA");
    strcpy(sNamRes[(int)'C'],"CYS");
    strcpy(sNamRes[(int)'D'],"ASP");
    strcpy(sNamRes[(int)'E'],"GLU");
    strcpy(sNamRes[(int)'F'],"PHE");
    strcpy(sNamRes[(int)'G'],"GLY");
    strcpy(sNamRes[(int)'H'],"HIS");
    strcpy(sNamRes[(int)'I'],"ILE");
    strcpy(sNamRes[(int)'K'],"LYS");
    strcpy(sNamRes[(int)'L'],"LEU");
    strcpy(sNamRes[(int)'M'],"MET");
    strcpy(sNamRes[(int)'N'],"ASN");
    strcpy(sNamRes[(int)'P'],"PRO");
    strcpy(sNamRes[(int)'Q'],"GLN");
    strcpy(sNamRes[(int)'R'],"ARG");
    strcpy(sNamRes[(int)'S'],"SER");
    strcpy(sNamRes[(int)'T'],"THR");
    strcpy(sNamRes[(int)'V'],"VAL");
    strcpy(sNamRes[(int)'W'],"TRP");
    strcpy(sNamRes[(int)'Y'],"TYR");
    strcpy(sNamRes[(int)'Z'],"ZZZ");

    return;

}

/* --------------------------------------------	*/
/* return the 3 letter code of 1 letter code	*/
/* --------------------------------------------	*/
static char *GetNamRes(char c) {
    return sNamRes[(int)c];
}







/************************************************************************/
/* EcritTousInndentiteFROST						*/
/*     Ecrire les identites pour FROST					*/
/************************************************************************/
int EcritTousInndentiteFROST(tyProt *pQuery, FILE *pFiOut, tyParametres *pParms, tyStatResult *pStat) {

    int i;
    tyProt *pProt=NULL;

    if (pFiOut==NULL || pQuery==NULL || pParms == NULL || pStat == NULL) {
        fprintf(stderr,
                "Cannot write results (one parameters is NULL)\n");
        return(PASBON);
    }



    fprintf(pFiOut, "%s \n", NAME_SEQ_LENGTH);
    /* Fait expres d'inverser Length et query	*/
    /* pour faciliter parsage			*/


    i=0;
    pProt=Get_Prot_Fin(i);
    if (pProt==NULL) {
        fprintf(stderr, "EcritTousResult:: No protein returned from minimier function\n");
        return(PASBON);
    }
    while ( pProt !=NULL) {
        pProt=Get_Prot_Fin(i);
    }
    return(BON);
}
/***********************************************************************/
/*
 */
/*************************************************************************/
int EcritTousMolscript(tyProt *pQuery, FILE *pFiOut, tyParametres *pParms,  int nbStructVoulues) {

    int i;
    tyProt *pProt=NULL;

    if (pFiOut==NULL || pQuery==NULL || pParms == NULL ) {
        fprintf(stderr, "Ecriture du fichier pour molScript impossibles (un des parametres est null)\n");
        return(PASBON);
    }



    i=0;
    pProt=Get_Prot_Fin(i);
    if (pProt==NULL) {
        fprintf(stderr, "EcritTousMolscript:: No protein returned from the minimier\n");
        return(PASBON);
    }
    fprintf(pFiOut, "! MOLSCRIPT Data File, generated by yakusa\n");
    while ( pProt !=NULL) {
        if (EcritUnMolscript(pQuery, pProt, pFiOut, pParms, i+1)==PASBON)
            fprintf(stderr, "impossible d'afficher la proteine %d\n", i+1);
        i++;
        pProt=Get_Prot_Fin(i);
    }

    return(BON);
}


/* --------------------------------------------------------------------	*/
/* --------------------------------------------------------------------	*/
int EcritCeRasMolscript(tyProt *pQuery, FILE *script, FILE *pdb, int rangvoulu, int shspnum) {

    int longestSHSP, lengthLongestSHSP, i;
    tyProt *pProt=NULL;

    if (script==NULL || pQuery==NULL) {
        fprintf(stderr, "EcritCeRasMolscript:: cannot write of Rasmol script file :: a parameter is NULL\n");
        return(PASBON);
    }

    if ((pProt=Get_Prot_Fin(rangvoulu)) == NULL) {
        fprintf(stderr, "EcritCeRasMolscript:: No protein returned from the minimier\n");
        return(PASBON);
    }

    if (shspnum < 0 || shspnum >= pProt->pResult->nbMots) {

        /*Par defaut, aligne sur le plus long des SHSP des SHSPs compatibles du mailleur gp -> le trouver*/

        if (pProt->pResult->tSHSPCompatibles[pProt->pResult->num1erSHSPMeilleurGp] > 0) {
            fprintf(stderr, "1er meilleur %d -> %d au milieu d'un groupe !!! \n",pProt->pResult->num1erSHSPMeilleurGp,
                    pProt->pResult->tSHSPCompatibles[pProt->pResult->num1erSHSPMeilleurGp]);
            return(PASBON);
        }
        /*     fprintf(stderr, "1er meilleur %d -> %d lg %d\n",pProt->pResult->num1erSHSPMeilleurGp,  */
        /* 	    pProt->pResult->tSHSPCompatibles[pProt->pResult->num1erSHSPMeilleurGp], */
        /* 	    pProt->pResult->tMots[pProt->pResult->num1erSHSPMeilleurGp].lgMot); */
        longestSHSP=pProt->pResult->num1erSHSPMeilleurGp;
        lengthLongestSHSP=pProt->pResult->tMots[pProt->pResult->num1erSHSPMeilleurGp].lgMot;
        if (pProt->pResult->tSHSPCompatibles[pProt->pResult->num1erSHSPMeilleurGp]*-1 !=
                pProt->pResult->num1erSHSPMeilleurGp) { /* S'il n'est pas tout seul*/
            i=-1*pProt->pResult->tSHSPCompatibles[pProt->pResult->num1erSHSPMeilleurGp];
            /*     fprintf(stderr, "1er %d\n",i1erSHSPMeilleur);  */
            while (i!=pProt->pResult->tSHSPCompatibles[i]) {
                /* 	fprintf(stderr, "suivant: %d  -> %d lg %d\n", i,  */
                /* 	    pProt->pResult->tSHSPCompatibles[i], */
                /* 	    pProt->pResult->tMots[i].lgMot); */
                /*       fprintf(stderr, "i %d, tSHSPCompatibles[i]: %d\n", i, pProtBank->pResult->tSHSPCompatibles[i]); */

                if (lengthLongestSHSP < pProt->pResult->tMots[i].lgMot) {
                    longestSHSP=i;
                    lengthLongestSHSP=pProt->pResult->tMots[i].lgMot;
                }
                i=pProt->pResult->tSHSPCompatibles[i];

            }
            /* 	fprintf(stderr, "dernier: %d  -> %d lg %d\n", i,  */
            /* 	    pProt->pResult->tSHSPCompatibles[i], */
            /* 	    pProt->pResult->tMots[i].lgMot); */
            if (lengthLongestSHSP < pProt->pResult->tMots[i].lgMot) {
                longestSHSP=i;
                lengthLongestSHSP=pProt->pResult->tMots[i].lgMot;
            }
        }
        shspnum=longestSHSP;
    }

    if (EcritRasmolScript(pQuery, pProt, script, pdb, shspnum) == PASBON)
        fprintf(stderr, "impossible d'afficher la proteine %d\n", rangvoulu+1);

    return(BON);
}



#endif
