/**************************************************************************/
/*                                                                        */
/* statistiques pour les sequences nucleiques et proteiques               */
/* issu de SSR                                                            */
/* modifie en mai 2005                                                    */
/*                                                                        */
/**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>


#include "macros.h"
#include "libfasta.h"
#include "swelfe.h"
#include "lecture.h"
#include "swalign.h"
#include "sim.h"
#include "stats_sequence.h"
#include "kontrol.proto.h"  /*pour controler la memoire*/


#define EPS FLT_MIN
#define RNMX    (1.0-EPS)
#define FLT_MIN 1.2e-7


#ifndef INFINI
#define INFINI 10.0e5
#endif

/*#define MEMOIRE 1*/

static char Codons[4*4*4]={
    '0','1','2','3',  '4','5','6','7',  '8','9','A','B',  'C','D','E','F',
    'G','H','I','J',  'K','L','M','N',  'O','P','Q','R',  'S','T','U','V',
    'W','X','Y','Z',  'a','b','c','d',  'e','f','g','h',  'i','j','k','l',
    'm','n','o','p',  'q','r','s','t',  'u','v','w','x',  'y','z','-', '_'
};



static int PasCodons[123]={
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  -1,-1,-1,-1,-1,62,-1,-1, 0, 1,    2, 3, 4, 5, 6, 7, 8, 9,-1,-1,
    -1,-1,-1,-1,-1,10,11,12,13,14,   15,16,17,18,19,20,21,22,23,24, 25,26,27,28,29,30,31,32,33,34,
    35,-1,-1,-1,-1, 63,-1,36,37,38,   39,44,41,42,43,44,45,46,47,48,  49,50,51,52,53,54,55,56,57,58,
    59,60,61
};

/* ---------------------------------------------------- */
/* Unix builtin random number generator         */
/* returns a uniform random deviate between 0. and 1.   */
/* (exclusive of the endpoint values).          */
/* period ~ 3.10^10 - ?                 */
/* ---------------------------------------------------- */

void sranUN(seed)
long seed;
{
    (void) srandom(seed);
}

/* -------------------------------- */

float ranUN()
{
    double temp;

    temp = (double) random() / (double) MBIG_UN;

    return ((temp < EPS) ? EPS : ((temp > RNMX) ? RNMX : (float) temp));
}


/* -------------------------------------------- */
/* my power function                    */
/* -------------------------------------------- */

static float MyPowbis(float elt, int pow)
{
    float res;
    int i;

    for (i=0, res=1.0; i<pow; i++)
        res=res*elt;

    return res;
}


/* -------------------------------------------- */
/* bug specific erreur                  */
/* -------------------------------------------- */
static void BugErreur(msg, stat)
char          *msg;
int       stat;
{
    char buffer[256];

    sprintf(buffer, "BUG: %s <Call Police>", msg);

    Error(buffer, stat);
}

/* ------------------------------------------------ */
/* built frequency hashtable for k-1 length words   */
/* ------------------------------------------------ */
int BuiltHashTable(hash, alphabet, ksize)
HashTable   *hash;
char    *alphabet;
int     ksize;
{
    int     i, bitsize;

    long   k, lk, pk;

    strcpy(hash->alphabet, alphabet);

    hash->ksize = ksize;

    hash->alphasize = strlen(alphabet);

    hash->tsum = 0;

    /* compute bitshift = log2(alphasize) */

    lk = 0;
    for (pk = 1 ; pk < hash->alphasize ; pk *= 2)
        lk++;

    hash->bitshift = lk;
    hash->wsize    = pk;

    bitsize = (hash->ksize - 1) * hash->bitshift;

    if (bitsize >= sizeof(long) * 8)
        Error("k-uple too big with this alphabet", 20);

    /* compute masks                */
    lk = 0x1;
    for (i = 1 ; i < bitsize ; i++)
        lk = (lk << 1) | 0x1;

    hash->wmask = lk;

    lk = 0x1;
    for (i = 1 ; i < hash->bitshift ; i++)
        lk = (lk << 1) | 0x1;

    hash->smask = lk;

    /* compute table size  = wsize^(ksize-1) */

    pk = 1;
    for (i = 1 ; i < hash->ksize ; i++)
        pk *= hash->wsize;

    hash->tsize = pk;

    /* allocate memory           */

    hash->table = NEWN(int *, hash->tsize);

    if (! hash->table)
        Error("not enough memory for hash table", 10);

    for (k = 0 ; k < hash->tsize ; k++) {
        hash->table[k] = NEWN(int, hash->alphasize + 1);
        if (! hash->table[k])
            Error("not enough memory for hash table", 10);
        for (i = 0 ; i <= hash->alphasize ; i++)
            hash->table[k][i] = 0;
    }

    return 0;
}

/* ------------------------------------------------ */
/* free frequency hashtable             */
/* ------------------------------------------------ */
void FreeHashTable(hash)
HashTable   *hash;
{
    int i;

    for (i = 0 ; i < hash->ksize ; i++)
        MYFREE(hash->table[i]);

    MYFREE (hash->table);

}

/* ------------------------------------------------ */
/* compute hash value                   */
/* ! entries are computed only on the k-1 first     */
/* symbols                      */
/* ------------------------------------------------ */
static long HashValue(word, hash)
char      *word;    /* encoded */
HashTable *hash;
{
    int i, ii, h;

    ii = hash->ksize - 1;

    if (! ii)
        return 0;   /* special case for ksize == 1 */

    h = word[0];

    for (i = 1 ; i < ii ; i++) {
        h <<= hash->bitshift;
        h |= (int) word[i];
    }

    return h;
}

/* ------------------------------------------------ */
/* Invert function: computes word (non encoded)     */
/* returns NULL if not a word               */
/* ------------------------------------------------ */
static char *ShaValue(h, hash)
long     h;
HashTable *hash;
{
    int   i, j;
    char *b;

    static char sBuf[512];

    b = sBuf + hash->ksize - 2;

    for (i = 0 ; i < (hash->ksize - 1) ; i++, b--) {
        j = h & hash->smask;
        if (j >= hash->alphasize)
            return NULL;
        *b = hash->alphabet[j];
        h >>= hash->bitshift;
    }

    sBuf[hash->ksize - 1] = sBuf[hash->ksize] = '\000';

    return sBuf;
}

/* ------------------------------------------------ */
/* update hash value                    */
/* should be inline                 */
/* ------------------------------------------------ */
static long UpdateHashValue(old, newsymb, hash)
long     old;
int       newsymb;  /* aka encoded char */
HashTable *hash;
{
    return  (hash->ksize == 1)
            ? 0
            : ((old << hash->bitshift) & hash->wmask) | newsymb;
}

/* ------------------------------------------------ */
/* reset frequency of k-uples to 0          */
/* ------------------------------------------------ */
void ResetFrequencies(HashTable *hash)
{
    int  i, j;

    for (i = 0 ; i < hash->tsize ; i++) {
        for (j = 0 ; j <= hash->alphasize ; j++)
            hash->table[i][j] = 0;
    }

    hash->tsum = 0;
}

/* ------------------------------------------------ */
/* encode sequence                  */
/* ------------------------------------------------ */
void EncodeSequence(char *seq, HashTable *hash)
{
    int  pos, i=0;
    char *cseq;

    for (cseq = seq , i=0; *cseq ; cseq++, i++) {
        pos = (int) (strchr(hash->alphabet, *cseq) - hash->alphabet);
        *cseq = (char) pos;
    }

    *cseq = EOS; /* marker */
}
/* ------------------------------------------------ */
/* compute frequency of k-uples             */
/* ------------------------------------------------ */
void ComputeFrequencies(char *seq, HashTable *hash, int len)
{
    long  h, up;
    int i;
    char   *cseq;

    if (len < hash->ksize)
        return;

    up = hash->alphasize;

    h = HashValue(seq, hash);


    for (i=0, cseq = seq + hash->ksize - 1; *cseq != EOS ; cseq++, i++) {
        hash->table[h][(int)*cseq]++;   /* this count   */
        hash->table[h][up]++;       /* sum for h    */

        hash->tsum++;           /* sum of all   */

        h = UpdateHashValue(h, *cseq, hash);
    }
}

/* ------------------------------------------------ */
/* return uniform random number in [from, to[       */
/* ------------------------------------------------ */
static long UniformRandom(from, to, ran)
long   from, to;
RanFunc ran;
{
    long i;

    i  = ran() * (to - from);   /* will truncate to integer part    */
    /* in [0, (to-from)[            */
    i += from;          /* integer shift origin         */

    if (i >= to)        /* very unlikely but == may appear  */
        i--;         /* from truncature error        */

    return i;
}

/* ------------------------------------------------ */
/* generate random mono sequence            */
/* exact case                           */
/* by permutations in shrinking                     */
/* ------------------------------------------------ */
static void ShuffleMonoRandom(char *seq, RanFunc ran,  int len)
{
    long i, j;
    char  *cseq, tmp;

    cseq = seq;

    for (i = 0 ; i < len ; i++) {

        /* -- draw integer in [i, seqlen[ -- */
        j = UniformRandom(i, len, ran);

        /* -- swap values ------------------ */
        tmp = cseq[j];
        cseq[j] = cseq[i];
        cseq[i] = tmp;
    }
}

/* ------------------------------------------------ */
/* return non-uniform random number         */
/* according to distribution in dist            */
/* ------------------------------------------------ */
static long DistribRandom(int *dist, long siz, RanFunc ran)
{
    long i, sum, dsum, unif;

    dsum = dist[siz];       /* where sum is stored      */

    /* draw int. in [i, dsum]   */
    unif = UniformRandom(0, dsum+1, ran);
    /* search on repart.        */
    for (i = sum = 0; i < siz ; i++) {
        sum += dist[i];
        if (unif <= sum)
            return i;
    }
    BugErreur("(i >= siz) in DistribRandom", 22);

    return (siz-1);
}

/* ------------------------------------------------ */
/* return non-uniform random number         */
/* according to distribution in hash            */
/* Note: a cleaner way is to :              */
/* -1- compute (alphasize)^ksize distribution from  */
/*     hash->table values.              */
/* -2- call DistribRandom               */
/* but, this will requires an extra storage     */
/* of (alphasize)^ksize long                */
/* so here is a 'schmutziger' version :(        */
/* ------------------------------------------------ */
static long HashRandom(hash, ran)
HashTable *hash;
RanFunc ran;
{
    long i, j, sum, unif;

    /* draw int. in [i, tsum]   */
    unif = UniformRandom(0, hash->tsum+1, ran);

    /* search on repart.        */
    for (i = sum = 0; i < hash->tsize ; i++) {
        for (j = 0 ; j < hash->alphasize ; j++) {
            sum += hash->table[i][j];
            if (unif <= sum)
                return (i*hash->alphasize + j);
        }
    }

    BugErreur("(i,j >= siz) in HashRandom", 22);

    return (hash->tsize*hash->alphasize - 1);
}

/* ------------------------------------------------ */
/* generate random poly sequence                    */
/* approximate case (generate)                      */
/* ------------------------------------------------ */
static void GeneratePolyRandom(char *seq, HashTable *hash,
                               RanFunc ran, int len)
{
    long i, j, h, k, newlen;
    char  *cseq;

    cseq = seq;
    newlen = strlen(seq);

    /* -- choose initial ktuple -------- */

    /* choose a ktuple           */
    i = HashRandom(hash, ran);  /* using kuples frequencies      */

    j = i % hash->alphasize;        /* kth symbol            */

    h = (i-j) / hash->alphasize;    /* hash of first (k-1)uple       */

    /* copy first kuple          */
    strncpy(cseq, ShaValue(h, hash), hash->ksize - 1);
    EncodeSequence(cseq, hash);
    cseq[hash->ksize - 1] = '\000';     /* EOS for encoding  */
    cseq[hash->ksize - 1] = j;          /* kth symbol        */

    /* hash of second (k-1)tuple     */
    h = UpdateHashValue(h, j, hash);

    /* -- choose next ktuples ---------- */

    for (k=hash->ksize , cseq = seq + hash->ksize ; k<len; cseq++, k++) {
        *cseq = DistribRandom(hash->table[h], hash->alphasize, ran);
        h = UpdateHashValue(h, *cseq, hash);
    }
}

/* ------------------------------------------------ */
/* generate random sequence (allopt=0=> shuffle not exact)*/
/* ------------------------------------------------ */
static void GenerateRandom(char *seq, HashTable *hash, RanFunc ran,
                           int len, int allopt)
{
    if (len < hash->ksize)
        return;             /* huh ?        */

    if (hash->ksize == 1 && allopt==0)
        ShuffleMonoRandom(seq, ran, len);
    else
        GeneratePolyRandom(seq, hash, ran, len);
}

/* ------------------------------------------------ */
/* decode sequence                  */
/* ------------------------------------------------ */
void DecodeSequence(char *seq, HashTable *hash)
{
    char *cseq;

    for (cseq = seq ; *cseq != EOS ; cseq++)
        *cseq = hash->alphabet[(int)*cseq];

    *cseq = '\000';
}
#if 0
/* ------------------------------------------------ */
/* decode sequence                  */
/* ------------------------------------------------ */
void DecodeSequence(char *seq, HashTable *hash, int lg)
{
    /* char *cseq; */
    int i;
    for (i=0; i<lg; i++)
        seq[i] = hash->alphabet[seq[i]];

    /*    *cseq = '\000'; */
    *cseq = '\0';
}
#endif

/* ------------------------------------------------ */
/* compute comparative frequency tables         */
/* and optionnally print it             */
/* debug                        */
/* ------------------------------------------------ */

static double ComputeStatistics(datahash, shufhash, flen)
HashTable *datahash, *shufhash;
int flen;
{
    long i, j, s1, f1, f2, s2, nu, last, diff, sum;
    double lchi, chi, prob;
    char  *buf;

    last = datahash->ksize-1;

    s1 = nu = 0;
    s2 = chi= 0.;

    for (i = 0 ; i < datahash->tsize ; i++) {

        if (! (buf = ShaValue(i, datahash)))
            continue;

        for (j = 0 ; j < datahash->alphasize ; j++) {

            buf[last] = datahash->alphabet[j];

            f1 = datahash->table[i][j];
            f2 = shufhash->table[i][j];
            s1 += f1;
            s2 += f2;
            diff = f1 - f2;
            sum  = f1 + f2;
            nu++;

            if (sum == 0) {
                lchi = 0.;
                nu--;
            }
            else
                lchi = diff * diff / sum;

            chi += lchi;
        }
    }
    prob = 1. - gammq((double)(0.5*(double)nu), (double)(0.5* chi));

    return prob;
}




/* -------------------------------------------- */
/* shuffle a sequence                           */
/* -------------------------------------------- */

static int shuffleSeq (double pval, HashTable *data_hash,
                       HashTable *shuf_hash, char *seq, int len, RanFunc rfunc)
{

    int iter;
    double prob;

    prob = pval;

    iter = 0;


    if (data_hash->ksize == 1)  /* since for 1 it is exact ... */
        GenerateRandom(seq, data_hash, rfunc, len, 0);
    else {
        do {
            iter++;

            ResetFrequencies(shuf_hash);
            GenerateRandom(seq, data_hash, rfunc, len, 0);
            ComputeFrequencies(seq, shuf_hash, len);
            prob = ComputeStatistics(data_hash, shuf_hash, len);

            /*      fprintf(stderr, "iter: %d prob=%.2g pval=%f, factor=%i\n",
                        iter, prob, pval, len);  */
            DecodeSequence(seq,data_hash);

        } while ((prob > pval) && (iter < MAX_PITER));
    }

    if (iter > MAX_PITER)
        return 0;

    return 1;
}
/***********************************************************/
/*  fonction qui copie une tyProt et renvoie la nouvelle   */
/***********************************************************/



void Copie_tyProt (tyProt *original, tyProt *copie) {

    int i;
    (void)strncpy (copie->nomProt, original->nomProt, PATH_MAX);
    (void)strncpy (copie->desc, original->desc, DESC_LEN );

    copie->nbRes =  original->nbRes ;

    for (i=0; i< original->nbRes; i++)
        copie->tAngles[i] =  original->tAngles[i];

    (void)strcpy (copie->tNameRes, original->tNameRes);

}

void Reinitialise_Prot (tyProt *new_seq, tyProt *original) {

    int i;

    (void)strcpy (new_seq->tNameRes, original->tNameRes);

    for (i=0; i< original->nbRes; i++)
        new_seq->tAngles[i] =  original->tAngles[i];


}


int corres(char c) {

    switch (c) {
    case 'A' :
        return 0;
    case 'C' :
        return 1;
    case 'G' :
        return 2;
    case 'T' :
        return 3;

    }
    return -1;
}

char inverse(int c) {

    switch (c) {
    case 0 :
        return 'A';
    case 1 :
        return 'C';
    case 2 :
        return 'G';
    case 3 :
        return 'T';

    }
    return -1;
}


char* codons(char *seq, int longueur) {

    int i,j;
    char *tempo, codon1='\0', codon2='\0', codon3='\0';
    int cod1 =-1, cod2=-1, cod3=-1;
    tempo = malloc(((int)(longueur/3)+1)*sizeof(char));
    if (! tempo )
        MyError(stderr,3,"swalign:: not enough memory for tempo file stat_sequences.c\n");

    /*          fprintf(stderr,"seq %s longeur %d\n", seq, longueur);*/


    for (i=0,j=0;i<(int)(longueur/3); i++, j+=3) {
        codon1 = seq[j];
        codon2 = seq[j+1];
        codon3 = seq[j+2];
        cod1 = corres(codon1);
        cod2 = corres(codon2);
        cod3 = corres(codon3);
        /*      fprintf(stderr," %c %c %c\n", codon1, codon2, codon3);
                fprintf(stderr," %d %d %d\n", cod1, cod2, cod3);*/

        if (cod1 <0 || cod2 <0 || cod3 <0) {
            /*          fprintf(stderr,"%d %d %d %d %d %c %c %c\n", cod1, cod2, cod3, i, j, codon1, codon2, codon3);
                        fprintf(stderr,"erreur sequence !!!!\n");*/
            return NULL;
        }

        tempo[i] = Codons[(cod1*4*4)+(cod2*4)+(cod3)];
    }
    tempo[i] = '\0';
    return tempo;
}


char *pas_codons(char *seq, int longueur) {

    int i,j, codon, cod1, cod2, cod3;
    char un, deux, trois;
    char *temposeq;
    temposeq = malloc((longueur+1)*sizeof(char));
    if (! temposeq)
        MyError(stderr,3,"swalign:: not enough memory for temposeq\n");

    for (i=0, j=0;i<(int)(longueur/3); i++, j+=3) {
        codon = PasCodons[(int)(seq[i])];
        cod1 = (int)(codon/(4*4));
        cod2 = (int)((codon-cod1*4*4)/4);
        cod3 = codon-cod1*4*4-cod2*4;
        un = inverse(cod1);
        deux = inverse(cod2);
        trois = inverse(cod3);
        temposeq[j] = un;
        temposeq[j+1] = deux;
        temposeq[j+2] = trois;

    }
    temposeq[j] = '\0';

    return temposeq;
}


/*fonction de tri d'un tableau du plus petit au plus grand */

void tri (float *tab, int longueur) {

    int i, j;
    float tmp;

    for (i=0; i<longueur; i++) {

        for (j=i+1; j<longueur; j++)
            if (tab[j]<tab[i]) {
                tmp = tab[i];
                tab[i] = tab[j];
                tab[j] = tmp;
            }

    }

}


/* ---------------------------------------------------- */
/*calcule log(-log P) pour le calcul de vingron*/
/* ---------------------------------------------------- */
int calcule_P(float *scorebis, int longueur, double **scores) {
    int i, lg=0, tmp=0;

    for (i=longueur-1; i>0; i--) {
        if (egal(scorebis[i], scorebis[i-1])==0) {
            scores[lg][0] = scorebis[i];  /*valeur*/
            scores[lg][1] = (double)(longueur -i -1)/(double)longueur; /*proba d'avoir cette valeur*/
            scores[lg][2] = 1;
            lg++;
        }
        else {
            scores[lg][0] = scorebis[i];
            scores[lg][1] = (double)(longueur - i -1)/(double)longueur;
            i--;
            tmp = 2;
            while ( i >= 1 && scorebis[i] == scorebis[i-1]) {
                i--;
                tmp++;
            }
            scores[lg][2] = tmp;
            lg++;
        }
    }
    if (egal(scorebis[0] , scorebis[1])==0) {
        scores[lg][0] = scorebis[0];  /*valeur*/
        scores[lg][1] = (double)(longueur -1)/(double)(longueur); /*proba d'avoir cette valeur*/
        scores[lg][2] = 1;
        lg++;


    }

    for (i=0; i<lg; i++)
        scores[i][1] = log(-log(1-scores[i][1]));

    return lg;
}


/* ---------------------------------------------------- */
/*fonction pour generer nb_shuffle sequences aleatoires*/
/* ---------------------------------------------------- */
static int calcule_shuffle2(tyProt *p0, tyProt *p1, int SPN, HashTable *data_hash,
                            HashTable *shuf_hash, char *alphabet, int automode,
                            int (*Igscore)(int a1, int a2, void *matthresh), void *mat_or_thresh,
                            float gap_o, float gap_e, float  recover, float cross_rms, int vverbose,
                            int nb_shuffle, float *scores, WV *water_ving)
{
    int i, nbres = 0;
    tyProt *new_seq = NewProt( p0->nbRes);
    char* temposeq, *temposeq2;
    RanFunc rfunc = ranUN;
    SWResult *swres=NULL;
    WV *tmp = NULL;

#ifdef MEMOIRE
    char *err_msg = NULL;
#endif

#ifdef MEMOIRE
    control_prog("./swelfe", getpid(), 1000000, 100000,&err_msg) ;
    fprintf(stderr, "memtest : avant shuffle \n");
    fflush(stderr);
#endif

    /* initialise graine aleatoire */
    sranUN(123456);

    nbres = 0; /*nb de resultats >0 (si swelfe renvoit des resultats <0*/

    for (i=0; i< nb_shuffle; i++) {
        Copie_tyProt (p0, new_seq);


        if (SPN == 2) { /*adn: codage en codons !!!*/
            temposeq = codons(new_seq->tNameRes, new_seq->nbRes);

            if (! shuffleSeq(P_VAL, data_hash, shuf_hash, temposeq,
                             (int)(new_seq->nbRes/3), rfunc)) {
                fprintf(stderr, "Error in Shuffling: exit... (%d)\n", i);
                FreeProt(new_seq);
                return 0;
            }
            temposeq2 = pas_codons(temposeq, new_seq->nbRes);
            strncpy(new_seq->tNameRes, temposeq2,new_seq->nbRes);
            MYFREE(temposeq);
            MYFREE(temposeq2);
        }
        /*shuffle la sequence */
        else if (! shuffleSeq(P_VAL, data_hash, shuf_hash, new_seq->tNameRes,
                              new_seq->nbRes, rfunc)) {
            fprintf(stderr, "Error in Shuffling: exit... (%d)\n", i);
            FreeProt(new_seq);
            return 0;
        }

        /*   fprintf(stderr,">seq%d \n%s \n",i,  new_seq->tNameRes);*/

#ifdef MEMOIRE
        control_prog("./swelfe", getpid(), 1000000, 100000, &err_msg) ;
        fprintf(stderr, "memtest : dans shuffle : %d \n", i);
        fflush(stderr);
#endif

        sIndexSequence(new_seq->tNameRes,new_seq->tAngles , new_seq->nbRes); /*convertit la sequence en index*/

        swres = simalign(new_seq, new_seq,1, 1, 0, 0, 0, /*algo SIM */
                         Igscore, mat_or_thresh, gap_o, gap_e,
                         recover, 0, vverbose, SPN, tmp, 0,0, 0, 0.0); /* automode : seq contre elle meme !!!!*/

        if (swres->nmatch > 0)  {
            /*seulement si score positif !!*/
            scores[nbres] = swres->score[0];  /*on garde le meilleur score a chaque fois*/

            /* decommenter pour calculer le score */
            if (vverbose)
               fprintf(stderr,"wv1: score %f  (%4d,%4d) (%4d,%4d)\n", swres->score[0],
                    swres->posd0[0], swres->posf0[0], swres->posd1[0], swres->posf1[0]);

            water_ving->lg_moy += strlen(swres->sali[0]);
            /* fprintf(stderr,"lg %f %f\n", water_ving->lg_moy, (float)(strlen(swres->sali[0])));*/

            /* PrintAliSeq(stderr, p0, swres->posd0[0], swres->sali[0], INSER);
            PrintIdAli(stderr, p0, swres->posd0[0], (swres->prot1), swres->posd1[0], swres->sali[0], &toto, SPN);
            PrintAliSeq(stderr, (swres->prot1), swres->posd1[0], swres->sali[0], DELET);
            */
            nbres++;
        } /*fin score >0  */

        /* Reinitialise_swres (swres); */  /*il ne faut pas free la proteine contenue dans swres*/
        FreeSWResult(swres, NOFREEPROT);
        Reinitialise_Prot (new_seq, p0);
    }


    FreeProt(new_seq);


    if (nbres < nb_shuffle/2)
        return 0;

    return nbres;
}

/* ---------------------------------------------------- */
/*fonction pour generer nb_shuffle sequences aleatoires*/
/* ---------------------------------------------------- */
int calcule_shuffle(tyProt *p0, tyProt *p1, int SPN, HashTable *data_hash, HashTable *shuf_hash, char *alphabet, int automode,
                    float (*gscore)(int a1, int a2, void *matthresh), void *mat_or_thresh, float gap_o, float gap_e,
                    float  recover, float cross_rms, int vverbose , int nb_shuffle, float *scores, WV *water_ving) {

    int i, nbres = 0;
    tyProt *new_seq = NewProt( p0->nbRes);
    char* temposeq, *temposeq2;
    RanFunc rfunc = ranUN;
    SWResult *swres=NULL;
    WV *tmp = NULL;

#ifdef MEMOIRE
    char *err_msg = NULL;
#endif




#ifdef MEMOIRE

    control_prog("./swelfe", getpid(), 1000000, 100000,&err_msg) ;
    fprintf(stderr, "memtest : avant shuffle \n");
    fflush(stderr);
#endif


    /* initialise graine aleatoire */
    sranUN(123456);

    nbres = 0; /*nb de resultats >0 (si swelfe renvoit des resultats <0*/

    for (i=0; i< nb_shuffle; i++) {
        Copie_tyProt (p0, new_seq);


        if (SPN == 2) { /*adn: codage en codons !!!*/
            temposeq = codons(new_seq->tNameRes, new_seq->nbRes);

            if (! shuffleSeq(P_VAL, data_hash, shuf_hash, temposeq,
                             (int)(new_seq->nbRes/3), rfunc)) {
                fprintf(stderr, "Error in Shuffling: exit... (%d)\n", i);
                FreeProt(new_seq);
                return 0;
            }
            temposeq2 = pas_codons(temposeq, new_seq->nbRes);
            strncpy(new_seq->tNameRes, temposeq2,new_seq->nbRes);
            MYFREE(temposeq);
            MYFREE(temposeq2);
        }


        /*shuffle la sequence */
        else if (! shuffleSeq(P_VAL, data_hash, shuf_hash, new_seq->tNameRes,
                              new_seq->nbRes, rfunc)) {
            fprintf(stderr, "Error in Shuffling: exit... (%d)\n", i);
            FreeProt(new_seq);
            return 0;
        }

        /*   fprintf(stderr,">seq%d \n%s \n",i,  new_seq->tNameRes);*/

#ifdef MEMOIRE

        control_prog("./swelfe", getpid(), 1000000, 100000, &err_msg) ;
        fprintf(stderr, "memtest : dans shuffle : %d \n", i);
        fflush(stderr);
#endif


        sIndexSequence(new_seq->tNameRes,new_seq->tAngles , new_seq->nbRes); /*convertit la sequence en index*/


        swres = swalign(new_seq, new_seq,1, 1, 0, 0, 0, /*algo SW*/
                        gscore, mat_or_thresh, gap_o, gap_e,
                        recover, 0, vverbose, SPN, tmp, 0,0, 0, 0.0); /* automode : seq contre elle mme !!!!*/

        if (swres->nmatch > 0)  {/*seulement si score positif !!*/
            scores[nbres] = swres->score[0];  /*on garde le meilleur score a chaque fois*/

            /* decommenter pour calculer le score */
            fprintf(stderr,"wv1: score %f  (%4d,%4d) (%4d,%4d)\n", swres->score[0],
                    swres->posd0[0], swres->posf0[0], swres->posd1[0], swres->posf1[0]);

            water_ving->lg_moy += strlen(swres->sali[0]);
            /*     fprintf(stderr,"lg %f %f\n", water_ving->lg_moy, (float)(strlen(swres->sali[0])));*/

            /*     PrintAliSeq(stderr, p0, swres->posd0[0], swres->sali[0], INSER);
                 PrintIdAli(stderr, p0, swres->posd0[0], (swres->prot1), swres->posd1[0], swres->sali[0], &toto, SPN);
                 PrintAliSeq(stderr, (swres->prot1), swres->posd1[0], swres->sali[0], DELET);
            */
            nbres++;
        } /*fin score >0  */

        /* Reinitialise_swres (swres); */  /*il ne faut pas free la proteine contenue dans swres*/
        FreeSWResult(swres, NOFREEPROT);

        Reinitialise_Prot (new_seq, p0);

    }

    FreeProt(new_seq);

    if (nbres < nb_shuffle/2)
        return 0;

    return nbres;
}


/* ---------------------------------------------------- */
void calcule_moyenne(float *moyenne, float *ecart_type, float *scores, int nb_shuffle) {
    int i;

    *moyenne = 0;
    for (i=0;i<nb_shuffle;i++)
        *moyenne += scores[i];
    *moyenne = *moyenne / nb_shuffle;

    *ecart_type =0;
    for (i=0;i<nb_shuffle;i++) {
        *ecart_type += MyPowbis( scores[i] - *moyenne, 2);
    }
    *ecart_type =  *ecart_type / (nb_shuffle-1);
    *ecart_type = sqrt( *ecart_type);

}
void  remplit_scores(float *scores, float *scorebis, int debut, int nb) {

    int i;
    for (i=0; i<nb; i++) {
        scores[i+debut] = scorebis[i];
    }
}

/* ---------------------------------------------------- */
WV *calcule_seuil_vingron2(tyProt *p0,  tyProt *p1, HashTable *data_hash, HashTable *shuf_hash,
                           int SPN, char *alphabet, double erreur, int lg_mot, float *moyenne,
                           float *ecart_type, float *zscore, int automode,
                           int (*Igscore)(int a1, int a2, void *matthresh),
                           void *mat_or_thresh, float gap_o, float gap_e,
                           float  recover, float cross_rms, int vverbose ,
                           int nb_shuffle, SWResult *swres)
{


    int i,  taille_tab, debut_tab = 0, nb_elts = nb_shuffle, somme_poids=0, nbres;  /*nbres = nb vrai de shuffle si cerains scores sont <0*/
    double a = 0.0, b = 0.0; /*equation de la droite*/
    double somme_xi = 0.0, somme_yi=0.0, somme_xy=0.0, somme_xcarre=0.0; /*pour regression lineaire*/
    double **scores;
    float *scorebis;
    float tmp;
    WV *water_ving;
#ifdef MEMOIRE
    char *err_msg = NULL;
#endif

    NEWY(water_ving,WV, 1);

    water_ving->n = 0.0;
    water_ving->m=p0->nbRes;
    water_ving->n=p1->nbRes;
    water_ving->gamma = 0.0;
    water_ving->lg_moy = 0.0;

    NEWY(scorebis,float,nb_shuffle);
#ifdef MEMOIRE
    fprintf(stderr, "memtest :dans wterving  1\n");
    fflush(stderr);
#endif

    NEWY (scores,double*, nb_shuffle );
    for (i=0;i<nb_shuffle; i++) {
        NEWY(scores[i] , double,  3);
    }
#ifdef MEMOIRE
    fprintf(stderr, "memtest :dans wterving 000\n");
    control_prog("./swelfe", getpid(), 1000000, 100000, &err_msg) ;
    fprintf(stderr, "memtest :dans wterving \n");
    fflush(stderr);
#endif


    if ((nbres=calcule_shuffle2(p0, p1, SPN, data_hash, shuf_hash, alphabet, automode,
                                Igscore, mat_or_thresh, gap_o, gap_e,
                                recover,  cross_rms, vverbose , nb_shuffle, scorebis, water_ving)) ==0) {
        *moyenne = 0.0;
        *ecart_type = 0.0;
        free(scorebis);
        return NULL;
    }

    tri(scorebis, nbres);  /*on trie les scores*/
    calcule_moyenne( moyenne, ecart_type, scorebis, nbres);
    water_ving->lg_moy = water_ving->lg_moy / nbres;




#if 0 /*************************regression lineaire simple****************************/
    taille_tab = calcule_P(scorebis, nb_shuffle, scores);

    /*on supprime les P=1 car log infini !!!*/
    while ((debut_tab < taille_tab) && (scores[debut_tab][1] < -10e5)) {
        debut_tab ++;
    }

    nb_elts= taille_tab - debut_tab;

    /*x = scores = scores[i][0] y = log(-log P) = scores[i][1]*/
    for (i=debut_tab; i<taille_tab; i++) {
        somme_xi += scores[i][0];
        somme_yi += scores[i][1];
        somme_xy += (scores[i][0]*scores[i][1]);
        somme_xcarre += (scores[i][0]*scores[i][0]);
    }
    moy_x = somme_xi / nb_elts;
    moy_y = somme_yi / nb_elts;
    b = (somme_xy - (somme_xi * somme_yi)/nb_elts)/(somme_xcarre - (somme_xi*somme_xi)/nb_elts);
    a = moy_y - b * moy_x;
    /*formule :
       Pr = e^-lambda * somme de j=0 a r-1 (lambda)^j / j!

       lambda = gamma *m*n*p^t
       r= nieme alignement
       n, m = longueur sequences

       a = log gamma *n* m
       b = log p

    */

    water_ving->p = exp(b);
    water_ving->gamma = exp(a)/(water_ving->m*water_ving->n);
#endif /*********************fin regression lineaire simple************************/

    taille_tab = calcule_P(scorebis, nbres, scores);

    /* for (i=0; i<taille_tab; i++)
    fprintf(stderr, " %f %f %f\n", scores[i][0], scores[i][1], scores[i][2]);
    */

    /*on supprime les P=1 car log infini !!!*/
    while ((debut_tab < taille_tab) && (scores[debut_tab][1] < -10e5)) {
        debut_tab ++;
    }

    nb_elts= taille_tab - debut_tab;

    /*x = scores = scores[i][0] y = log(-log P) = scores[i][1]*/
    for (i=debut_tab; i<taille_tab; i++) {
        somme_poids += scores[i][2];
        somme_xi += scores[i][0] * scores[i][2];
        somme_yi += scores[i][1] * scores[i][2];
        somme_xy += (scores[i][0]*scores[i][1] * scores[i][2]);
        somme_xcarre += (scores[i][0]*scores[i][0] * scores[i][2]);
    }
    /* fprintf(stderr, " debut tab fin tab nb poids %d %d %d\n",debut_tab, taille_tab, somme_poids); */

    b = ((somme_poids * somme_xy) - (somme_xi * somme_yi))/((somme_poids * somme_xcarre) - (somme_xi*somme_xi));
    a = ((somme_xcarre * somme_yi) - (somme_xi * somme_xy))/((somme_poids * somme_xcarre) - (somme_xi * somme_xi));

    /*formule :
       Pr = e^-lambda * somme de j=0 a r-1 (lambda)^j / j!

       lambda = gamma *m*n*p^t
       r= nieme alignement
       n, m = longueur sequences

       a = log gamma *n* m
       b = log p

     */

    water_ving->p = exp(b);
    water_ving->gamma = exp(a)/((water_ving->m*water_ving->n) /2);
    /* fprintf(stderr, "ab nb : %f %f %d\n", a, b, nb_elts); */
    tmp = log(-log(0.99)/ (water_ving->gamma * water_ving->m*water_ving->n /2))/log( water_ving->p );
    /* fprintf(stderr, "pwv : %f %f %f %f %f %f %f \n", water_ving->n, water_ving->m, water_ving->p, water_ving->gamma, tmp, a, b ); */

    water_ving->m = water_ving->m - water_ving->lg_moy;
    water_ving->n = water_ving->n - water_ving->lg_moy;

    MYFREE(scorebis);

    for (i=0;i<nb_shuffle; i++)
        MYFREE(scores[i]);
    MYFREE(scores);

    return  water_ving;
}
/* ---------------------------------------------------- */
WV *calcule_seuil_vingron (tyProt *p0,  tyProt *p1, HashTable *data_hash, HashTable *shuf_hash, int SPN, char *alphabet, double erreur,
                           int lg_mot,float *moyenne, float *ecart_type, float *zscore, int automode,
                           float (*gscore)(int a1, int a2, void *matthresh), void *mat_or_thresh, float gap_o, float gap_e,
                           float  recover, float cross_rms, int vverbose , int nb_shuffle, SWResult *swres) {



    int i,  taille_tab, debut_tab = 0, nb_elts = nb_shuffle, somme_poids=0, nbres;  /*nbres = nb vrai de shuffle si cerains scores sont <0*/
    double a = 0.0, b = 0.0; /*equation de la droite*/
    double somme_xi = 0.0, somme_yi=0.0, somme_xy=0.0, somme_xcarre=0.0; /*pour regression lineaire*/
    double **scores;
    float *scorebis;
    float tmp;
    WV *water_ving;
#ifdef MEMOIRE
    char *err_msg = NULL;
#endif

    NEWY(water_ving,WV, 1);

    water_ving->n = 0.0;
    water_ving->m=p0->nbRes;
    water_ving->n=p1->nbRes;
    water_ving->gamma = 0.0;
    water_ving->lg_moy = 0.0;




    NEWY(scorebis,float,nb_shuffle);
#ifdef MEMOIRE
    fprintf(stderr, "memtest :dans wterving  1\n");
    fflush(stderr);
#endif


    NEWY (scores,double*, nb_shuffle );
    for (i=0;i<nb_shuffle; i++) {
        NEWY(scores[i] , double,  3);
    }
#ifdef MEMOIRE
    fprintf(stderr, "memtest :dans wterving 000\n");

    control_prog("./swelfe", getpid(), 1000000, 100000, &err_msg) ;
    fprintf(stderr, "memtest :dans wterving \n");
    fflush(stderr);
#endif


    if ((nbres=calcule_shuffle(p0, p1, SPN, data_hash, shuf_hash, alphabet, automode,
                               gscore, mat_or_thresh,  gap_o, gap_e,
                               recover,  cross_rms, vverbose , nb_shuffle, scorebis, water_ving)) ==0) {
        *moyenne = 0.0;
        *ecart_type = 0.0;
        free(scorebis);
        return NULL;
    }

    tri(scorebis, nbres);  /*on trie les scores*/
    calcule_moyenne( moyenne, ecart_type, scorebis, nbres);
    water_ving->lg_moy = water_ving->lg_moy / nbres;




#if 0 /*************************regression lineaire simple****************************/
    taille_tab = calcule_P(scorebis, nb_shuffle, scores);

    /*on supprime les P=1 car log infini !!!*/
    while ((debut_tab < taille_tab) && (scores[debut_tab][1] < -10e5)) {
        debut_tab ++;
    }

    nb_elts= taille_tab - debut_tab;

    /*x = scores = scores[i][0] y = log(-log P) = scores[i][1]*/
    for (i=debut_tab; i<taille_tab; i++) {
        somme_xi += scores[i][0];
        somme_yi += scores[i][1];
        somme_xy += (scores[i][0]*scores[i][1]);
        somme_xcarre += (scores[i][0]*scores[i][0]);
    }
    moy_x = somme_xi / nb_elts;
    moy_y = somme_yi / nb_elts;
    b = (somme_xy - (somme_xi * somme_yi)/nb_elts)/(somme_xcarre - (somme_xi*somme_xi)/nb_elts);
    a = moy_y - b * moy_x;
    /*formule :
    Pr = e^-lambda * somme de j=0  r-1 (lambda)^j / j!

    lambda = gamma *m*n*p^t
    r= nieme alignement
    n, m = longueur sequences

    a = log gamma *n* m
    b = log p

    */

    water_ving->p = exp(b);
    water_ving->gamma = exp(a)/(water_ving->m*water_ving->n);
#endif /*********************fin regression linaire simple************************/

    taille_tab = calcule_P(scorebis, nbres, scores);

    /* for (i=0; i<taille_tab; i++)
        fprintf(stderr, " %f %f %f\n", scores[i][0], scores[i][1], scores[i][2]);
    */

    /*on supprime les P=1 car log infini !!!*/
    while ((debut_tab < taille_tab) && (scores[debut_tab][1] < -10e5)) {
        debut_tab ++;
    }

    nb_elts= taille_tab - debut_tab;

    /*x = scores = scores[i][0] y = log(-log P) = scores[i][1]*/
    for (i=debut_tab; i<taille_tab; i++) {
        somme_poids += scores[i][2];
        somme_xi += scores[i][0] * scores[i][2];
        somme_yi += scores[i][1] * scores[i][2];
        somme_xy += (scores[i][0]*scores[i][1] * scores[i][2]);
        somme_xcarre += (scores[i][0]*scores[i][0] * scores[i][2]);
    }
    /*          fprintf(stderr, " debut tab fin tab nb poids %d %d %d\n",debut_tab, taille_tab, somme_poids); */

    b = ((somme_poids * somme_xy) - (somme_xi * somme_yi))/((somme_poids * somme_xcarre) - (somme_xi*somme_xi));
    a = ((somme_xcarre * somme_yi) - (somme_xi * somme_xy))/((somme_poids * somme_xcarre) - (somme_xi * somme_xi));


    /*formule :
    Pr = e^-lambda * somme de j=0  r-1 (lambda)^j / j!

    lambda = gamma *m*n*p^t
    r= nieme alignement
    n, m = longueur sequences

    a = log gamma *n* m
    b = log p

    */

    water_ving->p = exp(b);
    water_ving->gamma = exp(a)/((water_ving->m*water_ving->n) /2);
    /*      fprintf(stderr, "ab nb : %f %f %d\n", a, b, nb_elts); */
    tmp = log(-log(0.99)/ (water_ving->gamma * water_ving->m*water_ving->n /2))/log( water_ving->p );
    /*          fprintf(stderr, "pwv : %f %f %f %f %f %f %f \n", water_ving->n, water_ving->m, water_ving->p, water_ving->gamma, tmp, a, b ); */

    water_ving->m = water_ving->m - water_ving->lg_moy;
    water_ving->n = water_ving->n - water_ving->lg_moy;

    MYFREE(scorebis);

    for (i=0;i<nb_shuffle; i++)
        MYFREE(scores[i]);
    MYFREE (scores);

    return  water_ving;
}
/* -----------------------------------------------------------------------------------------*/
/**************************** autre methode de waterman et vingron **************************/
/* -----------------------------------------------------------------------------------------*/
int new_calcule_shuffle2(tyProt *p0, tyProt *p1, int SPN, HashTable *data_hash, HashTable *shuf_hash,
                         char *alphabet, int automode, int (*Igscore)(int a1, int a2, void *matthresh),
                         void *mat_or_thresh, float gap_o, float gap_e, float recover, float cross_rms,
                         int vverbose , int nb_shuffle,int nb_repeats, float **scores, WV *water_ving)
{
    int i, nbres = 0, j=0;
    tyProt *new_seq = NewProt( p0->nbRes);
    char* temposeq, *temposeq2;
    RanFunc rfunc = ranUN;
    SWResult *swres=NULL;
    WV *tmp = NULL;
    float seuil = 0.0; /*pour savoir combien de repetitions on calcule*/

    /* initialise graine aleatoire */
    sranUN(123456);

    nbres = 0; /*nb de resultats >0 (si swelfe renvoit des resultats <0*/


    for (i=0; i< nb_shuffle; i++) {
        Copie_tyProt (p0, new_seq);
        if (SPN == 2) { /*adn: codage en codons !!!*/
            temposeq = codons(new_seq->tNameRes, new_seq->nbRes);

            if (! shuffleSeq(P_VAL, data_hash, shuf_hash, temposeq,
                             (int)(new_seq->nbRes/3), rfunc)) {
                fprintf(stderr, "Error in Shuffling: exit... (%d)\n", i);
                FreeProt(new_seq);
                return 0;
            }
            temposeq2 = pas_codons(temposeq, new_seq->nbRes);
            strncpy(new_seq->tNameRes, temposeq2,new_seq->nbRes);
            MYFREE(temposeq);
            MYFREE(temposeq2);
        }
        /* shuffle la sequence */
        else if (! shuffleSeq(P_VAL, data_hash, shuf_hash, new_seq->tNameRes,
                              new_seq->nbRes, rfunc)) {
            fprintf(stderr, "Error in Shuffling: exit... (%d)\n", i);
            FreeProt(new_seq);
            return 0;
        }

        sIndexSequence(new_seq->tNameRes,new_seq->tAngles , new_seq->nbRes); /*convertit la sequence en index*/

        if (i == 0) {
            swres = simalign(new_seq, new_seq,1, nb_repeats, 1, 0, 0, /*algo SIM *//*option score min 1*/
                             Igscore, mat_or_thresh, gap_o, gap_e,
                             recover, 0, vverbose, SPN, tmp, 0,0, 0, 0.0); /* automode : seq contre elle meme !!!!*/
            seuil = swres->score[swres->nmatch-1];
        }
        else {
            swres = simalign(new_seq, new_seq,1, -1, seuil, 0, 0, /*algo SIM *//*on met en seuil la plus faible valeur obtenue avant */
                             Igscore, mat_or_thresh, gap_o, gap_e,
                             recover, 0, vverbose, SPN, tmp, 0,0, 0, 0.0); /* automode : seq contre elle meme !!!!*/
        }

        scores[i][0] = swres->nmatch;
        if (swres->nmatch > 0) { /*seulement si score positif !!*/
            for (j=0; j<swres->nmatch; j++) {
                if (swres->score[j] > 0) {
                    scores[i][j+1] = swres->score[j];  /*on copie les scores a chaque fois*/
                }
                else
                    break;
            }
            /*   water_ving->lg_moy += strlen(swres->sali[0]);*/
            nbres++;
        } /*fin score >0  */

        /* Reinitialise_swres(swres);*/  /*il ne faut pas free la proteine contenue dans swres*/
        FreeSWResult(swres, NOFREEPROT);
        Reinitialise_Prot(new_seq, p0);

    }

    FreeProt(new_seq);

    if (nbres < nb_shuffle/2)
        return 0;

    return nbres;
}


/**********************************************************************************************************/
/**************************** autre methode de waterman et vingron ****************************************/
/**********************************************************************************************************/

int new_calcule_shuffle(tyProt *p0, tyProt *p1, int SPN, HashTable *data_hash, HashTable *shuf_hash, char *alphabet, int automode,
                        float (*gscore)(int a1, int a2, void *matthresh), void *mat_or_thresh, float gap_o, float gap_e,
                        float  recover, float cross_rms, int vverbose , int nb_shuffle,int nb_repeats, float **scores, WV *water_ving) {

    int i, nbres = 0, j=0;
    tyProt *new_seq = NewProt( p0->nbRes);
    char* temposeq, *temposeq2;
    RanFunc rfunc = ranUN;
    SWResult *swres=NULL;
    WV *tmp = NULL;
    float seuil = 0.0; /*pour savoir combien de repetitions on calcule*/

    /* initialise graine aleatoire */
    sranUN(123456);

    nbres = 0; /*nb de resultats >0 (si swelfe renvoit des resultats <0*/


    for (i=0; i< nb_shuffle; i++) {
        Copie_tyProt (p0, new_seq);

        if (SPN == 2) { /*adn: codage en codons !!!*/
            temposeq = codons(new_seq->tNameRes, new_seq->nbRes);

            if (! shuffleSeq(P_VAL, data_hash, shuf_hash, temposeq,
                             (int)(new_seq->nbRes/3), rfunc)) {
                fprintf(stderr, "Error in Shuffling: exit... (%d)\n", i);
                FreeProt(new_seq);
                return 0;
            }
            temposeq2 = pas_codons(temposeq, new_seq->nbRes);
            strncpy(new_seq->tNameRes, temposeq2,new_seq->nbRes);
            MYFREE(temposeq);
            MYFREE(temposeq2);


        }


        /*shuffle la sequence */
        else if (! shuffleSeq(P_VAL, data_hash, shuf_hash, new_seq->tNameRes,
                              new_seq->nbRes, rfunc)) {
            fprintf(stderr, "Error in Shuffling: exit... (%d)\n", i);
            FreeProt(new_seq);
            return 0;
        }

        sIndexSequence(new_seq->tNameRes,new_seq->tAngles , new_seq->nbRes); /*convertit la sequence en index*/


        if (i == 0) {
            swres = swalign(new_seq, new_seq,1, nb_repeats, 1, 0, 0, /*algo SW*//*option score min 1*/
                            gscore, mat_or_thresh, gap_o, gap_e,
                            recover, 0, vverbose, SPN, tmp, 0,0, 0, 0.0); /* automode : seq contre elle mme !!!!*/
            seuil = swres->score[swres->nmatch-1];


        }
        else {

            swres = swalign(new_seq, new_seq,1, -1, seuil, 0, 0, /*algo SW*//*on met en seuil la plus faible valeur obtenue avant */
                            gscore, mat_or_thresh, gap_o, gap_e,
                            recover, 0, vverbose, SPN, tmp, 0,0, 0, 0.0); /* automode : seq contre elle mme !!!!*/
        }

        scores[i][0] = swres->nmatch;
        if (swres->nmatch > 0) {/*seulement si score positif !!*/
            for (j=0; j<swres->nmatch; j++) {
                if (swres->score[j] > 0) {
                    scores[i][j+1] = swres->score[j];  /*on copie les scores a chaque fois*/
                }
                else
                    break;
            }
            /*   water_ving->lg_moy += strlen(swres->sali[0]);*/
            nbres++;
        } /*fin score >0  */

        /* Reinitialise_swres (swres); */  /*il ne faut pas free la proteine contenue dans swres*/
        FreeSWResult(swres, NOFREEPROT);

        Reinitialise_Prot (new_seq, p0);

    }


    FreeProt(new_seq);

    if (nbres < nb_shuffle/2)
        return 0;

    return nbres;

}


/* ---------------------------------------------------- */
/*garde un score de chaque, trie + compte le nombre de scores de chaque */
/* ---------------------------------------------------- */
int new_tri (float **tab, int longueur, int largeur,  double **tab_uniq) {

    int i, j, k, nb;
    float tmp, tmp2, tmp3, tmp4;

    nb = 1; /*taille du tableau+1*/
    for (i=0; i<longueur*largeur*2; i++)
        tab_uniq[i][0] = 0.0;

    for (i=0; i<longueur; i++) {
        for (j=1; j<=tab[i][0]; j++) {
            k=0;
            while (k<nb&& tab_uniq[k][0]>tab[i][j])
                k++;
            if (tab_uniq[k][0] !=tab[i][j]) { /*si la valeur n'y est pas on la rajoute*/
                tmp = tab_uniq[k][0];
                tmp3 = tab_uniq[k][1]; /*comptage*/
                tab_uniq[k][0] = tab[i][j];
                tab_uniq[k][1] = 1;
                k++;
                nb++;

                while (k<nb) {
                    tmp2 = tab_uniq[k][0];
                    tmp4 = tab_uniq[k][1];
                    tab_uniq[k][0] = tmp;
                    tab_uniq[k][1] = tmp3;
                    tmp = tmp2;
                    tmp3 = tmp4;
                    k++;
                }
            }
            else { /*la valeur y est deja on compte*/
                tab_uniq[k][1] ++;
            }
        }
    }
    nb --;
    return nb;
}

/* ---------------------------------------------------- */
void new_calcule_moyenne(float *moyenne, float *ecart_type, float *scores, int nb_shuffle) {
    int i;

    *moyenne = 0;
    for (i=0;i<nb_shuffle;i++)
        *moyenne += scores[i];
    *moyenne = *moyenne / nb_shuffle;

    *ecart_type =0;
    for (i=0;i<nb_shuffle;i++) {
        *ecart_type += MyPowbis( scores[i] - *moyenne, 2);
    }
    *ecart_type =  *ecart_type / (nb_shuffle-1);
    *ecart_type = sqrt( *ecart_type);

}

/* ---------------------------------------------------- */
WV *new_calcule_seuil_vingron (tyProt *p0,  tyProt *p1, HashTable *data_hash, HashTable *shuf_hash, int SPN, char *alphabet, double erreur,
                               int lg_mot,float *moyenne, float *ecart_type, float *zscore, int automode,
                               float (*gscore)(int a1, int a2, void *matthresh), void *mat_or_thresh, float gap_o, float gap_e,
                               float  recover, float cross_rms, int vverbose , int nb_shuffle, int nb_repeats, SWResult *swres) {



    int i, j, k, somme_poids=0, nbres, *nb;  /*nbres = nb vrai de shuffle si cerains scores sont <0*/
    double a = 0.0, b = 0.0; /*equation de la droite*/
    double somme_xi = 0.0, somme_yi=0.0, somme_xy=0.0, somme_xcarre=0.0; /*pour regression lineaire*/
    double **scores;/*contient tous les calculs pour les scores, un score de chaque*/
    float **scorebis; /*contient les scores */
    float moy_nb;
    float tmp2;
    WV *water_ving;

    float nb2;
    int nb_repet;

#ifdef MEMOIRE
    char *err_msg = NULL;
#endif

    /*allocation memoire*/
    NEWY(water_ving,WV, 1);

    water_ving->n = 0.0;
    water_ving->m=p0->nbRes;
    water_ving->n=p1->nbRes;
    water_ving->gamma = 0.0;
    water_ving->lg_moy = 0.0;

    NEWY(nb,int,nb_shuffle);

    NEWY(scorebis,float*,nb_shuffle); /*1ere colonne = nb de repetitions par ligne*/
    for (i=0;i<nb_shuffle; i++) {
        NEWY(scorebis[i] , float,  nb_repeats*4);  /*on prevoit 4 x + de  memoire parce qu'on sait pas combien on en a */
    }
    NEWY (scores,double*, nb_shuffle*nb_repeats*2 );  /*on double au cas o il y ait plus de repetitions que le nombre prevu*/
    for (j=0;j<nb_shuffle*nb_repeats*2; j++)
        NEWY(scores[j] , double,  3);


    /* calcule sequences aleatoires et scores de chaque, stocke dans score_bis*/
    if ((nbres=new_calcule_shuffle(p0, p1, SPN, data_hash, shuf_hash, alphabet, automode,
                                   gscore, mat_or_thresh,  gap_o, gap_e,
                                   recover,  cross_rms, vverbose , nb_shuffle, nb_repeats, scorebis, water_ving)) ==0) {
        *moyenne = 0.0;
        *ecart_type = 0.0;
        free(scorebis);
        return NULL;
    }



    /*decommenter pour calculer le score */
  #if 0
  for (i=0; i<nbres; i++) {
        /*fprintf(stderr, "scores  %f \n************************   ", scorebis[i][0]);*/
        fprintf(stderr, "scores   ");
        for (j=1; j<=scorebis[i][0]; j++) {

            fprintf(stderr, " %f ", scorebis[i][j]);
        }
        fprintf(stderr, " \n");
    }
#endif

    /*on garde un score de chaque, tries, dans la 1ere colonne du tableau*/
    nbres = new_tri(scorebis, nbres, nb_repeats, scores);


    /*for (i=0; i<nbres; i++){

      fprintf(stderr, " %f %f \n", scores[i][0], scores[i][1]);
      }
    */





    /* pour chaque score du tableau, on calcule le nb de scores > au score courant*/

    /*on recommence avec moyenne globale sur tous les scores */
    for (i=0; i<nbres; i++) { /*pour chaque score*/
        moy_nb = 0;
        nb_repet = 0;
        scores[i][2] =0.0;
        nb2=0;
        for (j=0; j<nb_shuffle; j++) {  /*pour chaque sequence*/
            k=1;
            nb_repet +=scorebis[j][0];
            while (k<= scorebis[j][0]&& scorebis[j][k]> scores[i][0]) {
                k++;
                nb2 ++;
            }
        }
        scores[i][2] = (float)(nb2) / nb_shuffle;

        scores[i][2] = log(scores[i][2]);

    }
    /*
     for (i=1; i<nbres; i++)
           fprintf(stderr, "%%%%%%%%%%%%%% %f %f %f \n",scores[i][0], scores[i][1], scores[i][2]);
    */
    /**************** regression lineaire ***********************/
    /*on supprime les P=1 car log infini !!!*/

    /* IL FAUT SUPPRIMER L'INFINI donc on enleve le 1er*/
    /*  nb_elts= taille_tab - debut_tab;*/

    /*x = scores = scores[i][0] y = (log (moyenne)) = scores[i][2]*/
    for (i=1; i<nbres; i++) {  /*on commence a 1 pour enlever l'infini*/
        somme_poids += scores[i][1];
        somme_xi += scores[i][0] * scores[i][1];
        somme_yi += scores[i][2] * scores[i][1];
        somme_xy += (scores[i][0]*scores[i][2] * scores[i][1]);
        somme_xcarre += (scores[i][0]*scores[i][0] * scores[i][1]);
    }
    /*          fprintf(stderr, " debut tab fin tab nb poids %d %d %d\n",debut_tab, taille_tab, somme_poids); */

    b = ((somme_poids * somme_xy) - (somme_xi * somme_yi))/((somme_poids * somme_xcarre) - (somme_xi*somme_xi));
    a = ((somme_xcarre * somme_yi) - (somme_xi * somme_xy))/((somme_poids * somme_xcarre) - (somme_xi * somme_xi));


    /*formule :
    Pr = e^-lambda * somme de j=0 a r-1 (lambda)^j / j!

    lambda = gamma *m*n*p^t
    r= nieme alignement
    n, m = longueur sequences

    a = log gamma *n* m
    b = log p

    */

    water_ving->p = exp(b);
    water_ving->gamma = exp(a)/((water_ving->m*water_ving->n)/2);
    /*      fprintf(stderr, "ab nb : %f %f %d\n", a, b, nb_elts); */
    tmp2 = log(-log(0.99)/ (water_ving->gamma * water_ving->m*water_ving->n /2))/log( water_ving->p );
    /*          fprintf(stderr, "pwv : %f %f %f %f %f %f %f \n", water_ving->n, water_ving->m, water_ving->p, water_ving->gamma, tmp2, a, b); */
    /*        water_ving->m = water_ving->m - water_ving->lg_moy;
            water_ving->n = water_ving->n - water_ving->lg_moy;*/






    MYFREE(nb);

    for (i=0;i<nb_shuffle; i++)
        MYFREE(scorebis[i]);
    MYFREE (scorebis);

    for (i=0;i<nb_shuffle*nb_repeats; i++)
        MYFREE(scores[i]);
    MYFREE (scores);

    return  water_ving;
}

/* ---------------------------------------------------- */
WV* new_calcule_seuil_vingron2(tyProt *p0,  tyProt *p1, HashTable *data_hash, HashTable *shuf_hash,
                               int SPN, char *alphabet, double erreur, int lg_mot, float *moyenne,
                               float *ecart_type, float *zscore, int automode,
                               int (*Igscore)(int a1, int a2, void *matthresh), void *mat_or_thresh,
                               float gap_o, float gap_e, float  recover, float cross_rms, int vverbose ,
                               int nb_shuffle, int nb_repeats, SWResult *swres)
{
    int i, j, k, somme_poids=0, nbres, *nb;  /*nbres = nb vrai de shuffle si cerains scores sont <0*/
    double a = 0.0, b = 0.0; /*equation de la droite*/
    double somme_xi = 0.0, somme_yi=0.0, somme_xy=0.0, somme_xcarre=0.0; /*pour regression lineaire*/
    double **scores;/*contient tous les calculs pour les scores, un score de chaque*/
    float **scorebis; /*contient les scores */
    float moy_nb;
    float tmp2;
    WV *water_ving;
    float nb2;
    int nb_repet;

#ifdef MEMOIRE
    char *err_msg = NULL;
#endif

    /*allocation memoire*/
    NEWY(water_ving,WV, 1);

    water_ving->n = 0.0;
    water_ving->m=p0->nbRes;
    water_ving->n=p1->nbRes;
    water_ving->gamma = 0.0;
    water_ving->lg_moy = 0.0;

    NEWY(nb,int,nb_shuffle);

    NEWY(scorebis,float*,nb_shuffle); /*1ere colonne = nb de repetitions par ligne*/
    for (i=0;i<nb_shuffle; i++) {
        NEWY(scorebis[i] , float,  nb_repeats*4);  /*on prevoit 4 x + de  memoire parce qu'on sait pas combien on en a */
    }
    NEWY (scores,double*, nb_shuffle*nb_repeats*2 );  /*on double au cas o il y ait plus de repetitions que le nombre prevu*/
    for (j=0;j<nb_shuffle*nb_repeats*2; j++)
        NEWY(scores[j] , double,  3);

    /* calcule sequences aleatoires et scores de chaque, stocke dans score_bis*/
    if ((nbres=new_calcule_shuffle2(p0, p1, SPN, data_hash, shuf_hash, alphabet, automode,
                                    Igscore, mat_or_thresh,  gap_o, gap_e,
                                    recover,  cross_rms, vverbose , nb_shuffle, nb_repeats, scorebis, water_ving)) ==0) {
        *moyenne = 0.0;
        *ecart_type = 0.0;
        free(scorebis);
        return NULL;
    }


    /*  decommenter pour calculer le score */
#if 0
 for (i=0; i<nbres; i++) {
        /*fprintf(stderr, "scores  %f \n************************   ", scorebis[i][0]);*/
        fprintf(stderr, "scores   ");
        for (j=1; j<=scorebis[i][0]; j++) {
            fprintf(stderr, " %f ", scorebis[i][j]);
        }
        fprintf(stderr, " \n");
    }
#endif
    /*on garde un score de chaque, tries, dans la 1ere colonne du tableau*/
    nbres = new_tri(scorebis, nbres, nb_repeats, scores);

    /*for (i=0; i<nbres; i++){
      fprintf(stderr, " %f %f \n", scores[i][0], scores[i][1]);
      }
    */





    /* pour chaque score du tableau, on calcule le nb de scores > au score courant*/
    /*on recommence avec moyenne globale sur tous les scores */
    for (i=0; i<nbres; i++) { /*pour chaque score*/
        moy_nb = 0;
        nb_repet = 0;
        scores[i][2] =0.0;
        nb2=0;
        for (j=0; j<nb_shuffle; j++) {  /*pour chaque sequence*/
            k=1;
            nb_repet +=scorebis[j][0];
            while (k<= scorebis[j][0]&& scorebis[j][k]> scores[i][0]) {
                k++;
                nb2 ++;
            }
        }
        scores[i][2] = (float)(nb2) / nb_shuffle;

        scores[i][2] = log(scores[i][2]);

    }
    /*
    for (i=1; i<nbres; i++)
       fprintf(stderr, "%%%%%%%%%%%%%% %f %f %f \n",scores[i][0], scores[i][1], scores[i][2]);
    */

    /**************** regression lineaire ***********************/
    /*on supprime les P=1 car log infini !!!*/

    /* IL FAUT SUPPRIMER L'INFINI donc on enleve le 1er*/
    /*  nb_elts= taille_tab - debut_tab;*/

    /*x = scores = scores[i][0] y = (log (moyenne)) = scores[i][2]*/
    for (i=1; i<nbres; i++) {  /*on commence a 1 pour enlever l'infini*/
        somme_poids += scores[i][1];
        somme_xi += scores[i][0] * scores[i][1];
        somme_yi += scores[i][2] * scores[i][1];
        somme_xy += (scores[i][0]*scores[i][2] * scores[i][1]);
        somme_xcarre += (scores[i][0]*scores[i][0] * scores[i][1]);
    }
    /* fprintf(stderr, " debut tab fin tab nb poids %d %d %d\n",debut_tab, taille_tab, somme_poids); */
    b = ((somme_poids * somme_xy) - (somme_xi * somme_yi))/((somme_poids * somme_xcarre) - (somme_xi*somme_xi));
    a = ((somme_xcarre * somme_yi) - (somme_xi * somme_xy))/((somme_poids * somme_xcarre) - (somme_xi * somme_xi));


    /*formule :
       Pr = e^-lambda * somme de j=0 a r-1 (lambda)^j / j!

          lambda = gamma *m*n*p^t
          r= nieme alignement
          n, m = longueur sequences

          a = log gamma *n* m
          b = log p

    */

    water_ving->p = exp(b);
    water_ving->gamma = exp(a)/((water_ving->m*water_ving->n)/2);

    /* fprintf(stderr, "ab nb : %f %f %d\n", a, b, nb_elts); */
    tmp2 = log(-log(0.99)/ (water_ving->gamma * water_ving->m*water_ving->n /2))/log( water_ving->p );
    /* fprintf(stderr, "pwv : %f %f %f %f %f %f %f \n", water_ving->n, water_ving->m, water_ving->p, water_ving->gamma, tmp2, a, b); */
    /* water_ving->m = water_ving->m - water_ving->lg_moy;
    water_ving->n = water_ving->n - water_ving->lg_moy;*/

    MYFREE(nb);

    for (i=0;i<nb_shuffle; i++)
        MYFREE(scorebis[i]);
    MYFREE (scorebis);

    for (i=0;i<nb_shuffle*nb_repeats; i++)
        MYFREE(scores[i]);
    MYFREE (scores);

    return  water_ving;
}


#if 0
void Reinitialise_swres ( SWResult *swres) {

    int i;

    for (i=0; i < swres->nmatch; i++) {
        MYFREE(swres->sali[i]);
    }
    MYFREE(swres->sali);

    MYFREE(swres->matrix4x);

    if (swres->crossrms) {
        for (i=0; i < swres->nmatch; i++)
            MYFREE(swres->crossrms[i]);
        MYFREE(swres->crossrms);
    }

    if (swres->group)
        MYFREE(swres->group);

    if (swres ==NULL )
        fprintf(stderr,"MYFREE null \n");


    MYFREE(swres->posd0);
    MYFREE(swres->posd1);
    MYFREE(swres->posf0);
    MYFREE(swres->posf1);
    MYFREE(swres->score);
    MYFREE(swres->score_freq);
    MYFREE(swres->score_par_freq);
    MYFREE(swres->score_GL);
    MYFREE(swres->tm_score);
    MYFREE(swres->score_blosangle);
    MYFREE(swres->recover);
    MYFREE(swres->rms);
    MYFREE(swres->rev_rms);
    MYFREE(swres->tProbaMTD_p0);
    MYFREE(swres->tProbaMTD_p1);
    MYFREE(swres->tProbaIndep);
    MYFREE(swres->tProbaBi);
    MYFREE(swres->alpha);
    MYFREE(swres->beta);
    MYFREE(swres->gamma);



    MYFREE(swres);


}
#endif
