

#define MAX_SYMB    32  /* 'A' -> 'Z' */

#define MAX_PITER   100 /* max # of pval iterations	    */
#define MBIG_UN	    2147483647	/* 2^31 - 1 */

#define EOS     ((char) 0xff)   /* end of encoded sequence */


/* -------------------------------------------
 * Structures de donnees
 */

typedef struct {    /* alphabet + hash table */
    int	    alphasize;	    /* size of alphabet		    */
    int	    bitshift;	    /* log2(alphasize)		    */
    int	    wsize;	    /* 2^bitshift		    */
    int	    ksize;	    /* word size		    */
    int   tsize;	    /* (2^bitshift)^(ksize-1)	    */
    int   smask;	    /* symbol mask useful bits	    */
    int   wmask;	    /* word mask useful bits	    */
    int   tsum;	    /* sum of all hashtable counts  */
    int   **table;	    /* hashtable		    */
    char    alphabet[MAX_SYMB+1];
} HashTable;
   
typedef struct {    /* motif elements table */
    int	    large;	    /* larger shift (j) */
    int	    small;	    /* smaller shift (i) */
	int 	includ;		/* 0 means that it is not to be included*/
} MotifsTable;

/* -------------------------------------------
 * Macros standards
 */


#ifndef MIN
#define MIN(x, y) (((x) < (y)) ? (x) : (y))
#endif

#ifndef MAX
#define MAX(x, y) (((x) > (y)) ? (x) : (y))
#endif

#ifndef ABS
#define ABS(x) (((x) >= 0) ? (x) : -(x))
#endif

#ifndef P
# define 	P(s) ()
#endif
/*typedef void (*SeedFunc) P(( long seed ));*/
typedef  float   (*RanFunc)  P(( void ));


int Error(char *msg, int stat);
int BuiltHashTable( HashTable	*hash,  char	*alphabet,  int	ksize);


void ResetFrequencies(HashTable *hash);
void EncodeSequence(char *seq, HashTable *hash);

void ComputeFrequencies(char *seq, HashTable *hash, int len);
void DecodeSequence(char *seq, HashTable *hash);

double calcule_seuil_proba(tyProt *p0,  tyProt *p1, HashTable *data_hash, HashTable *shuf_hash,
                           int SPN, char *alphabet, double erreur, int lg_mot, float *moyenne,
                            float *ecart_type, float *zscore, int automode,  
			    float (*gscore)(int a1, int a2, void *matthresh), void *mat_or_thresh,
                            float gap_o, float gap_e, float  recover, float cross_rms,
                            int vverbose ,int nb_shuffle, SWResult *swres);
WV *calcule_seuil_vingron2(tyProt *p0,  tyProt *p1, HashTable *data_hash, HashTable *shuf_hash,
                            int SPN, char *alphabet, double erreur, int lg_mot, float *moyenne,
                            float *ecart_type, float *zscore, int automode,
                            int (*Igscore)(int a1, int a2, void *matthresh),
                            void *mat_or_thresh, float gap_o, float gap_e,
                            float  recover, float cross_rms, int vverbose ,
                            int nb_shuffle, SWResult *swres);
WV *calcule_seuil_vingron(tyProt *p0,  tyProt *p1, HashTable *data_hash, HashTable *shuf_hash,
                            int SPN, char *alphabet, double erreur, int lg_mot, float *moyenne,
                            float *ecart_type, float *zscore, int automode,
                            float (*gscore)(int a1, int a2, void *matthresh),
                            void *mat_or_thresh, float gap_o, float gap_e,
                            float  recover, float cross_rms, int vverbose ,
                            int nb_shuffle, SWResult *swres);
WV* new_calcule_seuil_vingron2(tyProt *p0,  tyProt *p1, HashTable *data_hash, HashTable *shuf_hash,
                            int SPN, char *alphabet, double erreur, int lg_mot, float *moyenne,
                            float *ecart_type, float *zscore, int automode, 
			    int (*Igscore)(int a1, int a2, void *matthresh), void *mat_or_thresh,
                            float gap_o, float gap_e, float  recover, float cross_rms, int vverbose ,
                            int nb_shuffle, int nb_repeats, SWResult *swres);
WV* new_calcule_seuil_vingron(tyProt *p0,  tyProt *p1, HashTable *data_hash, HashTable *shuf_hash,
                            int SPN, char *alphabet, double erreur, int lg_mot, float *moyenne,
                            float *ecart_type, float *zscore, int automode, 
			    float (*gscore)(int a1, int a2, void *matthresh), void *mat_or_thresh,
                            float gap_o, float gap_e, float  recover, float cross_rms, int vverbose ,
                            int nb_shuffle, int nb_repeats, SWResult *swres);

double nr_gammq(double a, double x);
double sCumNormal(double x, double mu, double sigma);


void sranUN(long seed);
float ranUN();

void FreeHashTable( HashTable	*hash);


double gammq(double a, double x);

double ltqnorm(double p);








