/* ---------------------------------------------------  */
/* swelfe.c : ABI 1st version April 2004                */
/* ---------------------------------------------------  */
/* this program makes use of adapted file from sim.c whose
   copyrigth is below
 */
/* A PROGRAM FOR LOCAL SIMILARITIES WITH AFFINE WEIGHTS:

    copyright (c) 1990-1997 Xiaoqiu Huang and Webb Miller
    The distribution of the program is granted provided no charge is made
    and the copyright notice is included.
    E-mail: huang@cs.mtu.edu

IN NO EVENT SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    Please cite the paper
    "A Time-Efficient, Linear-Space Local Similarity Algorithm"
	(Advances in Applied Mathematics, 12: 337-357, 1991)

	      Xiaoqiu Huang
	      Department of Computer Science
	      Michigan Technological University
	      Houghton, MI 49931

	      Webb Miller
	      Department of Computer Science
	      The Pennsylvania State University
	      University Park, PA 16802

     An early version of this program is presented in the paper
     "A Space-Efficient Algorithm for Local Similarities"
	(Computer Applications in the Biosciences, 6(4): 373-381, 1990)

	      Xiaoqiu Huang
	      Department of Computer Science
	      Michigan Technological University
	      Houghton, MI 49931

	      Ross C. Hardison and Webb Miller
	      Department of Molecular and Cell Biology
	      Department of Computer Science
	      The Pennsylvania State University
	      University Park, PA 16802

    The SIM program finds k best non-intersecting alignments between
    two sequences or within one sequence. Using dynamic programming
    techniques, SIM is guaranteed to find optimal alignments. The
    alignments are reported in order of similarity score, with the
    highest scoring alignment first. The k best alignments share no
    aligned pairs. SIM requires space proportional to the sum of the
    input sequence lengths and the output alignment lengths. Thus
    SIM can handle sequences of tens of thousands, even a hundred of
    thousands, of base pairs on a workstation. For example,
    on 73,360-bp and 44,594-bp sequences, SIM took 15 hours to find
    100 best local alignments on a Sun4 workstation.
 */

/* ---------------------- HEADER A INCLURE  ----------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>             /* pour memset */
#include <unistd.h>     /* pour getopt  */


/* ---------------------- HEADER LOCAUX --------------- */
#include "swelfe.h"
#include "swalign.h"
#include "lecture.h"
#include "macros.h"
#include "util.h"
#include "minimier.h"
#include "zlib.h"
#include "libunz.h"
#include "stats.h"
#include "sortie.h"
#include "libabiutils.h"
#include "Nmatrix.h"
#include "stats_sequence.h"
#include "fonctions_main.h"
#include "sim.h"
#include "kontrol.proto.h"  /*pour controler la memoire*/

/*#define MEMOIRE 1 */ /*pour verifier la memoire*/


/*--------------------------------------------- */
/* main routine                 */
/*--------------------------------------------- */

int main(int argc, char *argv[]) {
    int       automode, nprot0, nprot1, carg, errflg = 0,
            nbProtPbRecodage=0, nbProtPbLecture = 0;
    int       i;
    double    (*getscore) (void *) = GetSWRMS;
    int       (*compar) (double, double) = ComparNewRMS, (*qscompar)(const void *, const void *)=sCtreeComparDecroissant;
#if ( AFFICHAGE_SOMME_SCORES) || ( AFFICHAGE_TRANSITIONS) ||  (AFFICHAGE_TRANSITIONS2) ||  (CALCUL_BLOSUM)
    int i,j;
#endif
#if TOUTES_TRANSITIONS
    int  k, l;
#endif
    int proba = 0; /*par defaut, pas d'affichage des probabilites*/
    int WV2 = 0; /*proba waterman et vingron 2*/
    int SPN = 0; /*par defaut, structure; 1=proteique; 2=nucleique*/
    int       index_adn[128];
    tyParametres  parms;
    tyProt    *p0=NULL,*p1=NULL;
    SWResult  *swres = NULL, *oldres = NULL;
    FILE      *(pFi[2])= {NULL,NULL}, *fol = NULL;
    gzFile    mygzfile;
    float (*gscore)(int s1, int s2, void *matthresh) = NULL;
    int (*Igscore)(int s1, int s2, void *matthresh) = NULL;
    /* OLD SWResult (*myalign)(tyProt *p0, tyProt *p1, int automode, int K,
                 float minscore, float proba_limite, float minrms,
                 int (*Igscore)(int a1, int a2, void *mat_thresh),
                 void *mat_thresh, float gap_o, float gap_e,
                 float tauxrecouv, float cross_rms_seuil, int vverbose,
                 int SPN, WV *water_ving, int length, int elongation,
                 float rrmsd, int which_rmsd)=NULL;
    */
    void      *mat_or_thresh = NULL;
    pam_matrix    mat;
    char      matname[PATH_MAX] = "\0";

#if AFFICHAGE_TMP
    FILE  *pFi_tmp;
#endif
    FILE  *pFi_gok = NULL, *pFi_py = NULL;
#if CALCUL_MTD
    char * nom_mtd = "fichier_pour_proba_mtd.txt";
#endif
    FILE *f1 = NULL, *f2 = NULL;
    char nomFi_gok[PATH_MAX], nomFi_py[PATH_MAX];
    char nomFi[PATH_MAX] = "\0";
    char *nom_defaut = "out";
#if AFFICHAGE_SOMME_SCORES
    SommeRes **ParMatch;
    SommeRes ***Groupes;
#endif
#if (AFFICHAGE_TRANSITIONS ) || ( AFFICHAGE_TRANSITIONS2)
    int **tproba_simple = NULL; /*tableau contenant les nombre de transitions simples  */
    int ****tpar_paire = NULL;  /*tableau contenant les nombre de transitions en fonction des transitions precedentes*/
#endif

    /*mesure du temps !!*/
    time_t now;
    time_t now2;
    double diff_temps = 0.0;

#ifdef MEMOIRE
    char *err_msg = NULL;
#endif

    /*********************    pour les sequences    *************************/

    HashTable data_hash, shuf_hash ;
    char alphabet[MAX_SYMB];
    int lg_mot = 0;
    double seuil_score =0.0;
    float moyenne=0.0, ecart_type=0.0, zscore=0.0; /*pour le Z-score*/
    float freq[4];  /*contient les frequences des acides nucleiques*/
    WV *water_ving = NULL;

    /*Initialisations*/
    time(&now);


    strcpy(matname, "Blosum62");
    parms.minscore = -1;
    parms.proba      = PROBA;     /*N*/
    parms.minrms     = DEFAULT_MINRMS;        /*M*/
    parms.cross_rms  = DEFAULT_CROSSEDRMS;        /*Z*/
    parms.nbest      = -1;        /*n*/
    parms.angleMax   = DEFAULT_ANGLE_MAX;     /*g*/

    parms.gap_o        = -2;          /* bidon, DOIT etre change */
    parms.gap_e       = -2;           /* bidon, DOIT etre change */

    parms.recover    = DEFAULT_RECOVER;       /*r*/
    parms.nbkept     = DEFAULT_KEPT_STRUCTURES;       /*k*/
    parms.threshold  = DEFAULT_THRESHOLD;     /*w*/
    automode = 0;             /* normal SW algo by default    */
    parms.st[0][0] = parms.st[1][0] = '\0';       /* input files */
    parms.format[0] = parms.format[1] = PDB_FORMAT;   /* PDB format by default */

    parms.verbose=0;          /* not in verbose mode by default   */
    parms.vverbose=0;         /* not in very verbose mode by default  */
    parms.nbStructSet=0;
    parms.nbStructpymol=0;

    /* algo utilise par default */
    /* JOMPO: on pourra enlever le cast quand swalign utilisera des */
    /* scores entiers						*/
    parms.sim = 1;             /* Huang algo by default, not SW    */
    parms.myalign = simalign;
    parms.nb_shuffle = -1;
    parms.nb_repeats = -1 ;

    parms.length = DEFAULT_LENGTH;
    parms.rrmsd = DEFAULT_RRMSD;
    parms.which_rmsd = 0;


    parms.output = 0;  /*par defaut output grand*/
    parms.elongation = 0;  /*elongation chaine de moins de 1 A pour structure */
    alphabet[0] = '\0';


    /* index_adn contient les indices A,C,G et T    */
    /* sous forme de 0,1,2,3            */
    for (i=0; i < 128; i++)
        index_adn[i] = 0;

    index_adn['A'] = 0;
    index_adn['C'] = 1;
    index_adn['G'] = 2;
    index_adn['T'] = 3;

    /* les lettres non suivies de ':' ne demandent  */
    /* pas d'argument supplementaire        */
#ifdef ADVANCED
    while ((carg = getopt(argc, argv, "a:A:b:B:c:D:e:Fg:G:hi:j:k:lL:m:M:n:N:o:pP:qr:R:s:S:t:T:u:UvVw:Wx:Z:z")) != -1) {
#else
    while ((carg = getopt(argc, argv, "a:b:D:e:Fg:G:hi:j:lL:m:M:n:N:pP:qr:s:u:UvVw:Wx:z")) != -1) {
#endif

        switch ((char)carg) {
        case 's':
        case 'b':
        case 'a':
            switch ((char)carg) {
            case 's':
                parms.format[0] = PDB_FORMAT;
                break;
            case 'b':
                parms.format[0] = BIN_FORMAT;
                break;
            case 'a':
                parms.format[0] = ASC_FORMAT;
                break;
            }
            if (sscanf(optarg, "%s", parms.st[0]) != 1)
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read 1st input file name\n", argv[0]);
            break;
#ifdef ADVANCED
        case 'S':
        case 'B':
        case 'A':
            switch ((char)carg) {
            case 'S':
                parms.format[1] = PDB_FORMAT;
                break;
            case 'B':
                parms.format[1] = BIN_FORMAT;
                break;
            case 'A':
                parms.format[1] = ASC_FORMAT;
                break;
            }
            if (sscanf(optarg, "%s", parms.st[1]) != 1)
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read 2nd input file name\n", argv[0]);
            break;
#endif

        case 'x':
            if (sscanf(optarg, "%s", matname) != 1)
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read matrix (option -x)\n", argv[0]);
            break;

        case 'W':
            parms.sim = 0;
            break;

        case 'F':
            parms.no_size_limit = 1;
            break;

        case 'P':
            if (sscanf(optarg, "%d", &SPN) != 1)
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read P : type of analysis\n", argv[0]);
            break;

        case 'q':
            getscore = GetSWScore;
            compar = ComparScore;
            qscompar=sCtreeComparCroissant;
            break;



        case 'G':
            if (sscanf(optarg, "%f", &(parms.gap_o)) != 1) {
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read gap open penalty (-G option)\n", argv[0]);
            }
            /* si le gap est negatif, il sera remis comme il faut en dessous    */
            break;

        case 'g':
            if (sscanf(optarg, "%f", &(parms.gap_e)) != 1) {
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read gap extention penalty (-g option)\n", argv[0]);
            }
            /* si le gap est negatif, il sera remis comme il faut en dessous    */
            break;

#ifdef ADVANCED
        case 'Z':
            if (sscanf(optarg, "%f", &(parms.cross_rms)) != 1) {
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read RMS threshold for compatible blocs (-Z option)\n", argv[0]);
            }
            if (parms.cross_rms <= 0.0) {
                fprintf(stderr,"option -Z: RMS threshold for compatible blocs is <= 0.0, setting it to %f\n",
                        DEFAULT_CROSSEDRMS);
                parms.cross_rms = DEFAULT_CROSSEDRMS;
            }
            break;

        case 'k':
            if (sscanf(optarg, "%d", &(parms.nbkept)) != 1) {
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read number of kept structures for output (-k option)\n", argv[0]);
            }
            if (parms.nbkept <= 0.0) {
                fprintf(stderr,"option -k: number of kept structures for output is < 0, setting it to %d\n",
                        DEFAULT_KEPT_STRUCTURES);
                parms.nbkept = DEFAULT_KEPT_STRUCTURES;
            }
            break;
#endif
        case 'w':
            if (sscanf(optarg, "%f", &(parms.threshold)) != 1) {
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read threshold (-W option)\n", argv[0]);
            }
            if (parms.threshold <= 0.0) {
                fprintf(stderr,"option -w: threshold is <= 0.0, setting it to %f\n",
                        DEFAULT_THRESHOLD);
                parms.threshold = DEFAULT_THRESHOLD;
            }
            break;

        case 'r':
            if (sscanf(optarg, "%f", &(parms.recover)) != 1) {
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read recovering ratio (-r option)\n", argv[0]);
            }
            if (parms.recover < 0.0 || parms.recover > 1.0) {
                fprintf(stderr,"option -r: recovering ratio is not in [0,1], setting it to %f\n",
                        DEFAULT_RECOVER);
                parms.recover = DEFAULT_RECOVER;
            }
            break;

        case 'N':
            if (sscanf(optarg, "%f", &(parms.proba)) != 1) {
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read proba (-N option)\n", argv[0]);
            }
            parms.proba = 1-parms.proba; /*on inverse pour les calculs*/
            if (parms.proba <= 0.0) {
                fprintf(stderr,"option -N: proba is <= 0.0, setting it to %f\n", PROBA);
                parms.proba = PROBA;
            }
            break;

        case 'l':
            parms.output = 1;
            break;

        case 'm':
            if (sscanf(optarg, "%f", &(parms.minscore)) != 1) {
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read minimum score (-m option)\n", argv[0]);
            }
            break;

        case 'z':

            if (SPN == STRUCTURE)
                parms.minscore = DEFAULT_MINSCORE_STRUCT;
            if (SPN == PROTEIQUE)
                parms.minscore = DEFAULT_MINSCORE_PROT;
            if (SPN == NUCLEIQUE)
                parms.minscore = DEFAULT_MINSCORE_NUC;

            break;

        case 'M':
            if (sscanf(optarg, "%f", &(parms.minrms)) != 1) {
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read minimum rms (-M option)\n", argv[0]);
            }
            if (parms.minrms <= 0.0) {
                fprintf(stderr,"option -m: minimum rms is <= 0.0, setting it to %f\n",
                        DEFAULT_MINRMS);
                parms.minrms = DEFAULT_MINRMS;
            }
            break;

        case 'v':
            parms.verbose = 1;  /* verbose mode */
            break;

        case 'V':
            parms.vverbose = 1; /* very verbose mode    */
            parms.verbose = 1;
            break;

        case 'h':
            PrintHelp(argv[0]);
            exit(0);
            break;

        case 'n':
            if (sscanf(optarg, "%d", &(parms.nbest)) != 1)
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read number of wanted best subalignments\n",argv[0]);
            if (parms.nbest < 1) {
                fprintf(stderr,
                        "%s: subalignment number cannot be less than 1, setting it to %d\n",
                        argv[0], DEFAULT_NBEST);
                parms.nbest = DEFAULT_NBEST;
            }
            break;

        case 'L':
            if (sscanf(optarg, "%d", &(parms.length)) != 1)
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read minimal length\n",argv[0]);
            if (parms.length < 0) {
                fprintf(stderr,
                        "%s: minimal length cannot be less than 0, setting it to %d\n",
                        argv[0], DEFAULT_LENGTH);
                parms.length = DEFAULT_LENGTH; /* 0 */
            }
            break;

        case 'p':
            WV2 = 1;
            break;

#ifdef ADVANCED
        case 'R':
            if (sscanf(optarg, "%d", &parms.nb_repeats) != 1) {
                fprintf(stderr, " cannot read number of repeats for statistics, setting it to default" );
            }
            break;


        case 'c':
            if (sscanf(optarg, "%d", &(parms.angleMax)) != 1)
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read number of angles groups\n", argv[0]);
            if (parms.angleMax<3 || parms.angleMax>=NOT_AN_ANGLE ) {
                fprintf(stderr,"%s: bad number of groups (%d), setting to %d\n",
                        argv[0], parms.angleMax, DEFAULT_ANGLE_MAX);
                parms.angleMax=DEFAULT_ANGLE_MAX;
            }
            break;

        case 't':
            if (sscanf(optarg, "%d", &parms.nbStructSet) != 1) {
                fprintf(stderr, "%s: cannot read number of structures for Gok file,"
                        " set to all structures\n", argv[0]);
                parms.nbStructSet=-1;
            }
            break;

        case 'T':
            if (sscanf(optarg, "%d", &parms.nbStructpymol) != 1) {
                fprintf(stderr, "%s: cannot read number of structures for pymol file,"
                        " set to all structures\n", argv[0]);
                parms.nbStructpymol=-1;
            }
            break;


        case 'o':
            if (  parms.nbStructpymol<1 && parms.nbStructSet<1) {  /*pas de sortie speciale*/
                fprintf(stderr, " you haven't chosen any option : gok, pymol .... \n ignored ...\n");
                break;
            }
            if (sscanf(optarg, "%s", nomFi) != 1)
                MyError(stderr, ERR_SWAKUSA,  "%s: cannot read output file name\n", argv[0]);
            break;
#endif

        case 'D':
            if (sscanf(optarg, "%d", &parms.nb_shuffle) != 1) {
                fprintf(stderr, "%s: cannot read number of shuffle \n", argv[0]);
                parms.nb_shuffle = NB_SHUFFLE;
            }
            break;

        case 'e':
            if (sscanf(optarg, "%d", &parms.elongation) != 1) {
                fprintf(stderr, "%s: cannot read extend \n", argv[0]);
                parms.elongation = 0;
            }
            break;

        case 'i':
            if ((! ( f1 = freopen(optarg, "w", stdout)))) {
                MyError(stderr, ERR_SWAKUSA,  "Cannot open out file (-i option)\n", argv[0]);
            }
            break;

        case 'j':
            if ((! (f2 = freopen(optarg, "w", stderr)))) {
                MyError(stderr, ERR_SWAKUSA,  "Cannot open error file (-j option)\n", argv[0]);
            }
            break;

        case 'u':
            if (sscanf(optarg, "%f", &parms.rrmsd) != 1) {
                fprintf(stderr, "%s: cannot read maximum rrmsd \n", argv[0]);
                parms.rrmsd = DEFAULT_RRMSD;
            }
            break;

        case 'U':
            parms.which_rmsd = 1;   /* rmsd normal  */
            break;

        case '?':
            errflg++;
            break;

        }     /* end switch   */
    }            /* end while    */


    if (parms.sim == 1) { /* Huang and Miller algorithm */
        parms.myalign = simalign;
    } else { /* Smith and Waterman algorithm */
        parms.myalign = (SWResult*  (*)(tyProt *p0, tyProt *p1, int automode, int K,
                                        float minscore, float proba_limite, float minrms,
                                        int (gscore)(int a1, int a2, void *mat_thresh),
                                        void *mat_thresh, float gap_o, float gap_e,
                                        float tauxrecouv, float cross_rms_seuil, int vverbose,
                                        int SPN, WV *water_ving, int length, int elongation,
                                        float rrmsd, int which_rmsd) )swalign;
    }

    /* Selon structure, sequence proteique ou nucleique  */

    switch (SPN) {

    case STRUCTURE:
        strcpy(alphabet, PROT_ALPHABET);
        if (parms.gap_o <=0) {
            parms.gap_o = DEFAULT_GAP_STRUCT;
            parms.gap_e = DEFAULT_GAP_STRUCT_EXT;
        }

        if (parms.sim == 0)
            gscore = (float (*)(int a1, int a2, void *matthresh))anglescorenouveau;
        else if (parms.sim != 0)
            Igscore = (int (*)(int a1, int a2, void *matthresh))Ianglescorenouveau;


        if (parms.minscore < 0.0)
            parms.minscore = DEFAULT_MINSCORE_STRUCT;

        if (parms.nbest < 0)
            parms.nbest = DEFAULT_NBEST;

        mat_or_thresh = (void *)(&(parms.threshold));

        break;

    case PROTEIQUE:

        /* if not filled, set default values */
        if (WV2 == 1) {
            if (parms.nb_shuffle == -1)
                parms.nb_shuffle = NB_SHUFFLE_PROT;
            if (parms.nb_repeats == -1)
                parms.nb_repeats = NB_REPEATS_PROT;
        } else {
            if (parms.nb_shuffle == -1)
                parms.nb_shuffle = NB_SHUFFLE;
        }

        strcpy(alphabet, PROT_ALPHABET);
        lg_mot = 1;

        if ( parms.gap_o <=0) {
            parms.gap_o = DEFAULT_GAP_PROT;
            parms.gap_e = DEFAULT_GAP_PROT_EXT;
        }

        if (parms.sim == 0)
            gscore = (float (*)(int a1, int a2, void *matthresh))aascore;
        else
            Igscore = (int (*)(int a1, int a2, void *matthresh))Iaascore;

        if (matname[0] != '\0') {
            if (load_matrix(&mat, matname) == faux)
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read matrix in file %s\n", argv[0],matname);
            reorder_matrix(&mat, PROT_ALPHABET);
            /*          write_matrix("mat.txt", &mat);*/
        } else {

            if (load_matrix(&mat, "Blosum62") == faux)
                MyError(stderr, ERR_SWAKUSA,
                        "%s: cannot read matrix in file Blosum62\n", argv[0]);

            reorder_matrix(&mat, PROT_ALPHABET);
            /*  write_matrix("mat.txt", &mat);*/
            /*   unitary_matrix(&mat, MAT_PAM_TYPE, PROT_ALPHABET); */
        }

        mat_or_thresh = (void*)&mat;

        /* printinfo_matrix(stderr, &mat, "HEADER", 1 , 0 );*/
        break;

    case NUCLEIQUE:

        /*si pas remplit on met les valeurs par defaut */
        if (WV2 == 1) {
            if (parms.nb_shuffle == -1)
                parms.nb_shuffle = NB_SHUFFLE_ADN;
            if (parms.nb_repeats == -1)
                parms.nb_repeats = NB_REPEATS_ADN;
        } else {
            if (parms.nb_shuffle == -1)
                parms.nb_shuffle = NB_SHUFFLE;
        }


        strcpy(alphabet, "ACGT");
        lg_mot = 1;/*LGMOT_SHUFFLE_NUC;*/ /*en codon*/

        if (parms.gap_o <= 0.0) {

            parms.gap_o = DEFAULT_GAP_NUC;
            parms.gap_e = DEFAULT_GAP_NUC_EXT;
        }
        if (parms.sim == 0)
            gscore = (float (*)(int a1, int a2, void *matthresh))anuc_score;
        else
            Igscore = (int (*)(int a1, int a2, void *matthresh))Ianuc_score;

        mat_or_thresh = (void*)&mat;
        break;

    }

    /*  --------------------------------------------------  */
    /*        affiche le resume des parametres utilises     */
    /*  --------------------------------------------------  */

    PrintResumer (parms, SPN, proba, nomFi, matname, gscore, Igscore );

    /* ------------------------------------  */
    /* verification des parametres       */
    /* ------------------------------------  */

    if (errflg > 0) {
        MyError(stderr,0,"%s: option inexistante\n", argv[0]);
        PrintHelp(argv[0]);
        exit(ERR_SWAKUSA);
    }

    /***********************************************/
    /*mise en place des stats pour les sequences : */
    /***********************************************/


    if (SPN != STRUCTURE) {
        BuiltHashTable(&data_hash, alphabet, lg_mot);
        BuiltHashTable(&shuf_hash, alphabet, lg_mot);
        sranUN((int) time(0) & 0xfffffff);
    }


    /* si fichier de sortie spcial : nom du fichier*/

    if (SPN == STRUCTURE) {
        if ( parms.nbStructpymol>0 || parms.nbStructSet>0) {
            if (strlen(nomFi) == 0)
                strcpy( nomFi , nom_defaut);
            if (parms.nbStructSet>0)
                sprintf( nomFi_gok,"%s%s" ,nomFi,  EXTENSION_GOK);
            if (parms.nbStructpymol>0)
                sprintf( nomFi_py,"%s%s", nomFi,  EXTENSION_PYMOL);

        }
    }


#if AFFICHAGE_SOMME_SCORES
    /*****************************************/
    /*  initialisation des tableaux de proba */
    /*****************************************/

    if (proba==1) {
        if ((ParMatch = NEWN(SommeRes *, LONGUEUR_STAT)) == NULL)
            MyError(stderr, 13, "Not enough memory for ParMatch, exiting...\n");

        for (i=0; i<LONGUEUR_STAT; i++)
            if ((ParMatch[i] = NewSommeRes()) == NULL)
                MyError(stderr, 13, "Not enough memory for ParMatch[%d], exiting...\n",i);


        if ((Groupes = NEWN(SommeRes **, NB_GROUPE_MAX)) == NULL)
            MyError(stderr, 13, "Not enough memory for Groupes, exiting...\n");

        for (i = 0; i < NB_GROUPE_MAX; i++) {
            if ((Groupes[i] = NEWN(SommeRes *, LONGUEUR_STAT)) == NULL)
                MyError(stderr, 13, "Not enough memory for Groupe[%d], exiting...\n",i);
            for (j=0; j<LONGUEUR_STAT; j++)
                if ( (Groupes[i][j]= NewSommeRes()) == NULL)
                    MyError(stderr, 13, "Not enough memory for Groupes[%d][%d], exiting...\n",i, j);
        }
    }
#endif

#if (AFFICHAGE_TRANSITIONS ) ||    (AFFICHAGE_TRANSITIONS2)
    /***************************************************/
    /*initialisation des tableaux de comptage transition*/
    /****************************************************/

    if (proba == 1) {
        if ((tproba_simple = NEWN(int *, DEFAULT_ANGLE_MAX)) == NULL)
            MyError(stderr, 13, "Not enough memory for tproba_simple, exiting...\n");

        for (i=0; i<DEFAULT_ANGLE_MAX; i++)
            if ((tproba_simple[i] = NEWN(int, DEFAULT_ANGLE_MAX))==NULL)
                MyError(stderr, 13, "Not enough memory for tproba_simple[%d], exiting...\n",i);
            else
                for (j=0; j< DEFAULT_ANGLE_MAX; j++)
                    tproba_simple[i][j]=0;

#if TOUTES_TRANSITIONS
        if ((tpar_paire = NEWN(int ***, ORDRE_MARKOV)) == NULL)
            MyError(stderr, 13, "Not enough memory for tpar_paire, exiting...\n");

        for (i = 0; i < ORDRE_MARKOV; i++) {
            if ((tpar_paire[i] = NEWN(int **,  DEFAULT_ANGLE_MAX)) == NULL)
                MyError(stderr, 13, "Not enough memory for tpar_paire[%d], exiting...\n",i);
            for (j=0; j< DEFAULT_ANGLE_MAX; j++) {
                if ( (tpar_paire[i][j]= NEWN(int*,  DEFAULT_ANGLE_MAX)) == NULL)
                    MyError(stderr, 13, "Not enough memory for tpar_paire[%d][%d], exiting...\n",i, j);
                for (k=0; k< DEFAULT_ANGLE_MAX; k++)
                    if ( (tpar_paire[i][j][k]= NEWN(int,  DEFAULT_ANGLE_MAX)) == NULL)
                        MyError(stderr, 13, "Not enough memory for tpar_paire[%d][%d][%d], exiting...\n",i, j, k);
                    else
                        for (l=0; l< DEFAULT_ANGLE_MAX; l++)
                            tpar_paire[i][j][k][l] = 0;
            }
        }
#endif

    }
#endif

    /* ------------------------------------ */
    /* Ouverture des fichiers d'entree  */
    /* ------------------------------------ */

    /*******************************************************************************/
    /*                                 VERIFICATIONS DES FICHIERS                       */
    /*******************************************************************************/
    if (parms.st[0][0] == '\0') {
        MyError(stderr, 0, "%s:: error: No input 1st file !\n", argv[0]);
        PrintHelp(argv[0]);
        MyError(stderr, ERR_SWAKUSA, " exiting...\n");
    } else if (parms.format[0] == BIN_FORMAT ) { /*format binaire*/
        if ((pFi[0]=fopen(parms.st[0], "rb"))==NULL) {
            MyError(stderr, ERR_SWAKUSA, "%s:: error: cannot open %s\n", argv[0], parms.st[0]);
        }
    } else {
        if ((pFi[0]=fopen(parms.st[0], "rt"))==NULL) {  /*format normal*/
            MyError(stderr, ERR_SWAKUSA, "%s:: error: cannot open %s\n", argv[0], parms.st[0]);
        }
    }

    if (parms.st[0][strlen(parms.st[0])-1]=='Z' && parms.st[0][strlen(parms.st[0])-2] == '.') {
        if ((fol = tmpfile()) == NULL) {
            MyError(stderr, 1,  "%s:: error: cannot open temporary file\n",
                    argv[0]);
        }
        if (unZ(pFi[0], fol) < 10) {
            MyError(stderr, 1,  "%s:: Problem uncompressing 1st file %s, exiting....\n",
                    argv[0], parms.st[0]);
        }
        fclose(pFi[0]);
        MyError(stderr, 0,  "-> 1st file uncompressed (%s)\n", parms.st[0]);
        pFi[0] = fol;
    }
    else if (parms.st[0][strlen(parms.st[0])-1]=='z' &&
             parms.st[0][strlen(parms.st[0])-2] == 'g' &&
             parms.st[0][strlen(parms.st[0])-3] == '.') {
        if ((fol = tmpfile()) == NULL) {
            MyError(stderr, 1,  "%s:: error: cannot open temporary file\n",
                    argv[0]);
        }
        fclose(pFi[0]);
        if ((mygzfile = gzopen(parms.st[0], "rb")) == NULL) {
           MyError(stderr, 2, "Cannot open gz file %s, exiting....\n",parms.st[0]);
        } 
        if (gz_uncompress(mygzfile, fol) != 0) {
            MyError(stderr, 1,  "%s:: Problem uncompressing 1st gz file %s, exiting....\n",
                    argv[0], parms.st[0]);
        }
        MyError(stderr, 0,  "-> 1st file uncompressed (%s)\n", parms.st[0]);
        pFi[0] = fol;
    }



    /* is there a second file ? if not : automode */
    if (parms.st[1][0] == '\0') {
        automode = 1;
    } else { /*2e fichier*/
        if (parms.format[1] == BIN_FORMAT ) { /*format binaire*/
            if ((pFi[1]=fopen(parms.st[1], "rb"))==NULL) {
                MyError(stderr, 0, "%s:: error: cannot open %s\n", argv[0], parms.st[1]);
                PrintHelp(argv[0]);
                exit(ERR_SWAKUSA);
            }
        } else {
            if ((pFi[1]=fopen(parms.st[1], "rt"))==NULL) { /*format normal*/
                MyError(stderr, 0, "%s:: error: cannot open %s\n", argv[0], parms.st[1]);
                PrintHelp(argv[0]);
                exit(ERR_SWAKUSA);
            }
        }
        if (parms.st[1][strlen(parms.st[1])-1]=='Z' && parms.st[1][strlen(parms.st[1])-2] == '.') {
            if ((fol = tmpfile()) == NULL) {
                MyError(stderr, 1,  "%s:: error: cannot open temporary file\n",
                        argv[0]);
            }
            if (unZ(pFi[1], fol) < 10) {
                MyError(stderr, 1,  "%s:: Problem uncompressing 2nd file %s, exiting....\n",
                        argv[0], parms.st[0]);
            }
            fclose(pFi[1]);
            MyError(stderr, 0,  "-> 2nd file uncompressed (%s)\n", parms.st[1]);
            pFi[1] = fol;
        }
        else if (parms.st[1][strlen(parms.st[1])-1]=='z' &&
                 parms.st[1][strlen(parms.st[1])-2] == 'g' &&
                 parms.st[1][strlen(parms.st[1])-3] == '.') {
            if ((fol = tmpfile()) == NULL) {
                MyError(stderr, 1,  "%s:: error: cannot open temporary file\n",
                        argv[0]);
            }
            fclose(pFi[1]);
            if ((mygzfile = gzopen(parms.st[1], "rb")) == NULL) {
               MyError(stderr, 2, "Cannot open gz file %s, exiting....\n",parms.st[1]);
            }
            if (gz_uncompress(mygzfile, fol) != 0) {
               MyError(stderr, 3, 
                          "Problem uncompressing gz file %s, exiting....\n",
                          parms.st[1]);
            }
            MyError(stderr, 0,  "-> 2nd file uncompressed (%s)\n", parms.st[1]);
            pFi[1] = fol;
        }
    }


#ifdef MEMOIRE

    control_prog(argv[0], getpid(), 1000000, 100000, &err_msg) ;
    fprintf(stderr, "memtest : avant lecture\n");
#endif

    /*******************************************************************************/
    /*                                 LECTURE DU 1ER FICHIER                       */
    /*******************************************************************************/

    nprot0 = 0;  /*nombre de proteines du premier fichier*/

    while (feof(pFi[0])==0) {

        if (SPN == STRUCTURE) {
            p0 = LitStructure(pFi[0], parms.st[0], parms.verbose, parms.format[0]);
            if (p0 == (void*)BAD_POINTER || p0 == NULL) {
                if (feof(pFi[0]) == 0) {  /* si pas fin de fichier  */
                    fflush(stderr);
                    MyError(stderr, ERR_SWAKUSA,
                            "Problem in reading protein structure in %s %d eme proteine\n, exiting...",  parms.st[0], nprot0);
                    continue;
                } else { /*on a lu la derniere proteine : fin de fichier*/
                    if (parms.verbose)
                        fprintf(stderr, "End of first database file, %d proteins have been read\n",nprot0);
                    continue;
                }
            } else if (SPN == STRUCTURE && (RecodeAngles(p0, parms.angleMax))==PASBON) {
                FreeProt(p0);
                MyError(stderr, ERR_SWAKUSA,
                        "Problem in recoding protein %s angles\nexiting...\n",  p0->nomProt);
            }
        }

        else {  /*PROTEINE ou NUCLEIQUE*/
            if (parms.format[0] == PDB_FORMAT) /* mean ascii compared to binary !!! */
                p0 = LitFormatFasta (pFi[0], parms.verbose , alphabet, 0);
            if (parms.format[0] == BIN_FORMAT)
                p0 = LitFormatFasta_Binaire (pFi[0], parms.verbose , alphabet, 0);
            while (( p0 == (void*)BAD_POINTER || p0 == NULL) && feof(pFi[0]) == 0) {
                fflush(stderr);
                MyError(stderr, 0,
                        "**********Problem in reading sequence in %s %d th reading\n...",  parms.st[0], nprot0);
                if (parms.format[0] == PDB_FORMAT)
                    p0 = LitFormatFasta (pFi[0], parms.verbose , alphabet, 0);
                if (parms.format[0] == BIN_FORMAT)
                    p0 = LitFormatFasta_Binaire (pFi[0], parms.verbose , alphabet, 0);
            }
            if (( p0 == (void*)BAD_POINTER || p0 == NULL) && feof(pFi[0]) != 0) { /* si fin de fichier  */
                if (parms.verbose)
                    fprintf(stderr, "End of first database file, %d proteins have been read\n",nprot0+1);
                continue;
            }

            /*calcul des frequences*/

            ResetFrequencies(&data_hash);
            EncodeSequence( p0->tNameRes, &data_hash);
            ComputeFrequencies( p0->tNameRes, &data_hash, p0->nbRes);
            DecodeSequence( p0->tNameRes, &data_hash);
        }

        /*moins de 15 residus : on arrete la boucle*/
        if (SPN == STRUCTURE && p0->nbRes < 15) {
            FreeProt(p0);
            continue;
        }
        if ( parms.sim == 0 && p0->nbRes >NB_MAX_RESIDUS && ! parms.no_size_limit) {
            /*plus de NB_MAX_RESIDUS, on arrete !!*/
            FreeProt(p0);
            fprintf(stderr, " plus de %d residus, on passe a la suivante ... %s %d \n",NB_MAX_RESIDUS, p0->nomProt, p0->nbRes);
            continue;
        }

        nprot0 ++;
        /*    if (parms.verbose || parms.vverbose)*/

        /*mesure du temps !!*/
        time(&now2);
        diff_temps = difftime(now2, now);
        now = now2;

        fprintf(stderr, "****************  reading %s temps %f num prot %d\n",p0->nomProt, diff_temps, nprot0);
       /* if (parms.output == 0)
            fprintf(stdout, "****************  reading %s\n",p0->nomProt);*/

        nprot1 = 0;  /*nombre de proteines dans le 2nd fichier*/

        if (automode)
            p1 = p0;
        else
            p1 = NULL;

        /*seulement si on n'est pas en automode */
        if (! automode)
            init_minimier(parms.nbkept);

        /*******************************************************************************/
        /*                                 LECTURE DU 2E FICHIER                       */
        /*******************************************************************************/

        /*si automode : on ne rentre qu'une fois dans la boucle pour chaque p0*/
        /*et p1 est non nul quand on entre dans la boucle et est mis  0 a la fin de la boucle  */
        /*si pas automode, on parcourt la boucle une fois par proteine du fichier*/
        while ((automode && p1 != NULL) || (!automode && feof(pFi[1])==0)) {

            if (! automode) {
                if (SPN == STRUCTURE)  {
                    p1 = LitStructure(pFi[1], parms.st[1], parms.verbose, parms.format[1]);
                    if (p1 == (void*)BAD_POINTER || p1 == NULL) {
                        if (feof(pFi[1])==0)  {
                            /*si ce n'est pas la fin du fichier*/
                            MyError(stderr, 0,
                                    "Problem in reading the %dth protein (ignored)\n",  nprot1+1);
                            nbProtPbLecture++;
                            continue;
                        } else {
                            /* end of database file   */
                            if (parms.verbose)
                                fprintf(stderr, "End of second database file, %d proteins have been read\n",nprot1+1);
                            continue;
                        }
                    } else {  /* la proteines a ete lue */
                        /* le recodage met les angles non definis a parms.angleMax   */
                        if (SPN == STRUCTURE && (RecodeAngles(p1, parms.angleMax))==PASBON) {
                            if (parms.verbose)
                                fprintf(stderr, "Problem in recoding protein %s angles (ignored)\n", p1->nomProt);
                            nbProtPbRecodage++;
                            FreeProt(p1);
                            continue;
                        }
                    }
                }  /*fin structure*/
                else { /*PROTEINE ou NUCLEIQUE*/
                    if (parms.format[0] == PDB_FORMAT)
                        p1 = LitFormatFasta (pFi[1], parms.verbose , alphabet, 1);
                    if (parms.format[0] == BIN_FORMAT)
                        p1 = LitFormatFasta_Binaire (pFi[1], parms.verbose , alphabet, 1);
                    while (( p1 == (void*)BAD_POINTER || p1 == NULL) && feof(pFi[1]) == 0) {
                        fflush(stderr);
                        MyError(stderr, 0,
                                "========Problem in reading sequence in %s %d th reading\n...",
                                parms.st[1], nprot1);
                        p1 = LitFormatFasta (pFi[1], parms.verbose , alphabet,1);
                        MyError(stderr, 0, "p0 : pb format binaire\n...");
                        nprot1 ++;
                    }
                    if (( p1 == (void*)BAD_POINTER || p1 == NULL) &&
                            feof(pFi[1]) != 0) {  /* si fin de fichier  */
                        if (parms.verbose)
                            fprintf(stderr, "End of first database file, %d proteins have been read\n",
                                    nprot1+1);
                        continue;
                    }
                }

                /*meme nom de prot*/
                if (p0 != NULL && p1 != NULL && strcmp(p0->nomProt, p1->nomProt) == 0 ) {
                    FreeProt(p1);
                    p1=NULL;
                    continue;
                }


                /*moins de 15 residus : on arrete la boucle*/
                if (SPN == STRUCTURE && p1->nbRes < 15) {
                    FreeProt(p1);
                    continue;
                }
                if ( parms.sim == 0 && p1->nbRes >NB_MAX_RESIDUS  && ! parms.no_size_limit) {
                    /*plus de NB_MAX_RESIDUS, on arrete !!*/
                    FreeProt(p1);
                    fprintf(stderr, " plus de %d residus, on passe a la suivante ... %s %d \n",
                            NB_MAX_RESIDUS, p1->nomProt, p1->nbRes);
                    continue;
                }

                nprot1++;
                /* if (parms.verbose || parms.vverbose)*/
                fprintf(stderr, "================   reading %s num prot %d\n",p1->nomProt, nprot1);
                /*    fprintf(stderr, " %s\n",p1->tNameRes);*/

            }  /* end if not automode    */


            /* affichage longueur p1*/
            /* fprintf(stderr, "longueurp1g %d  nom  %s %c\n",p1->nbRes,p1->nomProt,p1->tFrag[0].name);  */
            /* fprintf(stderr, "longueurp0g %d  nom  %s %c\n",p0->nbRes,p0->nomProt ,p0->tFrag[0].name );   */
            /* affiche les angles bizarres(dernier arg = 1) de p0   */
            /*     sPrintAngles(p0, 0, p0->nbRes-4, 1); */
#ifdef MEMOIRE
            control_prog(argv[0], getpid(), 1000000, 100000, &err_msg) ;
            fprintf(stderr, "memtest : apres lecture\n");
            fflush(stderr);
#endif

            /*******************************************************************************/
            /*                                 RESULTATS                                   */
            /*******************************************************************************/

            if (SPN == NUCLEIQUE) { /*calcul des frequences des bases*/
                /* mise a zero d'abord */
                for (i=0;i<4;i++)
                    freq[i] = 0;

                for (i=0; i<p0->nbRes; i++)
                    freq[index_adn[(int)(p0->tNameRes[i])]]++;

                for (i=0;i<4;i++) {
                    freq[i] = freq[i]/ p0->nbRes;
                }

                compute_nuc_matrix(&mat, freq, NULL); /*pas de calcul des gaps*/
            }

#ifdef MEMOIRE
            control_prog(argv[0], getpid(), 1000000, 100000, &err_msg) ;
            fprintf(stderr, "memtest : apres nuc matrice \n");
            fflush(stderr);
#endif



            /*affichage gaps*/
            /*       fprintf(stderr, "open gap %.1f extended gap %.1f\n", parms.gap_o, parms.gap_e);  */
            /*on cherche le seuil de score pour les sequences*/
            if (SPN != STRUCTURE) {
                if (parms.sim == 1) {
                    if (WV2 == 1) { /*waterman vingron methode 2*/
                        water_ving =  new_calcule_seuil_vingron2(p0, p1, &data_hash ,&shuf_hash,
                                SPN , alphabet, 0.3, lg_mot,&moyenne,
                                &ecart_type, &zscore, automode,
                                Igscore, mat_or_thresh, parms.gap_o, parms.gap_e,
                                parms.recover, parms.cross_rms,
                                parms.vverbose, parms.nb_shuffle, parms.nb_repeats, NULL);
                        if (water_ving == NULL) {
                            fprintf(stdout, "probleme shuffle\n");
                            parms.nbest = 1;
                        }
                    } else {
                        if (parms.minscore <0 && parms.nbest < 0 && parms.length == 0) {
                            /* generer des sequences aleatoires, seulement si pas de   */
                            /* score minimum, sinon, on prend le score min comme seuil, */
                            /* et si pas de nb precis  a sortir */ /*waterman vingron methode 1*/
                            water_ving =  calcule_seuil_vingron2(p0, p1, &data_hash ,&shuf_hash,
                                                                 SPN , alphabet, 0.3, lg_mot,&moyenne,
                                                                 &ecart_type, &zscore, automode,
                                                                 Igscore, mat_or_thresh, parms.gap_o, parms.gap_e,
                                                                 parms.recover, parms.cross_rms,
                                                                 parms.vverbose, parms.nb_shuffle, NULL);
                            if (water_ving == NULL) {
                                fprintf(stdout, "probleme shuffle\n");
                                parms.nbest = 1;
                            }
                        }
                    }
                } /* end if parms.sim */
                else {
                    if (WV2 == 1) { /*waterman vingron methode 2*/
                        water_ving =  new_calcule_seuil_vingron(p0, p1, &data_hash ,&shuf_hash,
                                                                SPN , alphabet, 0.3, lg_mot,&moyenne,
                                                                &ecart_type, &zscore, automode,
                                                                gscore, mat_or_thresh, parms.gap_o, parms.gap_e,
                                                                parms.recover, parms.cross_rms,
                                                                parms.vverbose, parms.nb_shuffle, parms.nb_repeats, NULL);
                        if (water_ving == NULL) {
                            fprintf(stdout, "probleme shuffle\n");
                            parms.nbest = 1;
                        }
                    } else {
                        if (parms.minscore <0 && parms.nbest < 0 && parms.length == 0) {
                            /* generer des sequences aleatoires, seulement si pas de   */
                            /* score minimum, sinon, on prend le score min comme seuil, */
                            /* et si pas de nb precis  a sortir */ /*waterman vingron methode 1*/
                            water_ving =  calcule_seuil_vingron(p0, p1, &data_hash ,&shuf_hash,
                                                                SPN , alphabet, 0.3, lg_mot,&moyenne,
                                                                &ecart_type, &zscore, automode,
                                                                gscore, mat_or_thresh, parms.gap_o, parms.gap_e,
                                                                parms.recover, parms.cross_rms,
                                                                parms.vverbose, parms.nb_shuffle, NULL);
                            if (water_ving == NULL) {
                                fprintf(stdout, "probleme shuffle\n");
                                parms.nbest = 1;
                            }
                        }
                    }
                } /* end if parms.sim */
            }

#ifdef MEMOIRE
            control_prog(argv[0], getpid(), 1000000, 100000, &err_msg) ;
            fprintf(stderr, "memtest : apres waterving \n");
#endif

            if  (parms.nbest >0 || parms.length > 0)
                seuil_score = 0;
            if (parms.minscore >0 ) {
                seuil_score = parms.minscore;
            }

            /* fprintf(stderr, "minscore %f, threshold score %f \n", parms.minscore, seuil_score); */

            /* alignement : tous les resultats sont contenus dans swres */
            if (parms.sim == 0)
                swres = swalign(p0,p1,automode,parms.nbest, seuil_score,
                                parms.proba, parms.minrms,
                                gscore, mat_or_thresh, parms.gap_o, parms.gap_e,
                                parms.recover, parms.cross_rms, parms.vverbose, SPN,
                                water_ving, parms.length, parms.elongation, parms.rrmsd,
                                parms.which_rmsd);
            else
                swres = simalign(p0,p1,automode,parms.nbest, seuil_score,
                                 parms.proba, parms.minrms,
                                 Igscore, mat_or_thresh, parms.gap_o, parms.gap_e,
                                 parms.recover, parms.cross_rms, parms.vverbose, SPN,
                                 water_ving, parms.length, parms.elongation, parms.rrmsd,
                                 parms.which_rmsd);
            /* NEW
            swres = parms.myalign(p0,p1,automode,parms.nbest, seuil_score,
                               parms.proba, parms.minrms,
                               gscore, mat_or_thresh, parms.gap_o, parms.gap_e,
                               parms.recover, parms.cross_rms, parms.vverbose, SPN,
                               water_ving, parms.length, parms.elongation, parms.rrmsd,
                               parms.which_rmsd);
             */

            if (parms.vverbose) {
                fprintf(stderr,"%s %s nmatch %d\n", p0->nomProt, p1->nomProt, swres->nmatch);
                fprintf(stderr,"score :  %f nb match %d threshold score=%f (ERR_SHUFFLE=%d)\n",
                        swres->score[0], swres->nmatch, seuil_score, ERR_SHUFFLE);
            }

#ifdef MEMOIRE
            control_prog(argv[0], getpid(), 1000000, 100000, &err_msg) ;
            fprintf(stderr, "memtest : apres SW \n");
#endif

            if (swres->nmatch > 0 ) {   /*calcul nb resultats */
                if (SPN != STRUCTURE && water_ving != NULL) {
                    swres->nbprint = swres->nmatch;
                } else { /* SPN == STRUCTURE)  */
                    swres->nbprint = swres->nmatch;
                    /*ecriture du fichier pour gok*/
                    if (parms.nbStructSet>0) {
                        /* strcpy(nomFi_gok, parms.fiOut); */
                        pFi_gok=fopen(nomFi_gok, "wt");
                        if (pFi_gok==NULL)
                            fprintf(stderr, "Can't open set file %s\n", nomFi_gok);
                        else {
                            fprintf(stderr, "Print results for Gok in %s\n", nomFi_gok);
                            EcritToutDansSet(p0, p1,swres,  pFi_gok, parms.nbStructSet);
                            fclose(pFi_gok);
                        }
                    }
                    /* fin ecriture fichier gok */

                    /*debut fichier pymol*/
                    if (parms.nbStructpymol>0) {
                        pFi_py=fopen(nomFi_py, "wt");
                        if (pFi_py==NULL)
                            fprintf(stderr, "Can't open pymol file %s\n", nomFi_py);
                        else {
                            if (parms.verbose)
                                fprintf(stderr, "Print results for pymol in %s\n", nomFi_py);
                            EcritTousPymol(p0, swres, pFi_py, &parms, parms.nbStructpymol);
                            fclose(pFi_py);
                        }
                    }
                    /* fin fichier pymol */

                    if (proba == 1) {       /* calcul MTD  */
                        Set_TableFreq("");
                        CalculProbaMtd(swres,p0);
                        Free_TableFreq();
#if AFFICHAGE_SOMME_SCORES
                        /*additionne les resultats contenus dans swres au fur et a mesure*/
                        AddSommeRes (ParMatch ,swres);
                        AddGroupes(Groupes, swres);
#endif
#if AFFICHAGE_TRANSITIONS
                        /* sortie comptage */
                        if (proba == 1)
                            OutputComptages( swres, p0, tproba_simple, tpar_paire, 1);
#endif
#if AFFICHAGE_TRANSITIONS2
                        /* sortie comptage */
                        if (proba == 1)
                            OutputComptages( swres, p0, tproba_simple, tpar_paire, 2);
#endif
#if CALCUL_MTD
                        pFi3=fopen(nom_mtd, "a");
                        if (pFi==NULL)
                            fprintf(stderr, "Can't open file %s\n", nom_mtd);
                        else {
                            fprintf(stderr, "Print results for MTD in %s\n", nom_mtd);
                            calcul_mtd (swres, pFi3, p0);
                            fclose(pFi3);
                        }
#endif

                    } /*fin proba == 1*/
                } /*fin SPN == STRUCTURE*/

                /* fprintf(stderr,"nb results = %d \n",   swres->nbprint );*/

                if (parms.output == 0)
                    PrintResult(stdout, swres, p0, proba, SPN, moyenne, ecart_type);
                else  /*small output*/
                    PrintResult2(stdout, swres, p0, proba, SPN, moyenne, ecart_type,
                                 mat_or_thresh, parms.gap_o, parms.gap_e);

                if (SPN != STRUCTURE  || automode) {
                    FreeSWResult(swres, FREEPROT); /* la proteine bank est liberee ici */
                } else {  /* not automode */
                    oldres = range(swres, getscore, compar);
                    if (oldres != NULL) {   /* si oldres est NULL, l'arbre  n'etait pas encore plein   */
                        FreeSWResult(oldres, FREEPROT); /* la proteine bank p1 est liberee ici */
                        oldres = NULL;
                    }
                }    /* end if automode  */
            }  /*fin si nb match > 0 */
            else {     /* if swres->nmatch <= 0    */
                FreeSWResult(swres, FREEPROT); /* p1 est liberee ici */
                swres = NULL;
                fprintf(stderr,"************************no match\n");
            }
#ifdef MEMOIRE
            control_prog(argv[0], getpid(), 1000000, 100000, &err_msg) ;
            fprintf(stderr, "memtest : apres ecriture \n");
#endif
            if (SPN != STRUCTURE)
                MYFREE(water_ving);

            p1 = NULL;

        }        /*fin while(feof(pFi[1]==0)*/  /*2e fichier d'entree*/

#if AFFICHAGE_TMP && ( AFFICHAGE_TRANSITIONS ||  AFFICHAGE_TRANSITIONS2 )
        if ((pFi_tmp=fopen("tmp.txt", "w"))==NULL) {
            MyError(stderr, ERR_SWAKUSA," error: cannot open tmp.txt\n");
        }
        fprintf(pFi_tmp, "****************\nreading %s\n",p0->nomProt);

        affichage_proba(tproba_simple, tpar_paire, pFi_tmp);
        fclose(pFi_tmp);
#endif

        if (parms.verbose) {
            if (nbProtPbLecture > 0)
               fprintf(stderr, "Pb in reading %d sequences/structures\n", nbProtPbLecture);
            if (nbProtPbRecodage > 0)
               fprintf(stderr, "Pb in recoding %d sequences/structures\n", nbProtPbRecodage);
        }
        /* rewind file pF1, print results if not in automode, free p1's  */
        if (! automode) {
            rewind(pFi[1]);        /*retourne au debut du fichier*/
            if (SPN == STRUCTURE) {
                /*              PrintArbreScan(stdout, p0, proba, SPN,mat_or_thresh, parms.gap_o, parms.gap_e, qscompar); */
                Free_Minimier();
            }
        }

        if (nprot0 == 0 ||(! automode &&  nprot1  == 0)) {
            fprintf(stderr, "No protein read in one of the database file : file1 : %d file2 : %d\n", nprot0, nprot1);
        }

        /*p0 est deja liberee en automode avec swres : FreeSWResult(swres, FREEPROT) */
        if (! automode) {
            FreeProt(p0);
        }
    } /* end while feof(pFi[0])  lecture du premier fichier d'entree */


    FreeMatrix();  /*libere la matrice d'alignement SW*/
    FreeMatrix_Gap();  /*libere la matrice d'extension de gap*/

    if (!automode)
        fclose(pFi[1]);

    fclose(pFi[0]);


    if (SPN != STRUCTURE) {
        FreeHashTable( &data_hash); /*libere la memoire des tables de hachage*/
        FreeHashTable( &shuf_hash);
    }

#ifdef MEMOIRE
    /* ------------------------------------  */
    /* Affichage de la memoire occupee   */
    /* ------------------------------------  */
    if (control_prog(argv[0], getpid(), 1000000, 100000, &err_msg) != 0)
        fprintf(stderr,"\n%s\n\n", err_msg);
    fprintf(stderr, "\n");
    fprintf(stderr, "memtest : fin fichier  \n");
#endif

    /*******************************************************************************/
    /*                               FIN LECTURE DU 1ER FICHIER                    */
    /*******************************************************************************/

    /* ----------------------------  */
    /* Ecriture des autres fichiers  */
    /* de sortie si necessaire           */
    /* ----------------------------  */

#if (AFFICHAGE_TRANSITIONS) ||  (AFFICHAGE_TRANSITIONS2)
    /*affichage des nombres de transition */
    if (proba == 1) {
        fprintf(stdout,"-------------- RESULTATS PROBA ---------------------\n");
        affichage_proba(tproba_simple, tpar_paire, stdout);
        FreeTransitions (tproba_simple, tpar_paire);
    }
#endif

#if AFFICHAGE_SOMME_SCORES
    /*affichage des sommes de tous les resultats*/
    if (proba == 1) {
        fprintf(stdout,"-------------- RESULTATS TOTAUX ---------------------\n");
        fprintf(stdout, "*******************************Resultats par match  ****************************\n");
        PrintSommeRes ( ParMatch);
        for (i=0; i<NB_GROUPE_MAX; i++) {
            fprintf(stdout, "*****************************Resultats pour le groupe %2d  ******************* \n", i);
            PrintSommeRes ( Groupes[i]);
        }
        /*libere les tableaux de stats qui additionnent tous les resultats*/
        FreeSommeRes (ParMatch, Groupes);
    }
#endif


    fprintf(stdout, "********** fin du fichier ************* \n");

    if (f1 != NULL)
        fclose(f1);
    if (f2 != NULL)
        fclose(f2);

    return(0);

}
