/* --------------------------------------------------	*/
/* file swakusa.h					*/
/* --------------------------------------------------	*/

#include "libphipsi.h"        /* for BinaryRes et PdbNum      */
#include "libqbestfit.h" /* for COORDS   */

#ifdef SUN
#else  /* not SUN */
#ifdef IRIX
#include <limits.h>             /* for MAX_PATH  */       
#endif /* IRIX */
#ifdef Darwin
#include <sys/syslimits.h> /* for MAX_PATH   */      
#endif
#ifdef Linux
#include <linux/limits.h>       /* for MAX_PATH */        
#endif /* Linux */
#endif /* not SUN */

/* deja defini dans libqbestfit
#ifndef COORDS_DEF_
#define COORDS_DEF_

typedef struct coords COORDS;
struct coords {
  float x,y,z;
};
#endif
*/


/*#define ADVANCED 1  */ /*options pour nous */
/*#define GSL 1 */ /*options pour la librairie GSL et calcul des angles de rotation */


#ifndef EPSILON
#define EPSILON 1.0e-6
#endif

#define MYFREE(x)    {if ((x)) {free(x);x=NULL;}}
#define BAD_POINTER   1L
#define BON 0
#define PASBON -1

#define NOT_AN_ANGLE 255
#define DIVANGLE	10
#define NOT_INT_ANGLE 	NOT_AN_ANGLE*DIVANGLE

#define DEFAULT_ANGLE_MAX 36 /*nb de groupes pour les angles*/

#define PROT_ALPHABET	"ABCDEFGHIJKLMNOPQRSTUVWXYZ"

#define ERR_SWAKUSA 1

#define DESC_LEN	96

		/* characters for path alignment matrix	*/
               /* used by swalign.c and stat.c    */
#define FORBI	'x'
#define SUBST	's'
#define INSER	'i'
#define DELET	'd'
#define ZERO	'n'


#define N_IDENT  1.0   /*identite nuc*/
#define N_SUBST   -0.9  /*substitution nuc*/


#define CHEMIN	0
#define CARRE	1
#define BANDE	2

#define STRUCTURE	0
#define PROTEIQUE	1
#define NUCLEIQUE	2

#define P_VAL 0.005   /*0.001*/            /* valeur minimale pour accepter le melange des sequences (mots de taille > 1) */
#define ERR_SHUFFLE       -100   /*valeur d'erreur quand mauvais shuffle*/
#define NBPASBON_MAX       10    /*pour les seq nucleiques : nombre maximum de lettre inconnues autorisees par sequence */
#define LGMOT_SHUFFLE_NUC      2

#define DIFF_GAP                1.3			/*difference pour calculer le gap*/


#define NB_MAX_RESIDUS 10000 /* Nb max of residues in input sequence/structure */

#define DEFAULT_NBEST    10		/*  defaut 10 */
#define DEFAULT_LENGTH    0		/*  defaut 0 */
#define DEFAULT_GAP_STRUCT     	200.0	/* the worst value for a angle score is	*/
					/* threshold - 180.0 i.e. 30-180=150	*/
					/* so default gap is set to 200		*/
#define DEFAULT_GAP_STRUCT_EXT   50.0					

#if 0
#define DEFAULT_GAP_STRUCT_EXT   50.0					
#define DEFAULT_GAP_STRUCT_BLOS     6	
#define DEFAULT_GAP_STRUCT_EXT_BLOS 3  	
#endif

#define DEFAULT_GAP_PROT     8.0	/*gap proteique 8.0 gap nucleique calcul auto??*/
#define DEFAULT_GAP_PROT_EXT    3.0  /*3.0*/		/*gap proteique 7 */
#define DEFAULT_GAP_NUC     	4.0		/* 4 gap nucleique */
#define DEFAULT_GAP_NUC_EXT     1.0		/* 1 gap nucleique */

/*valeurs par defaut*/
#ifdef ADVANCED
#define NB_SHUFFLE      1000    /*nombre de sequences aleatoires pour sequences nucleiques et proteiques*/
#else
#define NB_SHUFFLE      100    /*nombre de sequences aleatoires pour sequences nucleiques et proteiques*/
#endif

#define NB_SHUFFLE_ADN      20    /* WV2 nombre de sequences aleatoires pour sequences nucleiques et proteiques*/
#define NB_SHUFFLE_PROT      20    /* WV2 nombre de sequences aleatoires pour sequences nucleiques et proteiques*/
#define NB_REPEATS_ADN      50    /*nombre de repetitions pour les sequences aleatoires pour sequences nucleiques et proteiques*/
#define NB_REPEATS_PROT     50    /*nombre de repetitions pour les sequences aleatoires pour sequences nucleiques et proteiques*/

#define PROBA 0.99  /*proba pour limite scores sequences*/

#define DEFAULT_THRESHOLD	30.0
#define DEFAULT_RECOVER		0.0 /*avant 0.3*/
#define DEFAULT_MINSCORE_STRUCT	350.0 /* score minimum pour un sous-alignement 200 ou 350 pour structures */
#define DEFAULT_MINSCORE_STRUCT_BLOS	30.0 /* score minimum pour un sous-alignement 200 ou 350 pour structures */

#define DEFAULT_MINSCORE_PROT	30.0 /* score minimum pour un sous-alignement 30 pour prot */
#define DEFAULT_MINSCORE_NUC	15.0 /* score minimum pour un sous-alignement 15 pour nucleique */


#define DEFAULT_MINSN		0.0 /* score normalise minimum pour un sous-alignement */
#define DEFAULT_KEPT_STRUCTURES	50	/* number of structures in output	*/


/*structures ...*/
#define DEFAULT_CROSSEDRMS	10.0	/* default cross rms for compatible fragment	*/
#define DEFAULT_MINRMS		4.5  /* rms maximum pour un sous-alignement */
#define DEFAULT_RRMSD		0.5  /* rms normalise (skolnick) maximum pour un sous-alignement */

#define NB_SCORES_MAX     500000  /*si jamais sequence trop redondande, on garde les NB_SCORES_MAX 1ers scores*/

#define PDB_FORMAT	3
#define BIN_FORMAT	4
#define ASC_FORMAT	5


#define EXTENSION_GOK  ".set"
#define EXTENSION_PYMOL  ".pml"
#define EXTENSION_RASMOL  ".ras"
#define EXTENSION_WEBRASMOL  ".wras"

#define DEFAULT_DIFF_MAX 60 /*difference max acceptable entre 2 angles*/

#define AFFICHAGE_TRANSITIONS 0  /* affiche les sommes des transitions entre 2 angles*/
#define AFFICHAGE_TRANSITIONS2 0  /* autre affichage des sommes des transitions entre 2 angles*/
#define TOUTES_TRANSITIONS 0  /*affiche proba par paire en plus de proba simples*/
#define AFFICHAGE_SOMME_SCORES 0   /*affiche les sommes des scores, proba MTD, RMS ...*/
#define AFFICHAGE_TMP 0
#define CALCUL_MTD 0

/*fin structure*/

/*fonction*/
int egal (float a, float b);

/*Les differentes structures*/

typedef struct WV{  /* waterman vingron  */
    double gamma;
    double p;
    double m;
    double n;
    double lg_moy;
} WV;



typedef struct tyFrag {

  int debut; /*num 1er residu du fragment*/
  int fin; /*num dernier residu du fragment*/
              /*Donc lg de chaine=fin-debut+1: frag= [debut..fin]*/
  char name;
}tyFrag;

#if 0
typedef struct tyResult { /*structure definissant un SHSP*/

   int posQuery; /*indice du 1er res du mot dans le tableau des angles recodes de la chaine (commence a 0) */
   int decalage; /*indice ds 2eme seq ou 2 motif */
   int lgMot;
   double score;
} tyResult;

typedef struct tyChemin { /*structure definissant une suite de SHSP compatibles*/

  int lgChemin; /*Somme des lg des mots du chemin*/
  int nbMots;
  tyResult *tMots; /*Tableau contenant tous les meilleurs alignements */ 
                   /*Il faut que ces alignements soient classes selon l'indice dans la query*/

  double *tProbaIndep; /*tableau contenant les proba "indpendantes" d'apparitions des SHSP*/
  double *tProbaBi; /*tableau contenant les proba conditionnelles (2-uple) d'apparitions des SHSP*/
  double *tProbaMTD; /*tableau contenant les  proba MTD d'apparitions des SHSP*/
                     /*!!!!!!!!!!!!!!! Ce tableau contient les  proba et non plus les log*/

  int    *tAliCompatibles;   /* Tableau des "compatiblits spatiales" des alignements */
                             /* Chaque case contient le numro de l'alignement */

  double scoreTot; /*score cumule des alignements */
  double sumLogProbaIndep; /* log de la somme des probabilites de tous alignements 	*/
  double sumLogProbaMTD;         /*Somme des log des proba calculees par mtd*/
  double sumLogProbaBi;         /*Somme des log des proba conditionnelles (2-uple)*/

  double sumLogProbaIndep_Compatible; /* log de la somme des probabilites de tous SHSP compatibles de tyProt	*/
  double sumLogProbaMTD_Compatible;         /*Somme des log des proba SHSP compatibles calculees par mtd*/
  double sumLogProbaBi_Compatible;         /*Somme des log des proba SHSP compatibles conditionnelles (2-uple)*/

  double *tRMSD;    /*tableau des RMSD pour chaque alignement , */
  double *tRMSDCroises; /*si -R, tableau[nbMots,nbMots], cad RMSD du mot j avec la*/
                        /* matrice  de rotation du mot i*/
  double minRMSCroises;  /*Le plus petits des RMS croises, */
                         /*permet d'eviter certaines recherche de compatibilits spatiales*/
                         /*est initialis  PASBON (-1)*/
  double **tMatrixRT;  /*tableau des matrices de rotation et des 2 vecteurs de translation*/
                       /*pour aligner les 2 structures sur 1 SHSP*/
                       /*1 ptr sur tableau de taille 15, avec 3 cases par lignes */
                       /*(3*3) pour matrice de rotation de structure banque*/
                       /*(1*3) pour vecteur de translation de structure banque*/
                       /*(1*3) pour vecteur de translation de structure requte*/
 } tyChemin;

#endif

typedef struct tyProt {

  char pdbpath[PATH_MAX]; /*nom du fichier PDB (avec le path)*/
  char nomProt[PATH_MAX];        /*nom PDB de la proteine*/
  char desc[DESC_LEN]; /* description field of the protein */
  int nbFrag; /*Nombre de fragments*/
  int nbRes; /*nombre de residus de la proteine*/
  PdbNum *tNumPDB;	/* num des residus selon la		*/
			/* numerotation du fichier PDB		*/
                	/* Rq/pb: peut y avoir des lettres dans	*/
			/* ces numeros				*/
  int *tAnglesRecodes;	/* angles alpha recodes (toutes les	*/
			/* chaines sont toujours a la suite)	*/
  int *tAngles;		/* tableau contenant tous les angles	*/
			/* alpha de la proteine (toutes les	*/
			/* chaines sont a la suite) les angles	*/
			/* sont *10				*/
  COORDS *coords;	/* tableau contenant les coordonnees	*/
			/* des carbones alpha de la proteine	*/
  char *tNameRes;	/* noms des residus en code une lettre	*/
  tyFrag *tFrag;	/* indices dans tAngles du debut et de	*/
			/* la fin de chaque fragment de la	*/
			/* proteine				*/
#if 0
  tyChemin *pResult;	/* resultats et proba */
#endif
}tyProt;

typedef struct {
  int nmatch;			/* number of best SW alignments		*/
  int nbprint;			/*   nombre de resultats a afficher     */
  int realsize;		/* real size of arrays			*/
  int *posd0;			/* position of alignment in first structure	*/
  int *posd1;			/* position of alignment in second structure	*/
  int *posf0;			/* end position of alignment in first structure	*/
  int *posf1;			/* end position of alignment in second structure	*/
  double *tProbaMTD_p0;		/* MTD proba of the match/alignment	p0	*/
  double *tProbaMTD_p1;		/* MTD proba of the match/alignment	p1	*/
  float *recover;		/* recovering between matches/alignments	*/
  float *score;		/* alignment score (based upon angle differences	)*/
  double *score_GL;                  /*score Gerstein Levitt pour les structures */
  float *score_blosangle;		/* alignment score (score blosum angle) */
  double *tm_score;		/* score TM  */
  double *score_freq;		/* alignment score divided by angles'frequences	*/
  double *score_par_freq;		/* alignment score multiplied by angles'frequences	*/
  double *rms;			/* RMS of alignments or proba for sequence			*/
  float *rev_rms;		/* reverse RMS of alignments			*/
  tyProt *prot1;		/* Second protein in match	*/
  char **sali;			/* alignment strings (s,i,d)	*/
  float **crossrms;		/* cross RMS between SW alignments, non pertinent in automode	*/
  int *group;			/* array of group ID of spatially compatible alignments */
   				/* (begin at 0, -1 if no group)				*/
  int ngroup;			/* number of spatially compatible alignments	*/
  double *tProbaIndep; /*tableau contenant les proba "independantes" d'apparitions des SHSP*/
  double *tProbaBi; /*tableau contenant les proba conditionnelles (2-uple) d'apparitions des matchs*/
  DtMatrix4x4 *matrix4x; /*tableau contenant les matrices rotation translation des matchs*/
  double sumLogProbaIndep; /* log de la somme des probabilites de tous match de tyProt	*/
  double sumLogProbaBi;         /*Somme des log des proba conditionnelles (2-uple)*/
  double sumLogProbaMTD_p0;         /*Somme des log des proba */
  double sumLogProbaMTD_p1;         /*Somme des log des proba */
  float *alpha;   /*anlges de la matrice de rotation*/
  float *beta;
  float *gamma;
  double *new_rms; /*RMS normalise */
} SWResult;

typedef struct {
  char	  st[2][PATH_MAX];	/* file names of the two input files	*/
  int     format[2];		/* format PDBBIN, ASCII ALPHA...	*/
  int     nbest;		/* number of best alignment to keep	*/
  int     length;		/* minimum length for repeats	*/
  int     angleMax;		/* number of angle classes	*/
  int     verbose;
  int     vverbose;
  int 	  nbStructSet, nbStructpymol;
  int     nbkept;		/* number of structures to output in case of bank scanning	*/
  int     nb_shuffle;    /*nb sequences aleatoires totales (pour les stats de significativite )*/
  int     nb_repeats;    /*nb repetitions sur les sequences aleatoires totales (pour les stats de significativite )*/
  int     output;		/* short output*/
  int     elongation;
  float   threshold;		/* threshold on angle score (score= this constant minus angle diff) 	*/
  float   gap_o;         /* ouverture*/
  float   gap_e;            /* extention */
  float   minscore;		/* min score for output			*/
  float   proba;		/* min normalized score for output			*/
  float   minrms;		/* min rms for output			*/
  float   cross_rms;		/* limit for considering crossrms as spatially correct	*/
  float   rrmsd;		/* rrmsd normalise skolnick	*/
  int   which_rmsd;		/* use RMS instead of rrmsd normalise skolnick	*/
  float   recover;		/* threshold for recovering		*/
  int   sim;			/* 1 if Huang algorithm is be used	*/
  int   no_size_limit;		/* if 0, input seq size limit is NB_MAX_RESIDUES, else no limit */
  SWResult*  (*myalign)(tyProt *p0, tyProt *p1, int automode, int K,
                float minscore, float proba_limite, float minrms,
                int (*Igscore)(int a1, int a2, void *mat_thresh),
                void *mat_thresh, float gap_o, float gap_e,
                float tauxrecouv, float cross_rms_seuil, int vverbose,
                int SPN, WV *water_ving, int length, int elongation,
                float rrmsd, int which_rmsd); /* routine d'alignement */
} tyParametres;

